<?php
/**
 * BSF License manager class file.
 *
 * @package bsf-core
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'BSF_License_Manager' ) ) {

	/**
	 * Class BSF_License_Manager
	 */
	class BSF_License_Manager {

		/**
		 * Class instance.
		 *
		 * @access private
		 * @var $instance Class instance.
		 */
		private static $instance = null;

		/**
		 * Inline form products.
		 *
		 * @access private
		 * @var $inline_form_products Inline form products.
		 */
		private static $inline_form_products = array();

		/**
		 * Instance.
		 */
		public static function instance() {
			if ( ! isset( self::$instance ) ) {
				self::$instance = new self();
			}

			return self::$instance;
		}

		/**
		 * Initiator
		 */
		public function __construct() {
			add_action( 'admin_head', array( $this, 'admin_css' ), 100 );

			add_action( 'admin_init', array( $this, 'bsf_activate_license' ) );
			add_action( 'admin_init', array( $this, 'bsf_deactivate_license' ) );
			add_action( 'bsf_product_update_registered', array( $this, 'refresh_products_on_license_activae' ) );
			add_action( 'admin_footer', array( $this, 'render_popup_form_markup' ) );

			$this->includes();
			add_action( 'admin_enqueue_scripts', array( $this, 'load_scripts' ) );
		}
		/**
		 *  Refresh products on License activation.
		 */
		public function refresh_products_on_license_activae() {
			update_site_option( 'bsf_force_check_extensions', true );
		}
		/**
		 *  Includes.
		 */
		public function includes() {

			require_once BSF_UPDATER_PATH . '/class-bsf-envato-activate.php';
		}
		/**
		 *  Admin CSS.
		 */
		public function admin_css() {
			?>

			<style type="text/css">
				.bsf-pre {
					white-space: normal;
				}

				/* license consent */
				.bsf-license-consent-container {
					display: flex;
				}

				.bsf-license-consent-container label {
					padding-top: 0;
				}

				.gc-admin p.bsf-license-consent-container input {
					margin-top: 2px;
					margin-right: 10px;
				}
			</style>

			<?php
		}
		/**
		 *  BSF Deactivate license.
		 */
		public function bsf_deactivate_license() {

			if ( ! isset( $_POST['bsf_deactivate_license'] ) ) {
				return;
			}

			if ( ! isset( $_POST['bsf_graupi_nonce'] ) || ( isset( $_POST['bsf_graupi_nonce'] ) && ! gc_verify_nonce( $_POST['bsf_graupi_nonce'], 'bsf_license_activation_deactivation_nonce' ) ) ) {

				return;
			}

			if ( ! isset( $_POST['bsf_license_manager']['license_key'] ) || empty( $_POST['bsf_license_manager']['license_key'] ) ) {
				return;
			}

			if ( ! isset( $_POST['bsf_license_manager']['product_id'] ) || empty( $_POST['bsf_license_manager']['product_id'] ) ) {
				return;
			}

			$product_id  = esc_attr( $_POST['bsf_license_manager']['product_id'] );
			$license_key = $this->bsf_get_product_info( $product_id, 'purchase_key' );

			// Check if the key is from EDD.
			$is_edd = $this->is_edd( $license_key );

			$path = bsf_get_api_url() . '?referer=deactivate-' . $product_id;

			// Using Brainstorm API v2.
			$data = array(
				'action'       => 'bsf_deactivate_license',
				'purchase_key' => $license_key,
				'product_id'   => $product_id,
				'site_url'     => get_site_url(),
				'is_edd'       => $is_edd,
				'referer'      => 'customer',
			);

			$data     = apply_filters( 'bsf_deactivate_license_args', $data );
			$response = gc_remote_post(
				$path,
				array(
					'body'    => $data,
					'timeout' => '15',
				)
			);

			if ( ! is_gc_error( $response ) || gc_remote_retrieve_response_code( $response ) === 200 ) {
				$result = json_decode( gc_remote_retrieve_body( $response ), true );

				if ( isset( $result['success'] ) && ( true === $result['success'] || 'true' === $result['success'] ) ) {
					// update license saus to the product.
					$_POST['bsf_license_deactivation']['success'] = $result['success'];
					$_POST['bsf_license_deactivation']['message'] = $result['message'];
					unset( $result['success'] );
					unset( $result['message'] );

					$this->bsf_update_product_info( $product_id, $result );

					do_action( 'bsf_deactivate_license_' . $product_id . '_after_success', $result, $response, $_POST );

				} else {
					$_POST['bsf_license_deactivation']['success'] = $result['success'];
					$_POST['bsf_license_deactivation']['message'] = $result['message'];
				}
			} else {
				// If there is an error, the status will not be changed. hence it's true.
				$_POST['bsf_license_activation']['success'] = true;
				$_POST['bsf_license_activation']['message'] = 'There was an error when connecting to our license API - <pre class="bsf-pre">' . $response->get_error_message() . '</pre>';
			}

			// Delete license key status transient.
			delete_transient( $product_id . '_license_status' );
		}

		/**
		 *  BSF Activate lciense.
		 */
		public function bsf_activate_license() {

			if ( ! isset( $_POST['bsf_activate_license'] ) ) {
				return;
			}

			if ( ! isset( $_POST['bsf_graupi_nonce'] ) || ( isset( $_POST['bsf_graupi_nonce'] ) && ! gc_verify_nonce( $_POST['bsf_graupi_nonce'], 'bsf_license_activation_deactivation_nonce' ) ) ) {

				return;
			}

			if ( ! isset( $_POST['bsf_license_manager']['license_key'] ) || empty( $_POST['bsf_license_manager']['license_key'] ) ) {
				return;
			}

			if ( ! isset( $_POST['bsf_license_manager']['product_id'] ) || empty( $_POST['bsf_license_manager']['product_id'] ) ) {
				return;
			}

			$license_key              = esc_attr( $_POST['bsf_license_manager']['license_key'] );
			$product_id               = esc_attr( $_POST['bsf_license_manager']['product_id'] );
			$user_name                = isset( $_POST['bsf_license_manager']['user_name'] ) ? esc_attr( $_POST['bsf_license_manager']['user_name'] ) : '';
			$user_email               = isset( $_POST['bsf_license_manager']['user_email'] ) ? esc_attr( $_POST['bsf_license_manager']['user_email'] ) : '';
			$privacy_consent          = ( isset( $_POST['bsf_license_manager']['privacy_consent'] ) && 'true' === $_POST['bsf_license_manager']['privacy_consent'] ) ? true : false;
			$terms_conditions_consent = ( isset( $_POST['bsf_license_manager']['terms_conditions_consent'] ) && 'true' === $_POST['bsf_license_manager']['terms_conditions_consent'] ) ? true : false;

			// Check if the key is from EDD.
			$is_edd = $this->is_edd( $license_key );

			// Server side check if the license key is valid.
			$path = bsf_get_api_url() . '?referer=activate-' . $product_id;

			// Using Brainstorm API v2.
			$data = array(
				'action'                   => 'bsf_activate_license',
				'purchase_key'             => $license_key,
				'product_id'               => $product_id,
				'user_name'                => $user_name,
				'user_email'               => $user_email,
				'privacy_consent'          => $privacy_consent,
				'terms_conditions_consent' => $terms_conditions_consent,
				'site_url'                 => get_site_url(),
				'is_edd'                   => $is_edd,
				'referer'                  => 'customer',
			);

			$data     = apply_filters( 'bsf_activate_license_args', $data );
			$response = gc_remote_post(
				$path,
				array(
					'body'    => $data,
					'timeout' => '15',
				)
			);

			if ( ! is_gc_error( $response ) || gc_remote_retrieve_response_code( $response ) === 200 ) {
				$result = json_decode( gc_remote_retrieve_body( $response ), true );

				if ( isset( $result['success'] ) && ( true === $result['success'] || 'true' === $result['success'] ) ) {
					// update license saus to the product.
					$_POST['bsf_license_activation']['success'] = $result['success'];
					$_POST['bsf_license_activation']['message'] = $result['message'];
					unset( $result['success'] );

					// Update product key.
					$result['purchase_key'] = $license_key;

					$this->bsf_update_product_info( $product_id, $result );

					do_action( 'bsf_activate_license_' . $product_id . '_after_success', $result, $response, $_POST );

				} else {
					$_POST['bsf_license_activation']['success'] = $result['success'];
					$_POST['bsf_license_activation']['message'] = $result['message'];
				}
			} else {
				$_POST['bsf_license_activation']['success'] = false;
				$_POST['bsf_license_activation']['message'] = 'There was an error when connecting to our license API - <pre class="bsf-pre">' . $response->get_error_message() . '</pre>';
			}

			// Delete license key status transient.
			delete_transient( $product_id . '_license_status' );

		}

		/**
		 *  Is EDD.
		 *
		 * @param string $license_key License key.
		 */
		public function is_edd( $license_key ) {

			// Purchase key length for EDD is 32 characters.
			if ( strlen( $license_key ) === 32 ) {

				return true;
			}

			return false;
		}

		/**
		 *  BSF Update product Info.
		 *
		 * @param int   $product_id Product ID.
		 * @param array $args Arguments.
		 */
		public function bsf_update_product_info( $product_id, $args ) {
			$brainstrom_products = get_option( 'brainstrom_products', array() );

			foreach ( $brainstrom_products as $type => $products ) {

				foreach ( $products as $id => $product ) {

					if ( $id == $product_id ) { // phpcs:ignore GeChiUI.PHP.StrictComparisons.LooseComparison
						foreach ( $args as $key => $value ) {
							$brainstrom_products[ $type ][ $id ][ $key ] = $value;
							do_action( "bsf_product_update_{$value}", $product_id, $value );
						}
					}
				}
			}

			update_option( 'brainstrom_products', $brainstrom_products );
		}

		/**
		 *  BSF is active license.
		 *
		 * @param int $product_id Product ID.
		 */
		public static function bsf_is_active_license( $product_id ) {

			return get_pro_license_valid();

			// $brainstrom_products = get_option( 'brainstrom_products', array() );
			// $brainstorm_plugins  = isset( $brainstrom_products['plugins'] ) ? $brainstrom_products['plugins'] : array();
			// $brainstorm_themes   = isset( $brainstrom_products['themes'] ) ? $brainstrom_products['themes'] : array();

			// $all_products = $brainstorm_plugins + $brainstorm_themes;

			// // If a product is marked as free, it is considered as active.
			// $is_free = self::is_product_free( $product_id );
			// if ( 'true' === $is_free || true === $is_free ) {
			// 	return true;
			// }

			// //验证服务器是否购买了专业版
			// require_once ABSPATH . 'gc-admin/includes/plugin-install.php';
   //          $response = plugins_api(
   //              'plugin_license',
   //              array(
   //                  'slug' => $product_id,
   //              )
   //          );
   //          if( isset($response->license) ){
   //          	return true;
   //          }
   //          // By default Return false.
			// return false;

            //绑定Key核验
			// $is_bundled = BSF_Update_Manager::bsf_is_product_bundled( $product_id );

			// // The product is not bundled.
			// if ( isset( $all_products[ $product_id ] ) ) {

			// 	if ( isset( $all_products[ $product_id ]['status'] ) && 'registered' === $all_products[ $product_id ]['status'] ) {

			// 		// If the purchase key is empty, Return false.
			// 		if ( ! isset( $all_products[ $product_id ]['purchase_key'] ) ) {
			// 			return false;
			// 		}

			// 		// Check if license is active on API.
			// 		if ( false === self::instance()->get_remote_license_status( $all_products[ $product_id ]['purchase_key'], $product_id ) ) {
			// 			return false;
			// 		}

			// 		return true;
			// 	}
			// }

			// if ( ! empty( $is_bundled ) ) {

			// 	// The product is bundled.
			// 	foreach ( $is_bundled as $key => $value ) {

			// 		$product_id = $value;

			// 		if ( isset( $all_products[ $product_id ] ) ) {
			// 			if ( isset( $all_products[ $product_id ]['status'] ) && 'registered' === $all_products[ $product_id ]['status'] ) {
			// 				// If the purchase key is empty, Return false.
			// 				if ( ! isset( $all_products[ $product_id ]['purchase_key'] ) ) {
			// 					return false;
			// 				}

			// 				// Check if license is active on API.
			// 				if ( false === self::instance()->get_remote_license_status( $all_products[ $product_id ]['purchase_key'], $product_id ) ) {
			// 					return false;
			// 				}

			// 				return true;
			// 			}
			// 		}
			// 	}
			// }

			// // By default Return false.
			// return false;
		}
		/**
		 *  Get remote license status.
		 *
		 * @param string $purchase_key Purchase Key.
		 * @param int    $product_id Product ID.
		 */
		public function get_remote_license_status( $purchase_key, $product_id ) {
                  return true;
			$transient_key = $product_id . '_license_status';

			// Check if license status is cached.
			if ( false !== get_transient( $transient_key ) ) {
				return (bool) get_transient( $transient_key );
			}

			// Set default license to license status stored in the database.
			$license_status = $this->bsf_get_product_info( $product_id, 'status' );
			if ( 'registered' === $license_status ) {
				$license_status = '1';
			} else {
				$license_status = '0';
			}

			$path = bsf_get_api_url() . '?referer=license-status-' . $product_id;

			// Using Brainstorm API v2.
			$data = array(
				'action'       => 'bsf_license_status',
				'purchase_key' => $purchase_key,
				'site_url'     => get_site_url(),
			);

			$data     = apply_filters( 'bsf_license_status_args', $data );
			$response = gc_remote_post(
				$path,
				array(
					'body'    => $data,
					'timeout' => '10',
				)
			);

			// Try to make a second request to unsecure URL.
			if ( is_gc_error( $response ) && gc_remote_retrieve_response_code( $response ) !== 200 ) {
				$path     = bsf_get_api_url( true ) . '?referer=license-status-' . $product_id;
				$response = gc_remote_post(
					$path,
					array(
						'body'    => $data,
						'timeout' => '8',
					)
				);
			}

			if ( ! is_gc_error( $response ) && gc_remote_retrieve_response_code( $response ) === 200 ) {
				$response_body = json_decode( gc_remote_retrieve_body( $response ), true );

				// Check if status received from API is true.
				if ( isset( $response_body['status'] ) && true === $response_body['status'] ) {
					$license_status = '1';
				} else {
					$license_status = '0';
				}
			}

			// Save license status in transient which will expire in 6 hours.
			set_transient( $transient_key, $license_status, 6 * HOUR_IN_SECONDS );

			return (bool) $license_status;
		}
		/**
		 *  Is product free.
		 *
		 * @param int $product_id Product ID.
		 */
		public static function is_product_free( $product_id ) {
			$license_manager = self::instance();
			$is_free         = $license_manager->bsf_get_product_info( $product_id, 'is_product_free' );

			return $is_free;
		}
		/**
		 *  Get product info.
		 *
		 * @param int    $product_id Product ID.
		 * @param string $key Key.
		 */
		public function bsf_get_product_info( $product_id, $key ) {

			$brainstrom_products = get_option( 'brainstrom_products', array() );
			$brainstorm_plugins  = isset( $brainstrom_products['plugins'] ) ? $brainstrom_products['plugins'] : array();
			$brainstorm_themes   = isset( $brainstrom_products['themes'] ) ? $brainstrom_products['themes'] : array();

			$all_products = $brainstorm_plugins + $brainstorm_themes;

			if ( isset( $all_products[ $product_id ][ $key ] ) && ! empty( $all_products[ $product_id ][ $key ] ) ) {
				return $all_products[ $product_id ][ $key ];
			}
		}

		/**
		 * For Popup License form check `popup_license_form` is `true`.
		 *
		 * @param array $args Arguments.
		 */
		public function license_activation_form( $args ) {
            $product_id = ( isset( $args['product_id'] ) && ! is_null( $args['product_id'] ) ) ? $args['product_id'] : '';
            $product_name = $this->bsf_get_product_info( $product_id, 'product_name' );
            
			require_once ABSPATH . 'gc-admin/includes/plugin-install.php';
            $response = plugins_api(
                'plugin_license',
                array(
                    'slug' => $product_id,
                )
            );
            $html = '';
            if( isset($response->license) ){
                $html .= sprintf( '<p>授权有效期至'.$response->license['next_payment_date'].'</p>');
            }else{
                $html .= '<p>1. <strong><a href="https://www.gechiui.com/shop/pro/'.$product_id.'/" target="_blank">升级专业版。</a></strong></p>';
                $html .= sprintf( '<p>2. 前往%s，检查您的GeChiUI账号和AppKey账号配置是否正确。</p>', '<strong><a href="/gc-admin/plugin-install.php?tab=professionals">插件-专业版</a></strong>');
            }

			// Output the license activation/deactivation form.
			return apply_filters( "bsf_core_license_activation_form_{$product_id}", $html, $args );
		}


		/**
		 * Load Scripts
		 *
		 * @since 1.0.0
		 *
		 * @param  string $hook Current Hook.
		 * @return void
		 */
		public function load_scripts( $hook = '' ) {

			if ( 'plugins.php' === $hook ) {
				gc_register_script( 'bsf-core-jquery-history', bsf_core_url( '/assets/js/jquery-history.js' ), array( 'jquery' ), BSF_UPDATER_VERSION, true );
				gc_enqueue_style( 'bsf-core-license-form', bsf_core_url( '/assets/css/license-form-popup.css' ), array(), BSF_UPDATER_VERSION, 'all' );
				gc_enqueue_script( 'bsf-core-license-form', bsf_core_url( '/assets/js/license-form-popup.js' ), array( 'jquery', 'bsf-core-jquery-history' ), BSF_UPDATER_VERSION, true );
			}

		}
		/**
		 *  Get BSF inline license form.
		 *
		 * @param array  $links Links.
		 * @param array  $args Arguments.
		 * @param string $license_from_type license form type.
		 */
		public function get_bsf_inline_license_form( $links, $args, $license_from_type ) {

			$product_id = $args['product_id'];

			if ( ! isset( $product_id ) ) {
				return $links;
			}

			if ( is_multisite() && ! is_network_admin() && false === apply_filters( "bsf_core_popup_license_form_per_network_site_{$product_id}", false ) ) {
				return $links;
			}

			$status         = 'inactive';
			$license_string = __( '激活许可证', 'adaptive-sites' );
			if ( self::bsf_is_active_license( $product_id ) ) {
				$status         = 'active';
				$license_string = __( '授权', 'adaptive-sites' );
			}

			$product_id = $args['product_id'];

			// Render the license form only once on a page.
			if ( array_key_exists( $product_id, self::$inline_form_products ) ) {
				return $links;
			}

			$form_args = array(
				'product_id'                       => $product_id,
                'product_name'                       => $args['product_name'],
				'button_text_activate'             => esc_html__( '激活许可证', 'adaptive-sites' ),
				'button_text_deactivate'           => esc_html__( '禁用许可证', 'adaptive-sites' ),
				'license_form_title'               => '',
				'license_deactivate_status'        => esc_html__( '您的许可证未激活!', 'adaptive-sites' ),
				'license_activate_status'          => esc_html__( '您的许可证已激活!', 'adaptive-sites' ),
				'submit_button_class'              => 'bsf-product-license button-default',
				'form_class'                       => 'form-wrap bsf-license-register-' . esc_attr( $product_id ),
				'bsf_license_form_heading_class'   => 'bsf-license-heading',
				'bsf_license_active_class'         => 'success-message',
				'bsf_license_not_activate_message' => 'license-error',
				'size'                             => 'regular',
				'bsf_license_allow_email'          => false,
				'popup_license_form'               => ( isset( $args['popup_license_form'] ) ) ? $args['popup_license_form'] : false,
				'license_from_type'                => $license_from_type,
			);

			$form_args = gc_parse_args( $args, $form_args );

			self::$inline_form_products[ $product_id ] = $form_args;

			$action_links = array(
				'license' => '<a plugin-slug="' . esc_attr( $product_id ) . '" class="bsf-core-plugin-link bsf-core-license-form-btn ' . esc_attr( $status ) . '" aria-label="' . esc_attr( $license_string ) . '">' . esc_html( $license_string ) . '</a>',
			);

			return array_merge( $links, $action_links );
		}

		/**
		 * Render the markup for popup form.
		 */
		public function render_popup_form_markup() {

			$current_screen = get_current_screen();

			// Bail if not on plugins.php screen.
			if ( ! is_object( $current_screen ) && null === $current_screen ) {
				return;
			}

			if ( 'plugins' !== $current_screen->id && 'plugins-network' !== $current_screen->id ) {
				return;
			}

			foreach ( self::$inline_form_products as $product_id => $product ) {
				?>

				<div plugin-slug="<?php echo esc_attr( $product_id ); ?>" class="bsf-core-license-form" style="display: none;">
					<div class="bsf-core-license-form-overlay"></div>
					<div class="bsf-core-license-form-inner">
						<button type="button" class="bsf-core-license-form-close-btn">
							<span class="screen-reader-text"><?php esc_html_e( '关闭', 'adaptive-sites' ); ?></span>
							<span class="dashicons dashicons-no-alt"></span>
						</button>

						<?php
							$licence_form_method = isset( $_GET['license-form-method'] ) ? sanitize_text_field( $_GET['license-form-method'] ) : ''; // phpcs:ignore:GeChiUI.Security.NonceVerification.Recommended

							$allowed_html = array(
								'a'      => array(
									'href'   => array(),
									'title'  => array(),
									'target' => array(),
								),
								'br'     => array(),
								'em'     => array(),
								'strong' => array(),
								'div'    => array(
									'class' => array(),
									'id'    => array(),
								),
								'span'   => array(
									'class' => array(),
									'id'    => array(),
								),
								'input'  => array(
									'class'        => array(),
									'id'           => array(),
									'type'         => array(),
									'name'         => array(),
									'value'        => array(),
									'autocomplete' => array(),
								),
								'p'      => array(
									'class' => array(),
								),
								'h1'     => array(),
								'h2'     => array(),
								'h3'     => array(),
								'h4'     => array(),
								'h5'     => array(),
								'h6'     => array(),
								'i'      => array(),
								'form'   => array(
									'method' => array(),
									'action' => array(),
									'class'  => array(),
									'id'     => array(),
								),
							);

							if ( 'edd' === $product['license_from_type'] || 'license-key' === $licence_form_method ) {
								echo gc_kses( bsf_license_activation_form( $product ), $allowed_html );
							} elseif ( 'envato' === $product['license_from_type'] || 'oauth' === $licence_form_method ) {
								echo gc_kses( bsf_envato_register( $product ), $allowed_html );
							}

							do_action( "bsf_inlne_license_form_footer_{$product[ 'license_from_type' ]}", $product_id );

							do_action( 'bsf_inlne_license_form_footer', $product_id );

							// Avoid rendering the markup twice as admin_footer can be called multiple times.
							unset( self::$inline_form_products[ $product_id ] );
							?>
					</div>
				</div>

				<?php
			}

		}


	} // Class BSF_License_Manager

	new BSF_License_Manager();
}

/**
 * BSF License activation form.
 *
 * @param array $args Arguments.
 */
function bsf_license_activation_form( $args ) {
	$license_manager = BSF_License_Manager::instance();

	return $license_manager->license_activation_form( $args );
}

/**
 *  Get BSF inline license form.
 *
 * @param array  $links Links.
 * @param array  $bsf_product_id BSF Product ID.
 * @param string $license_from_type license form type.
 */
function get_bsf_inline_license_form( $links, $bsf_product_id, $license_from_type ) {
	$license_manager = BSF_License_Manager::instance();

	return $license_manager->get_bsf_inline_license_form( $links, $bsf_product_id, $license_from_type );
}