/**
 * AJAX Request Queue
 *
 * - add()
 * - remove()
 * - run()
 * - stop()
 *
 * @since 1.0.0
 */
var AdaptiveSitesAjaxQueue = (function () {

	var requests = [];

	return {

		/**
		 * Add AJAX request
		 *
		 * @since 1.0.0
		 */
		add: function (opt) {
			requests.push(opt);
		},

		/**
		 * Remove AJAX request
		 *
		 * @since 1.0.0
		 */
		remove: function (opt) {
			if (jQuery.inArray(opt, requests) > -1) {
				requests.splice($.inArray(opt, requests), 1);
			}
		},

		/**
		 * Run / Process AJAX request
		 *
		 * @since 1.0.0
		 */
		run: function () {
			var self = this,
				oriSuc;


			if (requests.length) {
				oriSuc = requests[0].complete;

				requests[0].complete = function () {
					if (typeof (oriSuc) === 'function') oriSuc();
					requests.shift();
					self.run.apply(self, []);
				};

				jQuery.ajax(requests[0]);
			} else {
				self.tid = setTimeout(function () {
					self.run.apply(self, []);
				}, 1000);
			}

		},

		/**
		 * Stop AJAX request
		 *
		 * @since 1.0.0
		 */
		stop: function () {
			requests = [];
			clearTimeout(this.tid);
		},

		/**
		 * Debugging.
		 *
		 * @param  {mixed} data Mixed data.
		 */
		_log: function (data, level) {
			var date = new Date();
			var time = date.toLocaleTimeString();

			var color = '#444';

			if (typeof data == 'object') {
				console.log(data);
			} else {
				console.log(data + ' ' + time);
			}
		},
	};

}());

(function ($) {

	/** Checking the element is in viewport? */
	$.fn.isInViewport = function () {

		// If not have the element then return false!
		if (!$(this).length) {
			return false;
		}

		var elementTop = $(this).offset().top;
		var elementBottom = elementTop + $(this).outerHeight();

		var viewportTop = $(window).scrollTop();
		var viewportBottom = viewportTop + $(window).height();

		return elementBottom > viewportTop && elementTop < viewportBottom;
	};

	var AdaptiveSSEImport = {
		complete: {
			posts: 0,
			media: 0,
			users: 0,
			comments: 0,
			terms: 0,
		},

		updateDelta: function (type, delta) {
			this.complete[type] += delta;

			var self = this;
			requestAnimationFrame(function () {
				self.render();
			});
		},
		updateProgress: function (type, complete, total) {
			var text = complete + '/' + total;

			if ('undefined' !== type && 'undefined' !== text) {
				total = parseInt(total, 10);
				if (0 === total || isNaN(total)) {
					total = 1;
				}

				var percent = parseInt(complete, 10) / total;
				var progress = Math.round(percent * 100) + '%';
				var progress_bar = percent * 100;

				if (progress_bar <= 100) {
					var process_bars = document.getElementsByClassName('adaptive-site-import-process');
					for (var i = 0; i < process_bars.length; i++) {
						process_bars[i].value = progress_bar;
					}
					AdaptiveSitesAdmin._log_title('正在导入内容.. ' + progress, false, false);
				}
			}
		},
		render: function () {
			var types = Object.keys(this.complete);
			var complete = 0;
			var total = 0;

			for (var i = types.length - 1; i >= 0; i--) {
				var type = types[i];
				this.updateProgress(type, this.complete[type], this.data.count[type]);

				complete += this.complete[type];
				total += this.data.count[type];
			}

			this.updateProgress('total', complete, total);
		}
	};

	AdaptiveSitesAdmin = {

		default_cta_link: adaptiveSitesVars.cta_link,
		quick_corner_cta_link: adaptiveSitesVars.cta_quick_corner_link,
		premium_popup_cta_link: adaptiveSitesVars.cta_premium_popup_link,
		import_source: 'legacy',
		gccontent_left_margin: $('#gccontent').css('margin-left'),
		header: $('#adaptive-sites-menu-page .nav-tab-wrapper'),
		header_offset: 0,
		header_gutter: null,
		header_stick_after: null,

		subscribe_status: false,
		subscribe_skiped: false,
		site_import_status: false,
		page_import_status: false,
		imported_page_data: null,
		first_import_complete: adaptiveSitesVars.first_import_complete,
		remaining_activate_plugins: [],
		required_plugins_original_list: [],
		subscription_form_submitted: adaptiveSitesVars.subscribed,

		compatibilities: [],

		skip_and_import_popups: [],
		required_plugins: [],

		_ref: null,

		/**
		 * _api_params = {
		 * 		'search'                  : '',
		 * 		'per_page'                : '',
		 * 		'adaptive-site-category'     : '',
		 * 		'adaptive-site-page-builder' : '',
		 * 		'page'                    : '',
		 *   };
		 *
		 * E.g. per_page=<page-id>&adaptive-site-category=<category-ids>&adaptive-site-page-builder=<page-builder-ids>&page=<page>
		 */
		_api_params: {},
		_breakpoint: 768,
		_has_default_page_builder: false,
		_first_time_loaded: true,

		visited_sites_and_pages: [],

		reset_remaining_posts: 0,
		reset_remaining_gc_forms: 0,
		reset_remaining_terms: 0,
		reset_processed_posts: 0,
		reset_processed_gc_forms: 0,
		reset_processed_terms: 0,
		site_imported_data: null,

		backup_taken: false,
		filter_array: [],
		autocompleteTags: [],
		templateData: {},
		mouseLocation: false,
		log_file: '',
		customizer_data: '',
		wxr_url: '',
		gcforms_url: '',
		cartflows_url: '',
		options_data: '',
		widgets_data: '',
		enabled_extensions: '',
		action_slug: '',
		import_start_time: '',
		import_end_time: '',
		search_terms: [],
		search_terms_with_count: [],
		page_settings_flag: true,
		delay_in_request: false,
		delay_value : 10000, // 10 seconds.

		init: function () {
			this._show_default_page_builder_sites();
			this._bind();
			this._addAutocomplete();
			this._autocomplete();
			this._load_large_images();
			this._prepare_markup();
		},

		_prepare_markup: function () {
			var GCAdminbarOuterHeight = parseFloat($('#gcadminbar').outerHeight());
			var HeaderOuterHeight = parseFloat(AdaptiveSitesAdmin.header.outerHeight());
			AdaptiveSitesAdmin.header
				.wrap('<div></div>')
				.parent().css('min-height', HeaderOuterHeight);

			$('.single-site-footer').css('margin-left', AdaptiveSitesAdmin.gccontent_left_margin);
			$('.single-site-pages-wrap').css('margin-right', AdaptiveSitesAdmin.gccontent_left_margin);

			AdaptiveSitesAdmin.header_gutter = GCAdminbarOuterHeight;
			AdaptiveSitesAdmin.header_offset = GCAdminbarOuterHeight + HeaderOuterHeight;
			AdaptiveSitesAdmin.header_stick_after = GCAdminbarOuterHeight - HeaderOuterHeight;
			AdaptiveSitesAdmin._stick_header();
		},

		/**
		 * load large image
		 *
		 * @return {[type]} [description]
		 */
		_load_large_image: function (el) {
			if (el.hasClass('loaded')) {
				return;
			}

			if (el.parents('.adaptive-theme').isInViewport()) {
				var large_img_url = el.data('src') || '';
				var imgLarge = new Image();
				imgLarge.src = large_img_url;
				imgLarge.onload = function () {
					el.removeClass('loading');
					el.addClass('loaded');
					el.css('background-image', 'url(\'' + imgLarge.src + '\'');
				};
			}
		},

		_load_large_images: function () {
			$('.theme-screenshot').each(function (key, el) {
				AdaptiveSitesAdmin._load_large_image($(el));
			});
		},

		_addAutocomplete: function () {

			var tags = adaptiveSitesVars.api_sites_and_pages_tags || [];
			var sites = adaptiveSitesVars.default_page_builder_sites || [];
			var strings = [];

			for (tag_index in tags) {
				strings.push(AdaptiveSitesAdmin._unescape_lower(tags[tag_index]['name']));
			}

			// Add site title's in autocomplete.
			for (site_id in sites) {

				if (adaptiveSitesVars.default_page_builder === sites[site_id]['adaptive-site-page-builder']) {
					var title = AdaptiveSitesAdmin._unescape(sites[site_id]['title']);

					// @todo check why below character not escape with function _.unescape();
					title = title.toLowerCase().replace('&#8211;', '-');

					strings.push(title);
				}
			}

			AdaptiveSitesAdmin.autocompleteTags = strings;
		},

		_autocomplete: function () {

			var strings = AdaptiveSitesAdmin.autocompleteTags;
			strings = _.uniq(strings);
			strings = _.sortBy(strings);

			$("#gc-filter-search-input").autocomplete({
				appendTo: ".adaptive-sites-autocomplete-result",
				classes: {
					"ui-autocomplete": "adaptive-sites-auto-suggest"
				},
				source: function (request, response) {
					var results = $.ui.autocomplete.filter(strings, request.term);

					// Show only 10 results.
					response(results.slice(0, 15));
				},
				open: function (event, ui) {
					$('.search-form').addClass('searching');
				},
				close: function (event, ui) {
					$('.search-form').removeClass('searching');
				}
			});

			$("#gc-filter-search-input").focus();
		},

		/**
		 * Debugging.
		 *
		 * @param  {mixed} data Mixed data.
		 */
		_log: function (data, level) {
			var date = new Date();
			var time = date.toLocaleTimeString();

			var color = '#444';

			switch (level) {
				case 'emergency': 	// color = '#f44336';
				case 'critical': 	// color = '#f44336';
				case 'alert': 		// color = '#f44336';
				case 'error': 		// color = '#f44336';
					if (typeof data == 'object') {
						console.error(data);
					} else {
						console.error(data + ' ' + time);
					}
					break;
				case 'warning': 	// color = '#ffc107';
				case 'notice': 		// color = '#ffc107';
					if (typeof data == 'object') {
						console.warn(data);
					} else {
						console.warn(data + ' ' + time);
					}
					break;
				default:
					if (typeof data == 'object') {
						console.log(data);
					} else {
						console.log(data + ' ' + time);
					}
					break;
				// case 'info': color = '#03a9f4';
				// break;
				// case 'debug': color = '#ffc107';
			}
		},

		_log_title: function (data, append) {

			var markup = '<p>' + data + '</p>';
			if (typeof data == 'object') {
				var markup = '<p>' + JSON.stringify(data) + '</p>';
			}

			var selector = $('.ast-importing-wrap');
			if ($('.current-importing-status-title').length) {
				selector = $('.current-importing-status-title');
			}

			if (append) {
				selector.append(markup);
			} else {
				selector.html(markup);
			}

		},

		/**
		 * Binds events for the Adaptive Sites.
		 *
		 * @since 1.0.0
		 * @access private
		 * @method _bind
		 */
		_bind: function () {
			$(window).on('resize scroll', AdaptiveSitesAdmin._load_large_images);

			$('.adaptive-sites__category-filter-anchor, .adaptive-sites__category-filter-items').hover(function () {
				AdaptiveSitesAdmin.mouseLocation = true;
			}, function () {
				AdaptiveSitesAdmin.mouseLocation = false;
			});

			$("body").mouseup(function () {
				if (!AdaptiveSitesAdmin.mouseLocation) AdaptiveSitesAdmin._closeFilter();
			});

			// Change page builder.
			$(document).on('click', '.nav-tab-wrapper .page-builders li', AdaptiveSitesAdmin._ajax_change_page_builder);
			$(document).on('click', '#adaptive-sites-welcome-form .page-builders li', AdaptiveSitesAdmin._change_page_builder);

			// Open & Close Popup.
			$(document).on('click', '.site-import-cancel, .adaptive-sites-result-preview .close, .adaptive-sites-popup .close', AdaptiveSitesAdmin._close_popup);
			$(document).on('click', '.adaptive-sites-popup .overlay, .adaptive-sites-result-preview .overlay', AdaptiveSitesAdmin._close_popup_by_overlay);

			$(document).on('click', '.ast-sites__filter-wrap-checkbox, .ast-sites__filter-wrap', AdaptiveSitesAdmin._filterClick);

			// Page.
			$(document).on('click', '.site-import-layout-button', AdaptiveSitesAdmin.show_page_popup_from_sites);
			$(document).on('click', '#adaptive-sites .adaptive-sites-previewing-page .theme-screenshot, #adaptive-sites .adaptive-sites-previewing-page .theme-name', AdaptiveSitesAdmin.show_page_popup_from_search);
			$(document).on('click', '.adaptive-sites-page-import-popup .site-install-site-button, .preview-page-from-search-result .site-install-site-button', AdaptiveSitesAdmin.import_page_process);
			$(document).on('adaptive-sites-after-site-pages-required-plugins', AdaptiveSitesAdmin._page_api_call);

			// Site reset warning.
			$(document).on('click', '.adaptive-sites-reset-data .checkbox', AdaptiveSitesAdmin._toggle_reset_notice);

			// Theme Activation warning.
			$(document).on('click', '.adaptive-sites-theme-activation .checkbox', AdaptiveSitesAdmin._toggle_theme_notice);

			// $(document).on('gc-theme-install-success', AdaptiveSitesAdmin._activateTheme);

			// Site.
			$(document).on('click', '.site-import-site-button', AdaptiveSitesAdmin._show_site_popup);
			$(document).on('click', '.adaptive-sites-get-agency-bundle-button', AdaptiveSitesAdmin._show_get_agency_bundle_notice);
			$(document).on('click', '.adaptive-sites-activate-license-button', AdaptiveSitesAdmin._show_activate_license_notice);
			$(document).on('click', '.adaptive-sites-invalid-mini-agency-license-button', AdaptiveSitesAdmin._show_invalid_mini_agency_license);
			$(document).on('click', '.adaptive-sites-site-import-popup .site-install-site-button', AdaptiveSitesAdmin._resetData);

			// Skip.
			$(document).on('click', '.adaptive-sites-skip-and-import-step', AdaptiveSitesAdmin._remove_skip_and_import_popup);

			// Skip & Import.
			$(document).on('adaptive-sites-after-adaptive-sites-required-plugins', AdaptiveSitesAdmin._start_site_import);

			$(document).on('adaptive-sites-reset-data', AdaptiveSitesAdmin._backup_before_rest_options);
			$(document).on('adaptive-sites-backup-settings-before-reset-done', AdaptiveSitesAdmin._reset_customizer_data);
			$(document).on('adaptive-sites-reset-customizer-data-done', AdaptiveSitesAdmin._reset_site_options);
			$(document).on('adaptive-sites-reset-site-options-done', AdaptiveSitesAdmin._reset_widgets_data);
			$(document).on('adaptive-sites-reset-widgets-data-done', AdaptiveSitesAdmin._reset_terms);
			$(document).on('adaptive-sites-delete-terms-done', AdaptiveSitesAdmin._reset_gc_forms);
			$(document).on('adaptive-sites-delete-gc-forms-done', AdaptiveSitesAdmin._reset_posts);

			$(document).on('adaptive-sites-reset-data-done', AdaptiveSitesAdmin._recheck_backup_options);
			$(document).on('adaptive-sites-backup-settings-done', AdaptiveSitesAdmin._startImportCartFlows);
			$(document).on('adaptive-sites-import-cartflows-done', AdaptiveSitesAdmin._startImportWPForms);
			$(document).on('adaptive-sites-import-gcforms-done', AdaptiveSitesAdmin._importCustomizerSettings);
			$(document).on('adaptive-sites-import-customizer-settings-done', AdaptiveSitesAdmin._importXML);
			$(document).on('adaptive-sites-import-xml-done', AdaptiveSitesAdmin.import_siteOptions);
			$(document).on('adaptive-sites-import-options-done', AdaptiveSitesAdmin._importWidgets);
			$(document).on('adaptive-sites-import-widgets-done', AdaptiveSitesAdmin._importEnd);

			// Try again.
			$(document).on('click', '.ast-try-again', AdaptiveSitesAdmin.tryAgain );

			$(document).on('click', '.adaptive-sites__category-filter-anchor', AdaptiveSitesAdmin._toggleFilter);

			// Tooltip.
			$(document).on('click', '.adaptive-sites-tooltip-icon', AdaptiveSitesAdmin._toggle_tooltip);

			// Plugin install & activate.
			$(document).on('gc-plugin-installing', AdaptiveSitesAdmin._pluginInstalling);
			$(document).on('gc-plugin-install-error', AdaptiveSitesAdmin._installPluginError);
			$(document).on('gc-plugin-install-success', AdaptiveSitesAdmin._installPluginSuccess);

			// Theme install & activate.
			$(document).on('gc-theme-installing', AdaptiveSitesAdmin._themeInstalling);
			$(document).on('gc-theme-install-error', AdaptiveSitesAdmin._installThemeError);
			$(document).on('gc-theme-install-success', AdaptiveSitesAdmin._installThemeSuccess);

			$(document).on('click', '#adaptive-sites .adaptive-sites-previewing-site .theme-screenshot, #adaptive-sites .adaptive-sites-previewing-site .theme-name', AdaptiveSitesAdmin._show_pages);
			$(document).on('click', '#single-pages .site-single', AdaptiveSitesAdmin._change_site_preview_screenshot);
			$(document).on('click', '.adaptive-sites-show-favorite-button', AdaptiveSitesAdmin._show_favorite);

			$(document).on('click', '.favorite-action-wrap', AdaptiveSitesAdmin._toggle_favorite);
			$(document).on('click', '.adaptive-previewing-single-pages .back-to-layout', AdaptiveSitesAdmin._go_back);
			$(document).on('click', '.adaptive-sites-showing-favorites .back-to-layout, .adaptive-sites-no-search-result .back-to-layout, .logo, .adaptive-sites-back', AdaptiveSitesAdmin._show_sites);

			$(document).on('keydown', AdaptiveSitesAdmin._next_and_previous_sites);

			$(document).on('click', '.adaptive-site-category a', AdaptiveSitesAdmin._filterSites);

			$(document).on('click', '.adaptive-sites-sync-library-button', AdaptiveSitesAdmin._sync_library);
			$(document).on('click', '.adaptive-sites-sync-library-message .notice-dismiss', AdaptiveSitesAdmin._sync_library_complete);
			$(document).on('click', '.page-builder-icon', AdaptiveSitesAdmin._toggle_page_builder_list);
			$(document).on('click', '.showing-page-builders #gcbody-content', AdaptiveSitesAdmin._close_page_builder_list);
			$(document).on('keyup input', '#gc-filter-search-input', AdaptiveSitesAdmin._search);
			$(document).on('keyup', '#gc-filter-search-input', _.debounce(AdaptiveSitesAdmin._searchPost, 1500));
			$(document).on('heartbeat-send', AdaptiveSitesAdmin._sendHeartbeat);
			$(document).on('heartbeat-tick', AdaptiveSitesAdmin._heartbeatDone);
			$(document).on('click', '.ui-autocomplete .ui-menu-item', AdaptiveSitesAdmin._show_search_term);

			$(document).on('click', '.button-subscription-submit', AdaptiveSitesAdmin._subscribe);
			$(document).on('click', '.button-subscription-skip', AdaptiveSitesAdmin._hide_subscription_popup);
			$(document).on('focusout change', '.subscription-input', AdaptiveSitesAdmin.validate_single_field);
			$(document).on('click input', '.subscription-input', AdaptiveSitesAdmin._animate_fields);
			$(document).on('click', '.adaptive-sites-advanced-options-heading', AdaptiveSitesAdmin.toggle_advanced);

			$(window).on('scroll', AdaptiveSitesAdmin._stick_header);
			$(document).on('gc-collapse-menu', AdaptiveSitesAdmin._manage_gc_collapse_menu);
			$(document).on('adaptive-sites-added-pages', AdaptiveSitesAdmin._stick_header);
			$(document).on('adaptive-sites-added-pages', AdaptiveSitesAdmin._manage_gc_collapse_menu);

		},

		/**
		 * Try again for import
		 * @param {*} event
		 */
		tryAgain: function( event ) {
			event.preventDefault();
			AdaptiveSitesAdmin.delay_in_request = true;
			$( '.site-import-site-button' ).trigger( 'click' );
		},

		/**
		 * Stick Header
		 */
		_stick_header: function () {
			if ($(window).outerWidth() > 768 && $(window).scrollTop() > AdaptiveSitesAdmin.header_stick_after) {
				AdaptiveSitesAdmin.header.addClass('stick').stop().css({
					'top': AdaptiveSitesAdmin.header_gutter,
					'margin-left': AdaptiveSitesAdmin.gccontent_left_margin,
				});
			} else {
				AdaptiveSitesAdmin.header.removeClass('stick').stop().css({
					'top': '',
				});
			}

		},

		/**
		 * Manage GC COllapse Menu
		 */
		_manage_gc_collapse_menu: function (event, state) {

			AdaptiveSitesAdmin.gccontent_left_margin = $('#gccontent').css('margin-left');

			if (AdaptiveSitesAdmin.header.hasClass('stick')) {
				AdaptiveSitesAdmin.header.css('margin-left', AdaptiveSitesAdmin.gccontent_left_margin);
			}

			$('.single-site-footer').css('margin-left', AdaptiveSitesAdmin.gccontent_left_margin);
			$('.single-site-pages-wrap').css('margin-right', AdaptiveSitesAdmin.gccontent_left_margin);

		},

		toggle_advanced: function (event) {
			const elScope = $('.adaptive-sites-advanced-options-heading span')
			if (elScope.hasClass('dashicons-arrow-right-alt2')) {
				elScope.removeClass('dashicons-arrow-right-alt2').addClass('dashicons-arrow-down-alt2');
			} else {
				elScope.removeClass('dashicons-arrow-down-alt2').addClass('dashicons-arrow-right-alt2');
			}
			$('.adaptive-sites-advanced-options').toggle();
		},

		_animate_fields: function (event) {
			event.preventDefault();
			event.stopPropagation();
			var parentWrapper = $(this).parents('.subscription-field-wrap');
			parentWrapper.addClass('subscription-anim');
		},

		validate_single_field: function (event) {
			event.preventDefault();
			event.stopPropagation();
			AdaptiveSitesAdmin._validate_field(event.target);
		},

		_validate_field: function (target) {

			var field = $(target);
			var fieldValue = field.val() || '';
			var parentWrapper = $(target).parents('.subscription-field-wrap');
			var fieldStatus = fieldValue.length ? true : false;

			if ((field.hasClass('subscription-input-email') && false === AdaptiveSitesAdmin.isValidEmail(fieldValue))) {
				fieldStatus = false;
			}

			if (fieldStatus) {
				parentWrapper
					.removeClass('subscription-error')
					.addClass('subscription-success');

			} else {
				parentWrapper
					.removeClass('subscription-success subscription-anim')
					.addClass('subscription-error');

				if (field.hasClass('subscription-input-email') && fieldValue.length) {
					parentWrapper
						.addClass('subscription-anim')
				}
			}

		},

		_hide_subscription_popup: function (event) {
			$('.subscription-popup').hide();
			$('.adaptive-sites-result-preview .default').show();

			AdaptiveSitesAdmin.subscribe_status = true;
			AdaptiveSitesAdmin.subscribe_skiped = true;

			if ('adaptive-sites' === AdaptiveSitesAdmin.action_slug) {
				$('.ast-importing-wrap').show();
				$('.adaptive-sites-result-preview').removeClass('adaptive-sites-subscription-popup');
				$('.ast-actioms-wrap .button').hide();
				$('.ast-actioms-wrap .site-install-site-button').show();
				$('.ast-actioms-wrap .site-import-cancel').show();

				$('.adaptive-sites-result-preview .heading h3').html(adaptiveSitesVars.headings.site_import);

				if (true === AdaptiveSitesAdmin.site_import_status) {
					AdaptiveSitesAdmin.import_complete();
				}
			} else {
				$('.adaptive-sites-result-preview .heading h3').html(adaptiveSitesVars.headings.page_import);

				if (true === AdaptiveSitesAdmin.page_import_status) {
					AdaptiveSitesAdmin.page_import_complete();
				}
			}
			if (event && event.target.classList.value == 'button-subscription-skip') {
				adaptiveSitesVars.subscribed = '';
				AdaptiveSitesAdmin.subscription_form_submitted = '';
			} else {
				adaptiveSitesVars.subscribed = 'yes';
				AdaptiveSitesAdmin.subscription_form_submitted = 'yes';
			}
		},

		_subscribe: function (event) {
			event.preventDefault();

			var submit_button = $(this);

			if (submit_button.hasClass('submitting')) {
				return;
			}

			var first_name_field = $('.subscription-input-name[name="first_name"]');
			var email_field = $('.subscription-input-email[name="email"]');
			var user_type_field = $('.subscription-input-gc-user-type[name="gc_user_type"]');
			var build_for_field = $('.subscription-input-build-website-for[name="build_website_for"]');

			var subscription_first_name = first_name_field.val() || '';
			var subscription_email = email_field.val() || '';
			var subscription_user_type = user_type_field.val() || '';
			var subscription_build_for = build_for_field.val() || '';

			AdaptiveSitesAdmin._validate_field(first_name_field);
			AdaptiveSitesAdmin._validate_field(email_field);
			if ($('#adaptive-sites-subscription-form-two').length) {
				AdaptiveSitesAdmin._validate_field(user_type_field);
				AdaptiveSitesAdmin._validate_field(build_for_field);
			}

			if ($('.subscription-field-wrap').hasClass('subscription-error')) {
				return;
			}

			submit_button.addClass('submitting');

			var subscription_fields = {
				EMAIL: subscription_email,
				FIRSTNAME: subscription_first_name,
				PAGE_BUILDER: adaptiveSitesVars.default_page_builder_data.name,
				GC_USER_TYPE: subscription_user_type,
				BUILD_WEBSITE_FOR: subscription_build_for,
			};

			$.ajax({
				url: adaptiveSitesVars.ajaxurl,
				type: 'POST',
				data: {
					action: 'adaptive-sites-update-subscription',
					_ajax_nonce: adaptiveSitesVars._ajax_nonce,
					data: JSON.stringify(subscription_fields),
				},
				beforeSend: function () {
					console.groupCollapsed('Email Subscription');
				},
			})
				.done(function (response) {
					AdaptiveSitesAdmin._log(response);

					submit_button
						.text(adaptiveSitesVars.subscriptionSuccessMessage)
						.removeClass('submitting')
						.addClass('submitted')
						.find('.dashicons')
						.removeClass('dashicons-update')
						.addClass('dashicons-yes')
					setTimeout(function () {
						AdaptiveSitesAdmin._hide_subscription_popup();
					}, 5000);
					console.groupEnd();
				});

		},

		isValidEmail: function (eMail) {
			if (/^[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+@[a-zA-Z0-9-]+(?:\.[a-zA-Z0-9-]+)*$/.test(eMail)) {
				return true;
			}

			return false;
		},

		_heartbeatDone: function (e, data) {
			// Check for our data, and use it.
			if (!data['ast-sites-search-terms']) {
				return;
			}
			AdaptiveSitesAdmin.search_terms = [];
			AdaptiveSitesAdmin.search_terms_with_count = [];
		},

		_sendHeartbeat: function (e, data) {
			// Add additional data to Heartbeat data.
			if (AdaptiveSitesAdmin.search_terms.length > 0) {
				data['ast-sites-search-terms'] = AdaptiveSitesAdmin.search_terms_with_count;
			}
		},

		_searchPost: function (e) {
			var term = $(this).val().toLowerCase();
			// Skip blank words and words smaller than 3 characters.
			if ('' === term || term.length < 3 ) {
				return;
			}

			if (!AdaptiveSitesAdmin.search_terms.includes(term)) {
				let count = $( '#adaptive-sites .ast-sites__search-wrap > div' ).length;
				AdaptiveSitesAdmin.search_terms.push(term);
				AdaptiveSitesAdmin.search_terms_with_count.push({'term': term, 'count': count });
			}
		},

		_toggleFilter: function (e) {

			var items = $('.adaptive-sites__category-filter-items');

			if (items.hasClass('visible')) {
				items.removeClass('visible');
				items.hide();
			} else {
				items.addClass('visible');
				items.show();
			}
		},

		_closeFilter: function (e) {

			var items = $('.adaptive-sites__category-filter-items');
			items.removeClass('visible');
			items.hide();
		},

		_filterClick: function (e) {

			AdaptiveSitesAdmin.filter_array = [];

			if ($(this).hasClass('ast-sites__filter-wrap')) {
				$('.adaptive-sites__category-filter-anchor').attr('data-slug', $(this).data('slug'));
				$('.adaptive-sites__category-filter-items').find('.ast-sites__filter-wrap').removeClass('category-active');
				$(this).addClass('category-active');
				$('.adaptive-sites__category-filter-anchor').text($(this).text());
				$('.adaptive-sites__category-filter-anchor').trigger('click');
				$('#gc-filter-search-input').val('');

				if ($('.adaptive-sites-show-favorite-button').hasClass('active')) {
					$('.adaptive-sites-show-favorite-button').removeClass('active');
					$('body').removeClass('adaptive-sites-showing-favorites');
					AdaptiveSitesAdmin._clean_url_params('favorites');
				}
			}

			var $filter_name = $('.adaptive-sites__category-filter-anchor').attr('data-slug');

			if ('' != $filter_name) {
				AdaptiveSitesAdmin.filter_array.push($filter_name);
			}

			if ($('.ast-sites__filter-wrap-checkbox input[name=ast-sites-radio]:checked').length) {
				$('.ast-sites__filter-wrap-checkbox input[name=ast-sites-radio]').removeClass('active');
				$('.ast-sites__filter-wrap-checkbox input[name=ast-sites-radio]:checked').addClass('active');
			}
			var $filter_type = $('.ast-sites__filter-wrap-checkbox input[name=ast-sites-radio]:checked').val();

			if ('' != $filter_type) {
				AdaptiveSitesAdmin.filter_array.push($filter_type);
			}

			AdaptiveSitesAdmin._closeFilter();
			$('#gc-filter-search-input').trigger('keyup');
		},

		_show_search_term: function () {
			var search_term = $(this).text() || '';
			$('#gc-filter-search-input').val(search_term);
			$('#gc-filter-search-input').trigger('keyup');
		},

		_search: function (event) {

			var search_input = $(this),
				search_term = $.trim(search_input.val()) || '';

			if (13 === event.keyCode) {
				$('.adaptive-sites-autocomplete-result .ui-autocomplete').hide();
				$('.search-form').removeClass('searching');
				$('#adaptive-sites-admin').removeClass('searching');
			}

			$('body').removeClass('adaptive-sites-no-search-result');

			var sites = $('#adaptive-sites .adaptive-theme'),
				titles = $('#adaptive-sites .adaptive-theme .theme-name'),
				searchTemplateFlag = false,
				items = [];

			AdaptiveSitesAdmin.close_pages_popup();

			if (search_term.length) {
				search_input.addClass('has-input');
				$('#adaptive-sites-admin').addClass('searching');
				searchTemplateFlag = true;
			} else {
				search_input.removeClass('has-input');
				$('#adaptive-sites-admin').removeClass('searching');
			}

			items = AdaptiveSitesAdmin._get_sites_and_pages_by_search_term(search_term);

			if (!AdaptiveSitesAdmin.isEmpty(items)) {
				if (searchTemplateFlag) {
					AdaptiveSitesAdmin.add_sites_after_search(items);
				} else {
					AdaptiveSitesAdmin.add_sites(items);
				}
			} else {
				if (search_term.length) {
					$('body').addClass('adaptive-sites-no-search-result');
				}
				$('#adaptive-sites').html(gc.template('adaptive-sites-no-sites'));
			}
		},

		/**
		 * Change URL
		 */
		_changeAndSetURL: function (url_params) {
			var current_url = window.location.href;
			var current_url_separator = (window.location.href.indexOf("?") === -1) ? "?" : "&";
			var new_url = current_url + current_url_separator + decodeURIComponent($.param(url_params));
			AdaptiveSitesAdmin._changeURL(new_url);
		},

		/**
		 * Clean the URL.
		 *
		 * @param  string url URL string.
		 * @return string     Change the current URL.
		 */
		_changeURL: function (url) {
			History.pushState(null, adaptiveSitesVars.whiteLabelName, url);
		},

		/**
		 * Get URL param.
		 */
		_getParamFromURL: function (name, url) {
			if (!url) url = window.location.href;
			name = name.replace(/[\[\]]/g, "\\$&");
			var regex = new RegExp("[?&]" + name + "(=([^&#]*)|&|#|$)"),
				results = regex.exec(url);
			if (!results) return null;
			if (!results[2]) return '';
			return decodeURIComponent(results[2].replace(/\+/g, " "));
		},

		_clean_url_params: function (single_param) {
			var url_params = AdaptiveSitesAdmin._getQueryStrings();
			delete url_params[single_param];
			delete url_params[''];		// Removed extra empty object.

			var current_url = window.location.href;
			var root_url = current_url.substr(0, current_url.indexOf('?'));
			if ($.isEmptyObject(url_params)) {
				var new_url = root_url + decodeURIComponent($.param(url_params));
			} else {
				var current_url_separator = (root_url.indexOf("?") === -1) ? "?" : "&";
				var new_url = root_url + current_url_separator + decodeURIComponent($.param(url_params));
			}

			AdaptiveSitesAdmin._changeURL(new_url);
		},

		/**
		 * Get query strings.
		 *
		 * @param  string string Query string.
		 * @return string     	 Check and return query string.
		 */
		_getQueryStrings: function (string) {
			return (string || document.location.search).replace(/(^\?)/, '').split("&").map(function (n) { return n = n.split("="), this[n[0]] = n[1], this }.bind({}))[0];
		},

		isEmpty: function (obj) {
			for (var key in obj) {
				if (obj.hasOwnProperty(key))
					return false;
			}
			return true;
		},

		_unescape: function (input_string) {
			var title = _.unescape(input_string);

			// @todo check why below character not escape with function _.unescape();
			title = title.replace('&#8211;', '-');
			title = title.replace('&#8217;', "'");

			return title;
		},

		_unescape_lower: function (input_string) {
			var input_string = AdaptiveSitesAdmin._unescape(input_string);
			return input_string.toLowerCase();
		},

		_get_sites_and_pages_by_search_term: function (search_term) {

			var items = [],
				tags_strings = [];
			search_term = search_term.toLowerCase();

			if (search_term == '' && AdaptiveSitesAdmin.filter_array.length == 0) {
				return adaptiveSitesVars.default_page_builder_sites;
			}

			var $filter_type = $('.ast-sites__filter-wrap-checkbox input[name=ast-sites-radio]:checked').val();
			var $filter_name = $('.adaptive-sites__category-filter-anchor').attr('data-slug');

			for (site_id in adaptiveSitesVars.default_page_builder_sites) {

				var current_site = adaptiveSitesVars.default_page_builder_sites[site_id];
				var text_match = true;
				var free_match = true;
				var category_match = true;
				var match_id = '';

				if ('' != search_term) {
					text_match = false;
				}

				if ('' != $filter_name) {
					category_match = false;
				}

				if ('' != $filter_type) {
					free_match = false;
				}

				// Check in site title.
				if (current_site['title']) {
					var site_title = AdaptiveSitesAdmin._unescape_lower(current_site['title']);

					if (site_title.toLowerCase().includes(search_term)) {
						text_match = true;
						match_id = site_id;
					}
				}

				// Check in site tags.
				if (Object.keys(current_site['adaptive-sites-tag']).length) {
					for (site_tag_id in current_site['adaptive-sites-tag']) {
						var tag_title = current_site['adaptive-sites-tag'][site_tag_id];
						tag_title = AdaptiveSitesAdmin._unescape_lower(tag_title.replace('-', ' '));
						if (tag_title.toLowerCase().includes(search_term)) {
							text_match = true;
							match_id = site_id;
						}
					}
				}

				for (filter_id in AdaptiveSitesAdmin.filter_array) {
					var slug = AdaptiveSitesAdmin.filter_array[filter_id];
					if (slug == 'free' && 'free' == current_site['adaptive-sites-type']) {
						free_match = true;
						match_id = site_id;
					}
					if (slug == 'agency' && 'free' != current_site['adaptive-sites-type']) {
						free_match = true;
						match_id = site_id;
					}
					if (slug != 'free' && slug != 'agency' && undefined != slug) {
						for (cat_id in current_site['adaptive-site-category']) {
							if (slug.toLowerCase() == current_site['adaptive-site-category'][cat_id]) {
								category_match = true;
								match_id = site_id;
							}
						}
					}
				}

				if ('' != match_id) {
					if (text_match && category_match && free_match) {
						items[site_id] = current_site;
						items[site_id]['type'] = 'site';
						items[site_id]['site_id'] = site_id;
						items[site_id]['pages-count'] = (undefined != current_site['pages']) ? Object.keys(current_site['pages']).length : 0;
						tags_strings.push(AdaptiveSitesAdmin._unescape_lower(current_site['title']));

						for (site_tag_id in current_site['adaptive-sites-tag']) {
							var tag_title = current_site['adaptive-sites-tag'][site_tag_id];
							tag_title = AdaptiveSitesAdmin._unescape_lower(tag_title.replace('-', ' '));
							if (tag_title.toLowerCase().includes(search_term)) {
								tags_strings.push(AdaptiveSitesAdmin._unescape_lower(tag_title));
							}
						}
					}
				}

				if (search_term != '') {

					// Check in page title.
					if (Object.keys(current_site['pages']).length) {

						var pages = current_site['pages'];

						for (page_id in pages) {

							var page_text_match = false;
							var page_free_match = true;
							var page_category_match = true;
							var page_match_id = '';

							if ('' != $filter_name) {
								page_category_match = false;
							}

							if ('' != $filter_type) {
								page_free_match = false;
							}
							// Check in site title.
							if (pages[page_id]['title']) {
								var page_title = AdaptiveSitesAdmin._unescape_lower(pages[page_id]['title']);

								if (page_title.includes(search_term)) {
									page_text_match = true;
									page_match_id = page_id;
								}
							}

							// Check in site tags.
							if (Object.keys(pages[page_id]['adaptive-sites-tag']).length) {
								for (page_tag_id in pages[page_id]['adaptive-sites-tag']) {
									var tag_title = pages[page_id]['adaptive-sites-tag'][page_tag_id];
									tag_title = AdaptiveSitesAdmin._unescape_lower(tag_title.replace('-', ' '));
									if (tag_title.toLowerCase().includes(search_term)) {
										page_text_match = true;
										page_match_id = page_id;
									}
								}
							}

							for (filter_id in AdaptiveSitesAdmin.filter_array) {
								var pslug = AdaptiveSitesAdmin.filter_array[filter_id];
								if (pslug == 'free' && 'free' == pages[page_id]['adaptive-sites-type']) {
									page_free_match = true;
									page_match_id = page_id;
								}
								if (pslug == 'agency' && 'free' != pages[page_id]['adaptive-sites-type']) {
									page_free_match = true;
									page_match_id = page_id;
								}
								if (pslug != 'free' && pslug != 'agency' && undefined != pslug) {
									for (cat_id in current_site['adaptive-site-category']) {
										if (pslug.toLowerCase() == current_site['adaptive-site-category'][cat_id]) {
											page_category_match = true;
											page_match_id = page_id;
										}
									}
								}
							}

							if ('' != page_match_id) {
								if (page_text_match && page_category_match && page_free_match) {
									items[page_id] = pages[page_id];
									items[page_id]['type'] = 'page';
									items[page_id]['site_id'] = site_id;
									items[page_id]['adaptive-sites-type'] = current_site['adaptive-sites-type'] || '';
									items[page_id]['site-title'] = current_site['title'] || '';
									items[page_id]['pages-count'] = 0;

									tags_strings.push(AdaptiveSitesAdmin._unescape_lower(current_site['title']));

									for (site_tag_id in pages[page_id]['adaptive-sites-tag']) {
										var tag_title = pages[page_id]['adaptive-sites-tag'][site_tag_id];
										tag_title = AdaptiveSitesAdmin._unescape_lower(tag_title.replace('-', ' '));
										if (tag_title.toLowerCase().includes(search_term)) {
											tags_strings.push(AdaptiveSitesAdmin._unescape_lower(tag_title));
										}
									}
								}
							}
						}
					}
				}
			}

			if (search_term != '') {
				console.groupCollapsed('Search for "' + search_term + '"');
				AdaptiveSitesAdmin._log(items);
				console.groupEnd();
			}

			if (tags_strings.length > 0) {
				AdaptiveSitesAdmin.autocompleteTags = tags_strings;
				AdaptiveSitesAdmin._autocomplete();
			}

			return items;
		},

		_close_page_builder_list: function (event) {
			event.preventDefault();
			$('body').removeClass('showing-page-builders');
			$('.page-builder-icon').removeClass('active');
		},

		_toggle_page_builder_list: function (event) {
			event.preventDefault();
			$(this).toggleClass('active');
			$('body').toggleClass('showing-page-builders');
		},

		_sync_library_complete: function () {
			$.ajax({
				url: adaptiveSitesVars.ajaxurl,
				type: 'POST',
				data: {
					action: 'adaptive-sites-update-library-complete',
				},
			}).done(function (response) {
				AdaptiveSitesAdmin._log(response);
				console.groupEnd('Update Library Request');
				$('.adaptive-sites-sync-library-message').remove();
			});
		},

		_sync_library_with_ajax: function (is_append) {

			$.ajax({
				url: adaptiveSitesVars.ajaxurl,
				type: 'POST',
				data: {
					action: 'adaptive-sites-get-sites-request-count',
				},
				beforeSend: function () {
					console.groupCollapsed('库同步');
					AdaptiveSitesAdmin._log('Sync Library..');
				},
			})
				.fail(function (jqXHR) {
					AdaptiveSitesAdmin._log(jqXHR, 'error');
					AdaptiveSitesAdmin._importFailMessage(jqXHR.status + jqXHR.statusText, 'Site Count Request Failed!', jqXHR);
					console.groupEnd('库同步');
				})
				.done(function (response) {
					AdaptiveSitesAdmin._log(response);
					if (response.success) {
						var total = response.data;

						AdaptiveSitesAdmin._log(total);

						for (let i = 1; i <= total; i++) {

							AdaptiveSitesAjaxQueue.add({
								url: adaptiveSitesVars.ajaxurl,
								type: 'POST',
								data: {
									action: 'adaptive-sites-import-sites',
									page_no: i,
								},
								success: function (result) {
									AdaptiveSitesAdmin._log(result);

									if (is_append) {
										if (!AdaptiveSitesAdmin.isEmpty(result.data)) {


											var template = gc.template('adaptive-sites-page-builder-sites');

											// First fill the placeholders and then append remaining sites.
											if ($('.placeholder-site').length) {
												for (site_id in result.data) {
													if ($('.placeholder-site').length) {
														$('.placeholder-site').first().remove();
													}
												}
												if ($('#adaptive-sites .site-single:not(.placeholder-site)').length) {
													$('#adaptive-sites .site-single:not(.placeholder-site)').last().after(template(result.data));
												} else {
													$('#adaptive-sites').prepend(template(result.data));
												}
											} else {
												$('#adaptive-sites').append(template(result.data));
											}

											adaptiveSitesVars.default_page_builder_sites = $.extend({}, adaptiveSitesVars.default_page_builder_sites, result.data);

											AdaptiveSitesAdmin._load_large_images();
											$(document).trigger('adaptive-sites-added-pages');
										}

									}

									if (i === total && adaptiveSitesVars.strings.syncCompleteMessage) {
										console.groupEnd('库同步');
										$('#gcbody-content').find('.adaptive-sites-sync-library-message').remove();
										var noticeContent = gc.updates.adminNotice({
											className: 'notice adaptive-sites-notice notice-success is-dismissible adaptive-sites-sync-library-message',
											message: adaptiveSitesVars.strings.syncCompleteMessage + ' <button type="button" class="notice-dismiss"><span class="screen-reader-text">' + adaptiveSitesVars.dismiss + '</span></button>',
										});
										$('#adaptive-sites-admin').before(noticeContent);
										$(document).trigger('gc-updates-notice-added');

										$('.adaptive-sites-sync-library-button').removeClass('updating-message');
									}
								}
							});
						}

						// Run the AJAX queue.
						AdaptiveSitesAjaxQueue.run();
					} else {
						AdaptiveSitesAdmin._importFailMessage(response.data, '站点计数请求失败！');
					}
				});

			// Import categories.
			$.ajax({
				url: adaptiveSitesVars.ajaxurl,
				type: 'POST',
				data: {
					action: 'adaptive-sites-import-categories',
				},
				beforeSend: function () {
					console.groupCollapsed('Importing Categories');
					AdaptiveSitesAdmin._log('Importing Categories..');
				},
			})
				.fail(function (jqXHR) {
					AdaptiveSitesAdmin._log(jqXHR);
					AdaptiveSitesAdmin._importFailMessage(jqXHR.status + jqXHR.statusText, '类别导入失败！', jqXHR);
					console.groupEnd('Importing Categories');
				}).done(function (response) {
					AdaptiveSitesAdmin._log(response);
					console.groupEnd('Importing Categories');
				});

			// Import Site Categories.
			$.ajax({
				url: adaptiveSitesVars.ajaxurl,
				type: 'POST',
				data: {
					action: 'adaptive-sites-import-site-categories',
				},
				beforeSend: function () {
					console.groupCollapsed('Importing Site Categories');
					AdaptiveSitesAdmin._log('Importing Site Categories..');
				},
			})
				.fail(function (jqXHR) {
					AdaptiveSitesAdmin._log(jqXHR);
					AdaptiveSitesAdmin._importFailMessage(jqXHR.status + jqXHR.statusText, '类别导入失败！', jqXHR);
					console.groupCollapsed('Importing Site Categories');
				}).done(function (response) {
					AdaptiveSitesAdmin._log(response);
					console.groupCollapsed('Importing Site Categories');
				});

			// Import page builders.
			$.ajax({
				url: adaptiveSitesVars.ajaxurl,
				type: 'POST',
				data: {
					action: 'adaptive-sites-import-page-builders',
				},
				beforeSend: function () {
					console.groupCollapsed('Importing Page Builders');
					AdaptiveSitesAdmin._log('Importing Page Builders..');
				},
			})
				.fail(function (jqXHR) {
					AdaptiveSitesAdmin._log(jqXHR);
					AdaptiveSitesAdmin._importFailMessage(jqXHR.status + ' ' + jqXHR.statusText, 'Page Builder 导入失败！', jqXHR);
					console.groupEnd('Importing Page Builders');
				}).done(function (response) {
					AdaptiveSitesAdmin._log(response);
					console.groupEnd('Importing Page Builders');
				});

			// Import Blocks.
			$.ajax({
				url: adaptiveSitesVars.ajaxurl,
				type: 'POST',
				data: {
					action: 'adaptive-sites-get-blocks-request-count',
				},
				beforeSend: function () {
					console.groupCollapsed('Updating Blocks');
					AdaptiveSitesAdmin._log('Updating Blocks');
				},
			})
				.fail(function (jqXHR) {
					AdaptiveSitesAdmin._log(jqXHR, 'error');
					AdaptiveSitesAdmin._importFailMessage(jqXHR.status + jqXHR.statusText, 'Blocks Count 请求失败！', jqXHR);
					console.groupEnd('Updating Blocks');
				})
				.done(function (response) {
					AdaptiveSitesAdmin._log(response);
					if (response.success) {
						var total = response.data;

						AdaptiveSitesAdmin._log(total);

						for (let i = 1; i <= total; i++) {
							AdaptiveSitesAjaxQueue.add({
								url: adaptiveSitesVars.ajaxurl,
								type: 'POST',
								data: {
									action: 'adaptive-sites-import-blocks',
									page_no: i,
								},
								beforeSend: function () {
									console.groupCollapsed('Importing Blocks - Page ' + i);
									AdaptiveSitesAdmin._log('Importing Blocks - Page ' + i);
								},
								success: function (response) {
									AdaptiveSitesAdmin._log(response);
									console.groupEnd('Importing Blocks - Page ' + i);
								}
							});
						}

						// Run the AJAX queue.
						AdaptiveSitesAjaxQueue.run();
					} else {
						AdaptiveSitesAdmin._importFailMessage(response.data, 'Blocks Count Request Failed!');
					}
				});

			// Import Block Categories.
			$.ajax({
				url: adaptiveSitesVars.ajaxurl,
				type: 'POST',
				data: {
					action: 'adaptive-sites-import-block-categories',
				},
				beforeSend: function () {
					console.groupCollapsed('Importing Block Categories');
					AdaptiveSitesAdmin._log('Importing Block Categories..');
				},
			})
				.fail(function (jqXHR) {
					AdaptiveSitesAdmin._log(jqXHR);
					AdaptiveSitesAdmin._importFailMessage(jqXHR.status + ' ' + jqXHR.statusText, '类别导入失败！', jqXHR);
					console.groupEnd('Importing Block Categories');
				}).done(function (response) {
					AdaptiveSitesAdmin._log(response);
					console.groupEnd('Importing Block Categories');
				});

			AdaptiveSitesAdmin._sync_library_complete();
		},

		_sync_library: function (event) {
			event.preventDefault();
			var button = $(this);

			if (button.hasClass('updating-message')) {
				return;
			}

			button.addClass('updating-message');

			$('.adaptive-sites-sync-library-message').remove();

			var noticeContent = gc.updates.adminNotice({
				className: 'adaptive-sites-sync-library-message adaptive-sites-notice notice notice-info',
				message: adaptiveSitesVars.syncLibraryStart + '<button type="button" class="notice-dismiss"><span class="screen-reader-text">' + adaptiveSitesVars.dismiss + '</span></button>',
			});
			$('#adaptive-sites-admin').before(noticeContent);

			$(document).trigger('gc-updates-notice-added');

			$.ajax({
				url: adaptiveSitesVars.ajaxurl,
				type: 'POST',
				data: {
					action: 'adaptive-sites-update-library',
				},
				beforeSend: function () {
					console.groupCollapsed('Update Library Request');
					AdaptiveSitesAdmin._log('Updating Library..');
				},
			})
				.fail(function (jqXHR) {
					AdaptiveSitesAdmin._log(jqXHR);
					AdaptiveSitesAdmin._importFailMessage(jqXHR.status + ' ' + jqXHR.statusText, 'Sync Library Failed!', jqXHR);
					console.groupEnd('Update Library Request');
				})
				.done(function (response) {
					console.log(response);

					if (response.success) {
						if ('updated' === response.data) {

							$('#gcbody-content').find('.adaptive-sites-sync-library-message').remove();
							var noticeContent = gc.updates.adminNotice({
								className: 'notice adaptive-sites-notice notice-success is-dismissible adaptive-sites-sync-library-message',
								message: adaptiveSitesVars.strings.syncCompleteMessage + ' <button type="button" class="notice-dismiss"><span class="screen-reader-text">' + adaptiveSitesVars.dismiss + '</span></button>',
							});
							$('#adaptive-sites-admin').before(noticeContent);
							$(document).trigger('gc-updates-notice-added');
							button.removeClass('updating-message');
							AdaptiveSitesAdmin._log('Already sync all the sites.');
							console.groupEnd('Update Library Request');
						} else {
							AdaptiveSitesAdmin._sync_library_with_ajax();
						}
					} else {
						$('#gcbody-content').find('.adaptive-sites-sync-library-message').remove();
						var noticeContent = gc.updates.adminNotice({
							className: 'notice adaptive-sites-notice notice-error is-dismissible adaptive-sites-sync-library-message',
							message: response.data + ' <button type="button" class="notice-dismiss"><span class="screen-reader-text">' + adaptiveSitesVars.dismiss + '</span></button>',
						});
						$('#adaptive-sites-admin').before(noticeContent);
						$(document).trigger('gc-updates-notice-added');
						button.removeClass('updating-message');
						AdaptiveSitesAdmin._log('Already sync all the sites.');
						console.groupEnd('Update Library Request');
					}
				});
		},

		_filterSites: function (event) {
			event.preventDefault();
			var current_class = $(this).attr('data-group') || '';
			$(this).parents('.filter-links').find('a').removeClass('current');
			$(this).addClass('current');

			var search_term = $(this).text() || '';

			if (current_class) {
				if ($('#adaptive-sites .adaptive-theme.' + current_class).length) {
					$('#gc-filter-search-input').val(search_term);

					// $('#adaptive-sites .adaptive-theme').removeClass('adaptive-show-site adaptive-hide-site');
					$('#adaptive-sites .adaptive-theme').addClass('adaptive-hide-site');
					$('#adaptive-sites .adaptive-theme.' + current_class).removeClass('adaptive-hide-site').addClass('adaptive-show-site');
				}
			} else {
				$('#adaptive-sites .adaptive-theme').removeClass('adaptive-hide-site').addClass('adaptive-show-site');
			}

			$('.filters-wrap-page-categories').removeClass('show');
		},

		_next_and_previous_sites: function (e) {

			if (!$('body').hasClass('adaptive-previewing-single-pages')) {
				return;
			}

			if (e.key === "Escape") {
				AdaptiveSitesAdmin.close_pages_popup();
				return;
			}

			switch (e.which) {

				// Left Key Pressed
				case 37:
					if ($('#adaptive-sites .adaptive-theme.current').prev().length) {
						$('#adaptive-sites .adaptive-theme.current').prev().addClass('current').siblings().removeClass('current');
						var site_id = $('#adaptive-sites .adaptive-theme.current').prev().attr('data-site-id') || '';
						if (site_id) {
							AdaptiveSitesAdmin.show_pages_by_site_id(site_id);
						}
					}
					break;

				// Right Key Pressed
				case 39:
					if ($('#adaptive-sites .adaptive-theme.current').next().length) {
						$('#adaptive-sites .adaptive-theme.current').next().addClass('current').siblings().removeClass('current');
						var site_id = $('#adaptive-sites .adaptive-theme.current').next().attr('data-site-id') || '';
						if (site_id) {
							AdaptiveSitesAdmin.show_pages_by_site_id(site_id);
						}
					}
					break;
			}

		},

		show_pages_by_site_id: function (site_id, page_id) {

			var sites = adaptiveSitesVars.default_page_builder_sites || [];

			var data = sites[site_id];

			if ('undefined' !== typeof data) {
				var site_template = gc.template('adaptive-sites-single-site-preview');

				if (!AdaptiveSitesAdmin._getParamFromURL('adaptive-site')) {
					var url_params = {
						'adaptive-site': site_id,
					};
					AdaptiveSitesAdmin._changeAndSetURL(url_params);
				}

				$('#adaptive-sites').hide();
				$('#site-pages').show().html(site_template(data)).removeClass('elementor gutenberg').addClass(adaptiveSitesVars.default_page_builder);

				$('body').addClass('adaptive-previewing-single-pages');
				$('#site-pages').attr('data-site-id', site_id);

				if (AdaptiveSitesAdmin._getParamFromURL('adaptive-page')) {
					AdaptiveSitesAdmin._set_preview_screenshot_by_page($('#single-pages .site-single[data-page-id="' + AdaptiveSitesAdmin._getParamFromURL('adaptive-page') + '"]'));
					// Has first item?
					// Then set default screnshot in preview.
				} else if (page_id && $('#single-pages .site-single[data-page-id="' + page_id + '"]').length) {
					AdaptiveSitesAdmin._set_preview_screenshot_by_page($('#single-pages .site-single[data-page-id="' + page_id + '"]'));
				} else if ($('#single-pages .site-single').eq(0).length) {
					AdaptiveSitesAdmin._set_preview_screenshot_by_page($('#single-pages .site-single').eq(0));
				}

				if (!$('#single-pages .site-single').eq(0).length) {
					$('.site-import-layout-button').hide();
				}

				$(document).trigger('adaptive-sites-added-pages');

				AdaptiveSitesAdmin._load_large_images();
			}

		},

		_show_sites: function (event) {

			event.preventDefault();

			$('.adaptive-sites-show-favorite-button').removeClass('active');
			$('body').removeClass('adaptive-sites-showing-favorites');
			$('body').removeClass('adaptive-sites-no-search-result');
			$('.adaptive-sites__category-filter-items').find('.ast-sites__filter-wrap').removeClass('category-active');
			$('.ast-sites__filter-wrap').first().addClass('category-active');
			$('.adaptive-sites__category-filter-anchor').attr('data-slug', '');
			AdaptiveSitesAdmin.filter_array = [];
			$('.ast-sites__filter-wrap-checkbox input:radio').attr('checked', false);
			$('.ast-sites__filter-wrap-checkbox input:radio').removeClass('active');
			$('#radio-all').trigger('click');
			$('#radio-all').addClass('active');
			$('.adaptive-sites__category-filter-anchor').text('All');
			AdaptiveSitesAdmin._closeFilter();
			$('#gc-filter-search-input').val('');
			$('#adaptive-sites-admin').removeClass('searching');
			AdaptiveSitesAdmin.add_sites(adaptiveSitesVars.default_page_builder_sites);
			AdaptiveSitesAdmin.close_pages_popup();

			AdaptiveSitesAdmin._clean_url_params('favorites');

			AdaptiveSitesAdmin._load_large_images();
		},

		/**
		 * Go back to all sites view
		 *
		 * @since 2.0.0
		 * @return null
		 */
		_go_back: function (event) {

			event.preventDefault();

			AdaptiveSitesAdmin._clean_url_params('search');
			AdaptiveSitesAdmin._clean_url_params('favorites');
			AdaptiveSitesAdmin._clean_url_params('license');
			AdaptiveSitesAdmin.close_pages_popup();
			AdaptiveSitesAdmin._load_large_images();
		},

		close_pages_popup: function () {
			adaptiveSitesVars.cpt_slug = 'adaptive-sites';

			$('#adaptive-sites').show();
			$('#site-pages').hide().html('');
			$('body').removeClass('adaptive-previewing-single-pages');
			$('.adaptive-sites-result-preview').hide();

			$('#adaptive-sites .adaptive-theme').removeClass('current');

			AdaptiveSitesAdmin._clean_url_params('adaptive-site');
			AdaptiveSitesAdmin._clean_url_params('adaptive-page');
			AdaptiveSitesAdmin._clean_url_params('license');
		},


		_toggle_favorite: function (event) {

			let is_favorite = $(this).data('favorite');
			let parent = $(this).parents('.adaptive-theme');
			let site_id = parent.data('site-id').toString();
			let new_array = Array();

			parent.toggleClass('is-favorite');
			$(this).data('favorite', !is_favorite);

			if (!is_favorite) {
				// Add.
				for (value in adaptiveSitesVars.favorite_data) {
					new_array.push(adaptiveSitesVars.favorite_data[value]);
				}
				new_array.push(site_id);
			} else {
				// Remove.
				for (value in adaptiveSitesVars.favorite_data) {
					if (site_id != adaptiveSitesVars.favorite_data[value].toString()) {
						new_array.push(adaptiveSitesVars.favorite_data[value]);
					}
				}
			}
			adaptiveSitesVars.favorite_data = new_array;

			// If in favorites preview window and unfavorite the item?
			if ($('body').hasClass('adaptive-sites-showing-favorites') && !parent.hasClass('is-favorite')) {

				// Then remove the favorite item from markup.
				parent.remove();

				// Show Empty Favorite message if there is not item in favorite.
				if (!$('#adaptive-sites .adaptive-theme').length) {
					$('#adaptive-sites').html(gc.template('adaptive-sites-no-favorites'));
				}
			}

			$.ajax({
				url: adaptiveSitesVars.ajaxurl,
				type: 'POST',
				dataType: 'json',
				data: {
					action: 'adaptive-sites-favorite',
					is_favorite: !is_favorite,
					site_id: site_id
				},
				beforeSend: function () {
					console.groupCollapsed('Toggle Favorite');
					AdaptiveSitesAdmin._log(!is_favorite);
				},
			})
				.fail(function (jqXHR) {
					AdaptiveSitesAdmin._log(jqXHR);
					AdaptiveSitesAdmin._importFailMessage(jqXHR.status + ' ' + jqXHR.statusText, 'Favorite/Unfavorite Failed!', jqXHR);
					console.groupEnd();
				})
				.done(function (response) {
					AdaptiveSitesAdmin._log(response);
					console.groupEnd();
				});
		},


		_show_favorite: function (event) {

			if (event) {
				event.preventDefault();
			}

			AdaptiveSitesAdmin.close_pages_popup();

			$('.adaptive-sites__category-filter-items').find('.ast-sites__filter-wrap').removeClass('category-active');
			$('.ast-sites__filter-wrap').first().addClass('category-active');
			$('.adaptive-sites__category-filter-anchor').attr('data-slug', '');
			AdaptiveSitesAdmin.filter_array = [];
			$('.ast-sites__filter-wrap-checkbox input:radio').attr('checked', false);
			$('.ast-sites__filter-wrap-checkbox input:radio').removeClass('active');
			$('#radio-all').trigger('click');
			$('#radio-all').addClass('active');
			$('.adaptive-sites__category-filter-anchor').text('All');
			$('#gc-filter-search-input').val('');

			if ($('.adaptive-sites-show-favorite-button').hasClass('active')) {
				$('.adaptive-sites-show-favorite-button').removeClass('active');
				$('body').removeClass('adaptive-sites-showing-favorites');
				AdaptiveSitesAdmin.add_sites(adaptiveSitesVars.default_page_builder_sites);
				AdaptiveSitesAdmin._clean_url_params('favorites');
			} else {
				AdaptiveSitesAdmin._clean_url_params('search');
				AdaptiveSitesAdmin._clean_url_params('adaptive-site');
				AdaptiveSitesAdmin._clean_url_params('adaptive-page');
				AdaptiveSitesAdmin._clean_url_params('license');
				AdaptiveSitesAdmin.close_pages_popup();

				if (!AdaptiveSitesAdmin._getParamFromURL('favorites')) {
					var url_params = {
						'favorites': 'show'
					};
					AdaptiveSitesAdmin._changeAndSetURL(url_params);
				}

				$('.adaptive-sites-show-favorite-button').addClass('active');
				$('body').addClass('adaptive-sites-showing-favorites');
				var items = [];
				for (favorite_id in adaptiveSitesVars.favorite_data) {
					var exist_data = adaptiveSitesVars.default_page_builder_sites[adaptiveSitesVars.favorite_data[favorite_id].toString()] || {};
					if (!$.isEmptyObject(exist_data)) {
						items[adaptiveSitesVars.favorite_data[favorite_id].toString()] = exist_data;
					}
				}

				if (!AdaptiveSitesAdmin.isEmpty(items)) {
					AdaptiveSitesAdmin.add_sites(items);
					$(document).trigger('adaptive-sites-added-sites');

				} else {
					$('#adaptive-sites').html(gc.template('adaptive-sites-no-favorites'));
				}
			}

		},

		_set_preview_screenshot_by_page: function (element) {
			var large_img_url = $(element).find('.theme-screenshot').attr('data-featured-src') || '';
			var url = $(element).find('.theme-screenshot').attr('data-src') || '';
			var page_name = $(element).find('.theme-name').text() || '';

			$(element).siblings().removeClass('current_page');
			$(element).addClass('current_page');

			var page_id = $(element).attr('data-page-id') || '';
			if (page_id) {

				AdaptiveSitesAdmin._clean_url_params('adaptive-page');

				var url_params = {
					'adaptive-page': page_id,
				};
				AdaptiveSitesAdmin._changeAndSetURL(url_params);
			}

			$('.site-import-layout-button').removeClass('disabled');
			if (page_name) {
				var title = adaptiveSitesVars.strings.importSingleTemplate.replace('%s', page_name.trim());
				$('.site-import-layout-button').text(title);
			}

			if (url) {
				$('.single-site-preview').animate({
					scrollTop: 0
				}, 0);
				$('.single-site-preview img').addClass('loading').attr('src', url);
				var imgLarge = new Image();
				imgLarge.src = large_img_url;
				imgLarge.onload = function () {
					$('.single-site-preview img').removeClass('loading');
					$('.single-site-preview img').attr('src', imgLarge.src);
				};
			}
		},

		/**
		 * Preview Inner Pages for the Site
		 *
		 * @since 2.0.0
		 * @return null
		 */
		_change_site_preview_screenshot: function (event) {
			event.preventDefault();

			var item = $(this);

			AdaptiveSitesAdmin._set_preview_screenshot_by_page(item);
		},

		_show_pages: function (event) {

			var perent = $(this).parents('.adaptive-theme');
			perent.siblings().removeClass('current');
			perent.addClass('current');

			var site_id = perent.attr('data-site-id') || '';
			AdaptiveSitesAdmin.show_pages_by_site_id(site_id);
		},

		_apiAddParam_status: function () {
			if (adaptiveSitesVars.sites && adaptiveSitesVars.sites.status) {
				AdaptiveSitesAdmin._api_params['status'] = adaptiveSitesVars.sites.status;
			}
		},

		_apiAddParam_per_page: function () {
			// Add 'per_page'
			var per_page_val = 30;
			if (adaptiveSitesVars.sites && adaptiveSitesVars.sites["per-page"]) {
				per_page_val = parseInt(adaptiveSitesVars.sites["per-page"]);
			}
			AdaptiveSitesAdmin._api_params['per_page'] = per_page_val;
		},

		_apiAddParam_adaptive_site_category: function () {
			// Add 'adaptive-site-category'
			var selected_category_id = jQuery('.filter-links[data-category="' + adaptiveSitesVars.category_slug + '"]').find('.current').data('group') || '';
			if ('' !== selected_category_id && 'all' !== selected_category_id) {
				AdaptiveSitesAdmin._api_params[adaptiveSitesVars.category_slug] = selected_category_id;
			} else if (adaptiveSitesVars.sites && adaptiveSitesVars['categories'].include) {
				if (AdaptiveSitesAdmin._isArray(adaptiveSitesVars['categories'].include)) {
					AdaptiveSitesAdmin._api_params[adaptiveSitesVars.category_slug] = adaptiveSitesVars['categories'].include.join(',');
				} else {
					AdaptiveSitesAdmin._api_params[adaptiveSitesVars.category_slug] = adaptiveSitesVars['categories'].include;
				}
			}
		},

		_apiAddParam_adaptive_page_parent_category: function () {

			// Add 'site-pages-parent-category'
			if ('' == adaptiveSitesVars.parent_category) {
				return;
			}

			var selected_category_id = jQuery('.filter-links[data-category="' + adaptiveSitesVars.parent_category + '"]').find('.current').data('group') || '';
			if ('' !== selected_category_id && 'all' !== selected_category_id) {
				AdaptiveSitesAdmin._api_params[adaptiveSitesVars.parent_category] = selected_category_id;
			} else if (adaptiveSitesVars.sites && adaptiveSitesVars['categories'].include) {
				if (AdaptiveSitesAdmin._isArray(adaptiveSitesVars['categories'].include)) {
					AdaptiveSitesAdmin._api_params[adaptiveSitesVars.parent_category] = adaptiveSitesVars['categories'].include.join(',');
				} else {
					AdaptiveSitesAdmin._api_params[adaptiveSitesVars.parent_category] = adaptiveSitesVars['categories'].include;
				}
			}
		},

		_apiAddParam_adaptive_site_page_builder: function () {
			// Add 'adaptive-site-page-builder'
			var selected_page_builder_id = jQuery('.filter-links[data-category="' + adaptiveSitesVars.page_builder + '"]').find('.current').data('group') || '';
			if ('' !== selected_page_builder_id && 'all' !== selected_page_builder_id) {
				AdaptiveSitesAdmin._api_params[adaptiveSitesVars.page_builder] = selected_page_builder_id;
			} else if (adaptiveSitesVars.sites && adaptiveSitesVars['page-builders'].include) {
				if (AdaptiveSitesAdmin._isArray(adaptiveSitesVars['page-builders'].include)) {
					AdaptiveSitesAdmin._api_params[adaptiveSitesVars.page_builder] = adaptiveSitesVars['page-builders'].include.join(',');
				} else {
					AdaptiveSitesAdmin._api_params[adaptiveSitesVars.page_builder] = adaptiveSitesVars['page-builders'].include;
				}
			}
		},

		_apiAddParam_purchase_key: function () {
			if (adaptiveSitesVars.sites && adaptiveSitesVars.sites.purchase_key) {
				AdaptiveSitesAdmin._api_params['purchase_key'] = adaptiveSitesVars.sites.purchase_key;
			}
		},

		_apiAddParam_site_url: function () {
			if (adaptiveSitesVars.sites && adaptiveSitesVars.sites.site_url) {
				AdaptiveSitesAdmin._api_params['site_url'] = adaptiveSitesVars.sites.site_url;
			}
		},

		_show_default_page_builder_sites: function () {

			if (!$('#adaptive-sites').length) {
				return;
			}

			if (Object.keys(adaptiveSitesVars.default_page_builder_sites).length) {
				var favorites = AdaptiveSitesAdmin._getParamFromURL('favorites');
				var search_term = AdaptiveSitesAdmin._getParamFromURL('search');
				if (search_term) {
					var items = AdaptiveSitesAdmin._get_sites_and_pages_by_search_term(search_term);

					if (!AdaptiveSitesAdmin.isEmpty(items)) {
						AdaptiveSitesAdmin.add_sites(items);
						$('#gc-filter-search-input').val(search_term);
					} else {
						$('#adaptive-sites').html(adaptiveSitesVars.default_page_builder_sites);
					}

				} else if (favorites) {
					AdaptiveSitesAdmin._show_favorite();
				} else {
					AdaptiveSitesAdmin.add_sites(adaptiveSitesVars.default_page_builder_sites);
				}

				// Show single site preview.
				var site_id = AdaptiveSitesAdmin._getParamFromURL('adaptive-site');

				if (site_id) {
					AdaptiveSitesAdmin.show_pages_by_site_id(site_id);
				}
			} else {

				var temp = [];
				for (var i = 0; i < 8; i++) {
					temp['id-' + i] = {
						'title': 'Lorem Ipsum',
						'class': 'placeholder-site',
					};
				}

				AdaptiveSitesAdmin.add_sites(temp);
				$('#adaptive-sites').addClass('temp');

				AdaptiveSitesAdmin._sync_library_with_ajax(true);
			}

			var show_license = AdaptiveSitesAdmin._getParamFromURL('license');
			if (show_license) {
				AdaptiveSitesAdmin._show_activate_license_notice();
			}
		},

		_change_page_builder: function () {
			var page_builder = $(this).attr('data-page-builder') || '';

			$(this).parents('.page-builders').find('img').removeClass('active');
			$(this).find('img').addClass('active');

			$.ajax({
				url: adaptiveSitesVars.ajaxurl,
				type: 'POST',
				data: {
					action: 'adaptive-sites-change-page-builder',
					page_builder: page_builder,
				},
				beforeSend: function () {
					console.groupCollapsed('Change Page Builder');
					AdaptiveSitesAdmin._log('Change Page Builder..');
				},
			})
				.done(function (response) {
					AdaptiveSitesAdmin._log(response);
					AdaptiveSitesAdmin._clean_url_params('adaptive-site');
					AdaptiveSitesAdmin._clean_url_params('adaptive-page');
					AdaptiveSitesAdmin._clean_url_params('change-page-builder');
					AdaptiveSitesAdmin._clean_url_params('license');
					console.groupEnd();
					location.reload();
				});
		},

		_ajax_change_page_builder: function () {

			var page_builder_slug = $(this).attr('data-page-builder') || '';
			var page_builder_img = $(this).find('img').attr('src') || '';
			var page_builder_title = $(this).find('.title').text() || '';
			if (page_builder_img) {
				$('.selected-page-builder').find('img').attr('src', page_builder_img);
			}
			if (page_builder_title) {
				$('.selected-page-builder').find('.page-builder-title').text(page_builder_title);
			}

			$('#gc-filter-search-input').val('');
			$('#adaptive-sites-admin').removeClass('searching');
			$('body').removeClass('adaptive-previewing-single-pages');

			if ($('.page-builders [data-page-builder="' + page_builder_slug + '"]').length) {
				$('.page-builders [data-page-builder="' + page_builder_slug + '"]').siblings().removeClass('active');
				$('.page-builders [data-page-builder="' + page_builder_slug + '"]').addClass('active');
			}

			if (page_builder_slug) {

				AdaptiveSitesAdmin._clean_url_params('adaptive-site');
				AdaptiveSitesAdmin._clean_url_params('adaptive-page');
				AdaptiveSitesAdmin._clean_url_params('license');

				$('#adaptive-sites').show();
				$('#site-pages').hide();

				$.ajax({
					url: adaptiveSitesVars.ajaxurl,
					type: 'POST',
					data: {
						action: 'adaptive-sites-change-page-builder',
						page_builder: page_builder_slug,
					},
					beforeSend: function () {
						console.groupCollapsed('Change Page Builder');
						AdaptiveSitesAdmin._log('Change Page Builder..');
					},
				})
					.done(function (response) {
						AdaptiveSitesAdmin._log(response);
						console.groupEnd();
						if (response.success) {
							$('.adaptive-sites__category-filter-items').find('.ast-sites__filter-wrap').removeClass('category-active');
							$('.ast-sites__filter-wrap').first().addClass('category-active');
							$('.adaptive-sites__category-filter-anchor').attr('data-slug', '');
							AdaptiveSitesAdmin.filter_array = [];
							$('.ast-sites__filter-wrap-checkbox input:radio').attr('checked', false);
							$('.ast-sites__filter-wrap-checkbox input:radio').removeClass('active');
							$('#radio-all').trigger('click');
							$('#radio-all').addClass('active');
							$('.adaptive-sites__category-filter-anchor').text('All');
							AdaptiveSitesAdmin._closeFilter();

							adaptiveSitesVars.default_page_builder = page_builder_slug;

							// Set changed page builder data as a default page builder object.
							adaptiveSitesVars.default_page_builder_sites = response.data;
							$('.adaptive-sites-show-favorite-button').removeClass('active');
							AdaptiveSitesAdmin.add_sites(response.data);

							AdaptiveSitesAdmin._autocomplete();
							AdaptiveSitesAdmin.quick_corner_cta_link = adaptiveSitesVars.cta_quick_corner_links[ page_builder_slug ];
							AdaptiveSitesAdmin.premium_popup_cta_link = adaptiveSitesVars.cta_premium_popup_links[ page_builder_slug ];
							AdaptiveSitesAdmin.default_cta_link = adaptiveSitesVars.cta_links[ page_builder_slug ];
							$(document).trigger('adaptive-sites-change-page-builder', page_builder_slug, response.data, response);
						}
					});

			}
		},

		add_sites_after_search: function (data) {
			var template = gc.template('adaptive-sites-page-builder-sites-search');
			$('#adaptive-sites').html(template(data));
			AdaptiveSitesAdmin._load_large_images();
			$(document).trigger('adaptive-sites-added-sites');
		},

		add_sites: function (data) {
			var template = gc.template('adaptive-sites-page-builder-sites');
			$('#adaptive-sites').html(template(data));
			AdaptiveSitesAdmin._load_large_images();
			$(document).trigger('adaptive-sites-added-sites');
		},

		_toggle_tooltip: function (event) {
			event.preventDefault();
			var tip_id = $(this).data('tip-id') || '';
			if (tip_id && $('#' + tip_id).length) {
				$('#' + tip_id).toggle();
			}
		},

		_resetData: function () {

			if ($(this).hasClass('updating-message')) {
				return;
			}
			if (AdaptiveSitesAdmin.subscribe_skiped || AdaptiveSitesAdmin.subscription_form_submitted == 'yes') {
				$('.user-building-for-title').hide();
				$('.adaptive-sites-advanced-options').show();
				$('.adaptive-sites-advanced-options-heading').hide();
				$('#adaptive-sites-subscription-form-one').hide();
			}
			if (false === AdaptiveSitesAdmin.subscribe_skiped && $('.subscription-enabled').length && AdaptiveSitesAdmin.subscription_form_submitted !== 'yes') {
				AdaptiveSitesAdmin._validate_field($('.subscription-input-gc-user-type'));
				AdaptiveSitesAdmin._validate_field($('.subscription-input-build-website-for'));

				if ($('.subscription-field-wrap').hasClass('subscription-error')) {
					console.log('error');
					return;
				}
			}

			$('.site-import-cancel').show();

			$('.install-theme-info').hide();

			if (false === AdaptiveSitesAdmin.subscribe_skiped && $('.subscription-enabled').length && AdaptiveSitesAdmin.subscription_form_submitted !== 'yes') {
				$('.subscription-popup').show();
				$('.adaptive-sites-result-preview .default').hide();
			} else {
				AdaptiveSitesAdmin.subscribe_status = true;
				$('.ast-importing-wrap').show();
			}

			AdaptiveSitesAdmin.import_start_time = new Date();

			$(this).addClass('updating-message installing').text('Importing..');
			$('body').addClass('importing-site');

			var output = '<div class="current-importing-status-title"></div><div class="current-importing-status-description"></div>';
			$('.current-importing-status').html(output);

			// Process Theme Activate and Install Process
			// if ($('.adaptive-sites-theme-activation .checkbox').is(':checked')) {
			// 	var status = $('.adaptive-sites-theme-activation .checkbox').data('status')
			// 	AdaptiveSitesAdmin._installAdaptive(status);
			// }

			$.ajax({
				url: adaptiveSitesVars.ajaxurl,
				type: 'POST',
				data: {
					action: 'adaptive-sites-set-reset-data',
					_ajax_nonce: adaptiveSitesVars._ajax_nonce,
				},
				beforeSend: function () {
					console.groupCollapsed('站点重置数据');
				},
			})
				.done(function (response) {
					console.log('List of Reset Items:');
					AdaptiveSitesAdmin._log(response);
					console.groupEnd();
					if (response.success) {
						AdaptiveSitesAdmin.site_imported_data = response.data;

						// Process Bulk Plugin Install & Activate.
						AdaptiveSitesAdmin._bulkPluginInstallActivate();
						// Process Bulk Theme Install & Activate.
						AdaptiveSitesAdmin._bulkThemeInstallActivate();
					}
				});

		},

		_remove_skip_and_import_popup: function (event) {
			event.preventDefault();

			$(this).parents('.skip-and-import').addClass('hide-me visited');

			if ($('.skip-and-import.hide-me').not('.visited').length) {
				$('.skip-and-import.hide-me').not('.visited').first().removeClass('hide-me');
			} else {
				$('.adaptive-sites-result-preview .default').removeClass('hide-me');

				if ($('.adaptive-sites-result-preview').hasClass('import-page')) {

					AdaptiveSitesAdmin.skip_and_import_popups = [];

					var notinstalled = AdaptiveSitesAdmin.required_plugins.notinstalled || 0;
					if (!notinstalled.length) {
						AdaptiveSitesAdmin.import_page_process();
					}
				}
			}
		},

		_start_site_import: function () {

			if (AdaptiveSitesAdmin._is_reset_data()) {
				$(document).trigger('adaptive-sites-reset-data');
			} else {
				$(document).trigger('adaptive-sites-reset-data-done');
			}
		},

		_reset_customizer_data: function () {
			$.ajax({
				url: adaptiveSitesVars.ajaxurl,
				type: 'POST',
				data: {
					action: 'adaptive-sites-reset-customizer-data',
					_ajax_nonce: adaptiveSitesVars._ajax_nonce,
				},
				beforeSend: function () {
					console.groupCollapsed('重置自定义数据');
					AdaptiveSitesAdmin._log_title('重置自定义数据..');
				},
			})
				.fail(function (jqXHR) {
					AdaptiveSitesAdmin._log(jqXHR);
					AdaptiveSitesAdmin._importFailMessage(jqXHR.status + ' ' + jqXHR.statusText, '重置自定义数据失败!', jqXHR);
					console.groupEnd();
				})
				.done(function (data) {
					AdaptiveSitesAdmin._log(data);
					AdaptiveSitesAdmin._log_title('重置自定义数据完成..');
					AdaptiveSitesAdmin._log('重置自定义数据完成..');
					console.groupEnd();
					$(document).trigger('adaptive-sites-reset-customizer-data-done');
				});
		},

		_reset_site_options: function () {
			// Site Options.
			$.ajax({
				url: adaptiveSitesVars.ajaxurl,
				type: 'POST',
				data: {
					action: 'adaptive-sites-reset-site-options',
					_ajax_nonce: adaptiveSitesVars._ajax_nonce,
				},
				beforeSend: function () {
					console.groupCollapsed('重置站点选项');
					AdaptiveSitesAdmin._log_title('重置站点选项..');
				},
			})
				.fail(function (jqXHR) {
					AdaptiveSitesAdmin._log(jqXHR);
					AdaptiveSitesAdmin._importFailMessage(jqXHR.status + ' ' + jqXHR.statusText, '重置站点选项失败!', jqXHR);
					console.groupEnd();
				})
				.done(function (data) {
					AdaptiveSitesAdmin._log(data);
					AdaptiveSitesAdmin._log_title('重置站点选项完成..');
					console.groupEnd();
					$(document).trigger('adaptive-sites-reset-site-options-done');
				});
		},

		_reset_widgets_data: function () {
			// Widgets.
			$.ajax({
				url: adaptiveSitesVars.ajaxurl,
				type: 'POST',
				data: {
					action: 'adaptive-sites-reset-widgets-data',
					_ajax_nonce: adaptiveSitesVars._ajax_nonce,
				},
				beforeSend: function () {
					console.groupCollapsed('正在重置小工具');
					AdaptiveSitesAdmin._log_title('正在重置小工具..');
				},
			})
				.fail(function (jqXHR) {
					AdaptiveSitesAdmin._log(jqXHR);
					AdaptiveSitesAdmin._importFailMessage(jqXHR.status + ' ' + jqXHR.statusText, '重置小工具数据失败!', jqXHR);
					console.groupEnd();
				})
				.done(function (data) {
					AdaptiveSitesAdmin._log(data);
					AdaptiveSitesAdmin._log_title('重置小工具完成..');
					console.groupEnd();
					$(document).trigger('adaptive-sites-reset-widgets-data-done');
				});
		},

		_reset_posts: function () {
			if (AdaptiveSitesAdmin.site_imported_data['reset_posts'].length) {

				AdaptiveSitesAdmin.reset_remaining_posts = AdaptiveSitesAdmin.site_imported_data['reset_posts'].length;

				console.groupCollapsed('正在删除文章');
				AdaptiveSitesAdmin._log_title('正在删除文章..');

				$.each(AdaptiveSitesAdmin.site_imported_data['reset_posts'], function (index, post_id) {

					AdaptiveSitesAjaxQueue.add({
						url: adaptiveSitesVars.ajaxurl,
						type: 'POST',
						data: {
							action: 'adaptive-sites-delete-posts',
							post_id: post_id,
							_ajax_nonce: adaptiveSitesVars._ajax_nonce,
						},
						success: function (result) {

							if (AdaptiveSitesAdmin.reset_processed_posts < AdaptiveSitesAdmin.site_imported_data['reset_posts'].length) {
								AdaptiveSitesAdmin.reset_processed_posts += 1;
							}

							AdaptiveSitesAdmin._log_title('正在删除文章 ' + AdaptiveSitesAdmin.reset_processed_posts + ' of ' + AdaptiveSitesAdmin.site_imported_data['reset_posts'].length + '<br/>' + result.data);

							AdaptiveSitesAdmin.reset_remaining_posts -= 1;
							if (0 == AdaptiveSitesAdmin.reset_remaining_posts) {
								console.groupEnd();
								$(document).trigger('adaptive-sites-delete-posts-done');
								$(document).trigger('adaptive-sites-reset-data-done');
							}
						}
					});
				});
				AdaptiveSitesAjaxQueue.run();

			} else {
				$(document).trigger('adaptive-sites-delete-posts-done');
				$(document).trigger('adaptive-sites-reset-data-done');
			}
		},

		_reset_gc_forms: function () {
			if (AdaptiveSitesAdmin.site_imported_data['reset_gc_forms'].length) {
				AdaptiveSitesAdmin.reset_remaining_gc_forms = AdaptiveSitesAdmin.site_imported_data['reset_gc_forms'].length;

				console.groupCollapsed('正在删除表单');
				AdaptiveSitesAdmin._log_title('正在删除表单..');

				$.each(AdaptiveSitesAdmin.site_imported_data['reset_gc_forms'], function (index, post_id) {
					AdaptiveSitesAjaxQueue.add({
						url: adaptiveSitesVars.ajaxurl,
						type: 'POST',
						data: {
							action: 'adaptive-sites-delete-gc-forms',
							post_id: post_id,
							_ajax_nonce: adaptiveSitesVars._ajax_nonce,
						},
						success: function (result) {

							if (AdaptiveSitesAdmin.reset_processed_gc_forms < AdaptiveSitesAdmin.site_imported_data['reset_gc_forms'].length) {
								AdaptiveSitesAdmin.reset_processed_gc_forms += 1;
							}

							AdaptiveSitesAdmin._log_title('正在删除表单 ' + AdaptiveSitesAdmin.reset_processed_gc_forms + ' of ' + AdaptiveSitesAdmin.site_imported_data['reset_gc_forms'].length + '<br/>' + result.data);
							AdaptiveSitesAdmin._log('正在删除表单 ' + AdaptiveSitesAdmin.reset_processed_gc_forms + ' of ' + AdaptiveSitesAdmin.site_imported_data['reset_gc_forms'].length + '<br/>' + result.data);

							AdaptiveSitesAdmin.reset_remaining_gc_forms -= 1;
							if (0 == AdaptiveSitesAdmin.reset_remaining_gc_forms) {
								console.groupEnd();
								$(document).trigger('adaptive-sites-delete-gc-forms-done');
							}
						}
					});
				});
				AdaptiveSitesAjaxQueue.run();

			} else {
				$(document).trigger('adaptive-sites-delete-gc-forms-done');
			}
		},

		_reset_terms: function () {

			if (AdaptiveSitesAdmin.site_imported_data['reset_terms'].length) {
				AdaptiveSitesAdmin.reset_remaining_terms = AdaptiveSitesAdmin.site_imported_data['reset_terms'].length;

				console.groupCollapsed('Deleting Terms');
				AdaptiveSitesAdmin._log_title('Deleting Terms..');

				$.each(AdaptiveSitesAdmin.site_imported_data['reset_terms'], function (index, term_id) {
					AdaptiveSitesAjaxQueue.add({
						url: adaptiveSitesVars.ajaxurl,
						type: 'POST',
						data: {
							action: 'adaptive-sites-delete-terms',
							term_id: term_id,
							_ajax_nonce: adaptiveSitesVars._ajax_nonce,
						},
						success: function (result) {
							if (AdaptiveSitesAdmin.reset_processed_terms < AdaptiveSitesAdmin.site_imported_data['reset_terms'].length) {
								AdaptiveSitesAdmin.reset_processed_terms += 1;
							}

							AdaptiveSitesAdmin._log_title('Deleting Term ' + AdaptiveSitesAdmin.reset_processed_terms + ' of ' + AdaptiveSitesAdmin.site_imported_data['reset_terms'].length + '<br/>' + result.data);
							AdaptiveSitesAdmin._log('Deleting Term ' + AdaptiveSitesAdmin.reset_processed_terms + ' of ' + AdaptiveSitesAdmin.site_imported_data['reset_terms'].length + '<br/>' + result.data);

							AdaptiveSitesAdmin.reset_remaining_terms -= 1;
							if (0 == AdaptiveSitesAdmin.reset_remaining_terms) {
								console.groupEnd();
								$(document).trigger('adaptive-sites-delete-terms-done');
							}
						}
					});
				});
				AdaptiveSitesAjaxQueue.run();
			} else {
				$(document).trigger('adaptive-sites-delete-terms-done');
			}

		},

		_toggle_reset_notice: function () {
			if ($(this).is(':checked')) {
				$('#adaptive-sites-tooltip-reset-data').show();
			} else {
				$('#adaptive-sites-tooltip-reset-data').hide();
			}
		},

		_toggle_theme_notice: function () {
			var adaptive_dependent_plugins = ['adaptive-addon'];

			if (AdaptiveSitesAdmin.isEmpty(AdaptiveSitesAdmin.required_plugins_original_list)) {
				AdaptiveSitesAdmin.required_plugins_original_list = adaptiveSitesVars.requiredPlugins;
			}

			var plugins = AdaptiveSitesAdmin.required_plugins_original_list;

			$(this).parents('.adaptive-site-contents').addClass('required-plugins-count-' + $('.adaptive-sites-import-plugins .required-plugins-list > li').length);

			if ($(this).is(':checked')) {

				$('#adaptive-sites-tooltip-theme-activation').hide();
				$('.adaptive-site-contents .adaptive-theme-module').show();
				$(this).parents('.adaptive-site-contents').removeClass('dont-use-adaptive-theme');

				adaptiveSitesVars.requiredPlugins = plugins;

			} else {
				$(this).parents('.adaptive-site-contents').addClass('dont-use-adaptive-theme');
				$('#adaptive-sites-tooltip-theme-activation').show();
				$('.adaptive-site-contents .adaptive-theme-module').hide();

				var new_plugins = [];
				for (plugin_group in plugins) {
					var temp = [];
					for (key in plugins[plugin_group]) {
						if (!adaptive_dependent_plugins.includes(plugins[plugin_group][key].slug)) {
							temp.push(plugins[plugin_group][key]);
						}
					}
					new_plugins[plugin_group] = temp;
				}

				adaptiveSitesVars.requiredPlugins = new_plugins;
			}
		},

		_backup_before_rest_options: function () {
			AdaptiveSitesAdmin._backupOptions('adaptive-sites-backup-settings-before-reset-done');
			AdaptiveSitesAdmin.backup_taken = true;
		},

		_recheck_backup_options: function () {
			AdaptiveSitesAdmin._backupOptions('adaptive-sites-backup-settings-done');
			AdaptiveSitesAdmin.backup_taken = true;
		},

		_backupOptions: function (trigger_name) {

			// Customizer backup is already taken then return.
			if (AdaptiveSitesAdmin.backup_taken) {
				$(document).trigger(trigger_name);
			} else {

				$.ajax({
					url: adaptiveSitesVars.ajaxurl,
					type: 'POST',
					data: {
						action: 'adaptive-sites-backup-settings',
						_ajax_nonce: adaptiveSitesVars._ajax_nonce,
					},
					beforeSend: function () {
						console.groupCollapsed('正在处理自定义设置备份');
						AdaptiveSitesAdmin._log_title('正在处理自定义设置备份..');
					},
				})
					.fail(function (jqXHR) {
						AdaptiveSitesAdmin._log(jqXHR);
						AdaptiveSitesAdmin._importFailMessage(jqXHR.status + ' ' + jqXHR.statusText, '备份自定义设置失败!', jqXHR);
						console.groupEnd();
					})
					.done(function (data) {
						AdaptiveSitesAdmin._log(data);

						// 1. Pass - Import Customizer Options.
						AdaptiveSitesAdmin._log_title('自定义设置备份完成..');

						console.groupEnd();
						// Custom trigger.
						$(document).trigger(trigger_name);
					});
			}

		},

		/**
		 * 5. 导入完成.
		 */
		_importEnd: function (event) {

			$.ajax({
				url: adaptiveSitesVars.ajaxurl,
				type: 'POST',
				dataType: 'json',
				data: {
					action: 'adaptive-sites-import-end',
					_ajax_nonce: adaptiveSitesVars._ajax_nonce,
				},
				beforeSend: function () {
					console.groupCollapsed('导入完成！');
					AdaptiveSitesAdmin._log_title('导入完成！');
				}
			})
				.fail(function (jqXHR) {
					AdaptiveSitesAdmin._log(jqXHR);
					AdaptiveSitesAdmin._importFailMessage(jqXHR.status + ' ' + jqXHR.statusText, '导入失败!', jqXHR);
					console.groupEnd();
				})
				.done(function (response) {
					AdaptiveSitesAdmin._log(response);
					console.groupEnd();

					// 5. Fail - 导入完成.
					if (false === response.success) {
						AdaptiveSitesAdmin._importFailMessage(response.data, '导入失败!');
					} else {
						AdaptiveSitesAdmin.site_import_status = true;
						AdaptiveSitesAdmin.import_complete();
					}
				});
		},

		page_import_complete: function () {
			if (false === AdaptiveSitesAdmin.subscribe_status) {
				return;
			}

			$('body').removeClass('importing-site');
			$('.rotating, .current-importing-status-wrap,.notice-warning').remove();
			var template = gc.template('adaptive-sites-page-import-success');
			$('.adaptive-sites-result-preview .inner').html(template(AdaptiveSitesAdmin.imported_page_data));

			AdaptiveSitesAdmin.page_import_status = false;
			AdaptiveSitesAdmin.subscribe_status = false;
		},

		import_complete: function () {

			if (false === AdaptiveSitesAdmin.subscribe_status) {
				return;
			}

			$('body').removeClass('importing-site');

			var template = gc.template('adaptive-sites-site-import-success');
			$('.adaptive-sites-result-preview .inner').html(template());

			$('.rotating,.current-importing-status-wrap,.notice-warning').remove();
			$('.adaptive-sites-result-preview').addClass('adaptive-sites-result-preview');

			// 5. Pass - 导入完成.
			AdaptiveSitesAdmin._importSuccessButton();

			AdaptiveSitesAdmin.site_import_status = false;
			AdaptiveSitesAdmin.subscribe_status = false;
			if (!AdaptiveSitesAdmin.first_import_complete) {
				AdaptiveSitesAdmin.first_import_complete = 'yes';
			}
		},

		/**
		 * 4. Import Widgets.
		 */
		_importWidgets: function (event) {
			if (AdaptiveSitesAdmin._is_process_widgets()) {
				$.ajax({
					url: adaptiveSitesVars.ajaxurl,
					type: 'POST',
					dataType: 'json',
					data: {
						action: 'adaptive-sites-import-widgets',
						widgets_data: AdaptiveSitesAdmin.widgets_data,
						_ajax_nonce: adaptiveSitesVars._ajax_nonce,
					},
					beforeSend: function () {
						console.groupCollapsed('Importing Widgets');
						AdaptiveSitesAdmin._log_title('导入小工具..');
					},
				})
					.fail(function (jqXHR) {
						AdaptiveSitesAdmin._log(jqXHR);
						AdaptiveSitesAdmin._failed( jqXHR.status + ' ' + jqXHR.statusText, 'Import Widgets Failed!' );
						console.groupEnd();
					})
					.done(function (response) {
						AdaptiveSitesAdmin._log(response);
						console.groupEnd();

						// 4. Fail - Import Widgets.
						if (false === response.success) {
							AdaptiveSitesAdmin._failed( response.data, 'Import Widgets Failed!' );
						} else {

							// 4. Pass - Import Widgets.
							$(document).trigger('adaptive-sites-import-widgets-done');
						}
					});
			} else {
				$(document).trigger('adaptive-sites-import-widgets-done');
			}
		},

		/**
		 * 3. Import Site Options.
		 */
		import_siteOptions: function (event) {

			if (AdaptiveSitesAdmin._is_process_xml()) {
				$.ajax({
					url: adaptiveSitesVars.ajaxurl,
					type: 'POST',
					dataType: 'json',
					data: {
						action: 'adaptive-sites-import-options',
						_ajax_nonce: adaptiveSitesVars._ajax_nonce,
					},
					beforeSend: function () {
						console.groupCollapsed('Importing Options');
						AdaptiveSitesAdmin._log_title('Importing Options..');
						$('.adaptive-demo-import .percent').html('');
					},
				})
					.fail(function (jqXHR) {
						AdaptiveSitesAdmin._log(jqXHR);
						AdaptiveSitesAdmin._failed( jqXHR.status + ' ' + jqXHR.statusText, 'Import Site Options Failed!' );
						console.groupEnd();
					})
					.done(function (response) {
						AdaptiveSitesAdmin._log(response);
						// 3. Fail - Import Site Options.
						if (false === response.success) {
							AdaptiveSitesAdmin._failed( response.data, 'Import Site Options Failed!' );
							console.groupEnd();
						} else {
							console.groupEnd();

							// 3. Pass - Import Site Options.
							$(document).trigger('adaptive-sites-import-options-done');
						}
					});
			} else {
				$(document).trigger('adaptive-sites-import-options-done');
			}
		},

		/**
		 * 2. Prepare XML Data.
		 */
		_importXML: function () {

			if (AdaptiveSitesAdmin._is_process_xml()) {
				$.ajax({
					url: adaptiveSitesVars.ajaxurl,
					type: 'POST',
					dataType: 'json',
					data: {
						action: 'adaptive-sites-import-prepare-xml',
						wxr_url: AdaptiveSitesAdmin.wxr_url,
						_ajax_nonce: adaptiveSitesVars._ajax_nonce,
					},
					beforeSend: function () {
						console.groupCollapsed('正在导入内容');
						AdaptiveSitesAdmin._log_title('正在导入内容..');
						AdaptiveSitesAdmin._log(AdaptiveSitesAdmin.wxr_url);
						$('.adaptive-site-import-process-wrap').show();
					},
				})
					.fail(function (jqXHR) {
						AdaptiveSitesAdmin._log(jqXHR);
						AdaptiveSitesAdmin._importFailMessage(jqXHR.status + ' ' + jqXHR.statusText, '准备导入XML失败!', jqXHR);
						console.groupEnd();
					})
					.done(function (response) {

						AdaptiveSitesAdmin._log(response);

						// 2. Fail - Prepare XML Data.
						if (false === response.success) {
							var error_msg = response.data.error || response.data;

							AdaptiveSitesAdmin._importFailMessage(adaptiveSitesVars.xmlRequiredFilesMissing);

							console.groupEnd();
						} else {

							var xml_processing = $('.adaptive-demo-import').attr('data-xml-processing');

							if ('yes' === xml_processing) {
								return;
							}

							$('.adaptive-demo-import').attr('data-xml-processing', 'yes');

							// 2. Pass - Prepare XML Data.

							// Import XML though Event Source.
							AdaptiveSSEImport.data = response.data;
							AdaptiveSSEImport.render();

							$('.current-importing-status-description').html('').show();

							$('.current-importing-status-wrap').append('<div class="adaptive-site-import-process-wrap"><progress class="adaptive-site-import-process" max="100" value="0"></progress></div>');

							var evtSource = new EventSource(AdaptiveSSEImport.data.url);
							evtSource.onmessage = function (message) {
								var data = JSON.parse(message.data);
								switch (data.action) {
									case 'updateDelta':

										AdaptiveSSEImport.updateDelta(data.type, data.delta);
										break;

									case 'complete':
										if ( false == data.error ) {
											evtSource.close();

											$('.current-importing-status-description').hide();
											$('.adaptive-demo-import').removeAttr('data-xml-processing');

											document.getElementsByClassName("adaptive-site-import-process").value = '100';

											$('.adaptive-site-import-process-wrap').hide();
											console.groupEnd();

											$(document).trigger('adaptive-sites-import-xml-done');
										} else {
											evtSource.close();
											AdaptiveSitesAdmin._importFailMessage(
												adaptiveSitesVars.xml_import_interrupted_error,
												'Import Process Interrupted!',
												'',
												'<p>' + adaptiveSitesVars.xml_import_interrupted_primary + '</p>',
												'<p>' + adaptiveSitesVars.xml_import_interrupted_secondary + '</p>'
											);
										}

										break;
								}
							};
							evtSource.onerror = function (error) {
								evtSource.close();
								console.log(error);
								AdaptiveSitesAdmin._importFailMessage('', '导入过程中断');
							};
							evtSource.addEventListener('log', function (message) {
								var data = JSON.parse(message.data);
								var message = data.message || '';
								if (message && 'info' === data.level) {
									message = message.replace(/"/g, function (letter) {
										return '';
									});
									$('.current-importing-status-description').html(message);
								}
								AdaptiveSitesAdmin._log(message, data.level);
							});
						}
					});
			} else {
				$(document).trigger('adaptive-sites-import-xml-done');
			}
		},

		_is_reset_data: function () {
			if ($('.adaptive-sites-reset-data').find('.checkbox').is(':checked')) {
				return true;
			}
			return false;
		},

		_is_process_xml: function () {
			if ($('.adaptive-sites-import-xml').find('.checkbox').is(':checked')) {
				return true;
			}
			return false;
		},

		_is_process_customizer: function () {
			var theme_status = $('.adaptive-sites-theme-activation .checkbox').length ? $('.adaptive-sites-theme-activation .checkbox').is(':checked') : true;
			var customizer_status = $('.adaptive-sites-import-customizer').find('.checkbox').is(':checked');

			if (theme_status && customizer_status) {
				return true;
			}
			return false;
		},

		_is_process_widgets: function () {
			if ($('.adaptive-sites-import-widgets').find('.checkbox').is(':checked')) {
				return true;
			}
			return false;
		},

		_startImportCartFlows: function (event) {
			if (AdaptiveSitesAdmin._is_process_xml() && '' !== AdaptiveSitesAdmin.cartflows_url) {

				$.ajax({
					url: adaptiveSitesVars.ajaxurl,
					type: 'POST',
					dataType: 'json',
					data: {
						action: 'adaptive-sites-import-cartflows',
						cartflows_url: AdaptiveSitesAdmin.cartflows_url,
						_ajax_nonce: adaptiveSitesVars._ajax_nonce,
					},
					beforeSend: function () {
						console.groupCollapsed('Importing Flows & Steps');
						AdaptiveSitesAdmin._log_title('Importing Flows & Steps..');
						AdaptiveSitesAdmin._log(AdaptiveSitesAdmin.cartflows_url);
					},
				})
					.fail(function (jqXHR) {
						AdaptiveSitesAdmin._log(jqXHR);
						AdaptiveSitesAdmin._failed( jqXHR.status + ' ' + jqXHR.statusText, 'Import Cartflows Flow Failed!' );
						console.groupEnd();
					})
					.done(function (response) {
						AdaptiveSitesAdmin._log(response);

						// 1. Fail - Import GCForms Options.
						if (false === response.success) {
							AdaptiveSitesAdmin._failed( response.data, 'Import Cartflows Flow Failed!' );
							console.groupEnd();
						} else {
							console.groupEnd();
							// 1. Pass - Import Customizer Options.
							$(document).trigger(AdaptiveSitesAdmin.action_slug + '-import-cartflows-done');
						}
					});

			} else {
				$(document).trigger(AdaptiveSitesAdmin.action_slug + '-import-cartflows-done');
			}

		},

		_startImportWPForms: function (event) {

			if (AdaptiveSitesAdmin._is_process_xml() && '' !== AdaptiveSitesAdmin.gcforms_url) {

				$.ajax({
					url: adaptiveSitesVars.ajaxurl,
					type: 'POST',
					dataType: 'json',
					data: {
						action: 'adaptive-sites-import-gcforms',
						gcforms_url: AdaptiveSitesAdmin.gcforms_url,
						_ajax_nonce: adaptiveSitesVars._ajax_nonce,
					},
					beforeSend: function () {
						console.groupCollapsed('正在导入表单');
						AdaptiveSitesAdmin._log_title('正在导入表单..');
						AdaptiveSitesAdmin._log(AdaptiveSitesAdmin.gcforms_url);
					},
				})
					.fail(function (jqXHR) {
						AdaptiveSitesAdmin._log(jqXHR);
						AdaptiveSitesAdmin._failed( jqXHR.status + ' ' + jqXHR.statusText, '导入表单失败' );
						console.groupEnd();
					})
					.done(function (response) {
						AdaptiveSitesAdmin._log(response);

						// 1. Fail - Import GCForms Options.
						if (false === response.success) {
							AdaptiveSitesAdmin._failed( response.data, '导入表单失败' );
							console.groupEnd();
						} else {
							console.groupEnd();
							// 1. Pass - Import Customizer Options.
							$(document).trigger(AdaptiveSitesAdmin.action_slug + '-import-gcforms-done');
						}
					});

			} else {
				$(document).trigger(AdaptiveSitesAdmin.action_slug + '-import-gcforms-done');
			}

		},

		/**
		 * 1. Import Customizer Options.
		 */
		_importCustomizerSettings: function (event) {
			if (AdaptiveSitesAdmin._is_process_customizer()) {
				$.ajax({
					url: adaptiveSitesVars.ajaxurl,
					type: 'POST',
					dataType: 'json',
					data: {
						action: 'adaptive-sites-import-customizer-settings',
						_ajax_nonce: adaptiveSitesVars._ajax_nonce,
					},
					beforeSend: function () {
						console.groupCollapsed('正在导入自定义设置');
						AdaptiveSitesAdmin._log_title('正在导入自定义设置..');
						AdaptiveSitesAdmin._log(JSON.parse(AdaptiveSitesAdmin.customizer_data));
					},
				})
					.fail(function (jqXHR) {
						AdaptiveSitesAdmin._failed( jqXHR.status + ' ' + jqXHR.statusText, '导入自定义设置失败!' );
						AdaptiveSitesAdmin._log(jqXHR);
						console.groupEnd();
					})
					.done(function (response) {
						AdaptiveSitesAdmin._log(response);

						// 1. Fail - Import Customizer Options.
						if (false === response.success) {
							AdaptiveSitesAdmin._failed( response.data, '导入自定义设置失败!' );
							console.groupEnd();
						} else {
							console.groupEnd();
							// 1. Pass - Import Customizer Options.
							$(document).trigger('adaptive-sites-import-customizer-settings-done');
						}
					});
			} else {
				$(document).trigger('adaptive-sites-import-customizer-settings-done');
			}

		},

		/**
		 * Import Success Button.
		 *
		 * @param  {string} data Error message.
		 */
		_importSuccessButton: function () {

			$('.adaptive-demo-import').removeClass('updating-message installing')
				.removeAttr('data-import')
				.addClass('view-site')
				.removeClass('adaptive-demo-import')
				.text(adaptiveSitesVars.strings.viewSite)
				.attr('target', '_blank')
				.append('<i class="dashicons dashicons-external"></i>')
				.attr('href', adaptiveSitesVars.siteURL);
		},

		_failed: function( errMessage, titleMessage ) {

			var link = adaptiveSitesVars.process_failed_secondary;
				link = link.replace( '#DEMO_URL#', AdaptiveSitesAdmin.templateData['adaptive-site-url'] );
				link = link.replace( '#SUBJECT#', encodeURI('AJAX failed: ' + errMessage ) );

			AdaptiveSitesAdmin._importFailMessage( errMessage, titleMessage, '', adaptiveSitesVars.process_failed_primary, link);

		},

		/**
		 * Import Error Button.
		 *
		 * @param  {string} data Error message.
		 */
		_importFailMessage: function (message, heading, jqXHR, topContent, bottomContent) {

			heading = heading || 'The import process interrupted';

			var status_code = '';
			if (jqXHR) {
				status_code = jqXHR.status ? parseInt(jqXHR.status) : '';
			}

			if (200 == status_code && adaptiveSitesVars.debug) {
				var output = adaptiveSitesVars.importFailedMessageDueToDebug;

			} else {
				var output = topContent || adaptiveSitesVars.importFailedMessage;

				if (message) {
					output += '<div class="current-importing-status">Error: ' + message + '</div>';
				}

				output += bottomContent || '';
			}

			$('.adaptive-sites-import-content').html(output);
			$('.adaptive-sites-result-preview .heading h3').html(heading);

			$('.adaptive-demo-import').removeClass('updating-message installing button-primary').addClass('disabled').text('Import Failed!');
		},

		ucwords: function (str) {
			if (!str) {
				return '';
			}

			str = str.toLowerCase().replace(/\b[a-z]/g, function (letter) {
				return letter.toUpperCase();
			});

			str = str.replace(/-/g, function (letter) {
				return ' ';
			});

			return str;
		},

		/**
		 * Install Plugin Success
		 */
		_installPluginSuccess: function (event, response) {

			event.preventDefault();

			console.groupEnd();

			// Reset not installed plugins list.
			var pluginsList = adaptiveSitesVars.requiredPlugins.notinstalled;
			adaptiveSitesVars.requiredPlugins.notinstalled = AdaptiveSitesAdmin._removePluginFromQueue(response.slug, pluginsList);

			// GeChiUI adds "Activate" button after waiting for 1000ms. So we will run our activation after that.
			setTimeout(function () {

				console.groupCollapsed('正在激活插件 "' + response.name + '"');

				AdaptiveSitesAdmin._log_title('正在激活插件 - ' + response.name);
				AdaptiveSitesAdmin._log('正在激活插件 - ' + response.name);

				$.ajax({
					url: adaptiveSitesVars.ajaxurl,
					type: 'POST',
					data: {
						'action': 'adaptive-required-plugin-activate',
						'slug': response.slug,
						'_ajax_nonce': adaptiveSitesVars._ajax_nonce,
					},
				})
					.done(function (result) {
						AdaptiveSitesAdmin._log(result);

						if (result.success) {
							var pluginsList = adaptiveSitesVars.requiredPlugins.inactive;

							AdaptiveSitesAdmin._log_title('已成功激活插件 - ' + response.name);
							AdaptiveSitesAdmin._log('已成功激活插件 - ' + response.name);

							// Reset not installed plugins list.
							adaptiveSitesVars.requiredPlugins.inactive = AdaptiveSitesAdmin._removePluginFromQueue(response.slug, pluginsList);

							// Enable Demo Import Button
							AdaptiveSitesAdmin._enable_demo_import_button();
						}
						console.groupEnd();
					});

			}, 3000);

		},

		/**
		 * Install Theme Success
		 */
		_installThemeSuccess: function (event, response) {

			event.preventDefault();

			console.groupEnd();

			// Reset not installed themes list.
			var themesList = adaptiveSitesVars.requiredThemes.notinstalled;
			adaptiveSitesVars.requiredThemes.notinstalled = AdaptiveSitesAdmin._removeThemeFromQueue(response.slug, themesList);

			// GeChiUI adds "Activate" button after waiting for 1000ms. So we will run our activation after that.
			setTimeout(function () {

				console.groupCollapsed('正在激活主题 "' + response.name + '"');

				AdaptiveSitesAdmin._log_title('正在激活主题 - ' + response.name);
				AdaptiveSitesAdmin._log('正在激活主题 - ' + response.name);

				$.ajax({
					url: adaptiveSitesVars.ajaxurl,
					type: 'POST',
					data: {
						'action': 'adaptive-required-theme-activate',
						'slug': response.slug,
						'_ajax_nonce': adaptiveSitesVars._ajax_nonce,
					},
				})
					.done(function (result) {
						AdaptiveSitesAdmin._log(result);

						if (result.success) {
							var themesList = adaptiveSitesVars.requiredThemes.inactive;

							AdaptiveSitesAdmin._log_title('已成功激活主题 - ' + response.name);
							AdaptiveSitesAdmin._log('已成功激活主题 - ' + response.name);

							// Reset not installed themes list.
							adaptiveSitesVars.requiredThemes.inactive = AdaptiveSitesAdmin._removeThemeFromQueue(response.slug, themesList);

							// Enable Demo Import Button
							AdaptiveSitesAdmin._enable_demo_import_button();
						}
						console.groupEnd();
					});

			}, 3000);

		},

		/**
		 * Plugin Installation Error.
		 */
		_installPluginError: function (event, response) {

			event.preventDefault();

			console.log(event);
			console.log(response);

			$('.adaptive-sites-result-preview .heading h3').text('插件安装失败');
			$('.adaptive-sites-import-content').html('<p>插件 "<b>' + response.name + '</b>" 安装失败。</p>');

			$('.adaptive-demo-import').removeClass('updating-message installing button-primary').addClass('disabled').text('导入失败!');

			gc.updates.queue = [];

			gc.updates.queueChecker();

			console.groupEnd();
		},

		/**
		 * Theme Installation Error.
		 */
		_installThemeError: function (event, response) {

			event.preventDefault();

			console.log(event);
			console.log(response);

			$('.adaptive-sites-result-preview .heading h3').text('主题安装失败');
			$('.adaptive-sites-import-content').html('<p>主题 "<b>' + response.name + '</b>" 安装失败。</p>');

			$('.adaptive-demo-import').removeClass('updating-message installing button-primary').addClass('disabled').text('导入失败!');

			gc.updates.queue = [];

			gc.updates.queueChecker();

			console.groupEnd();
		},

		/**
		 * 执行安装插件
		 */
		_pluginInstalling: function (event, args) {
			event.preventDefault();

			console.groupCollapsed('正在安装插件 "' + args.name + '"');

			AdaptiveSitesAdmin._log_title('正在安装插件 - ' + args.name);

			console.log(args);
		},

		/**
		 * 执行安装主题
		 */
		_themeInstalling: function (event, args) {
			event.preventDefault();

			console.groupCollapsed('正在安装主题 "' + args.name + '"');

			AdaptiveSitesAdmin._log_title('正在安装主题 - ' + args.name);

			console.log(args);
		},

		/**
		 * Bulk Plugin Active & Install
		 */
		_bulkPluginInstallActivate: function () {

			var not_installed = [];
			var activate_plugins = [];
			if( adaptiveSitesVars.requiredPlugins ) {
				activate_plugins = adaptiveSitesVars.requiredPlugins.inactive || [];
				not_installed = adaptiveSitesVars.requiredPlugins.notinstalled || [];
			}

			// If has class the skip-plugins then,
			// Avoid installing 3rd party plugins.
			if ($('.adaptive-sites-result-preview').hasClass('skip-plugins')) {
				not_installed = [];
			}

			// First Install Bulk.
			if (not_installed.length > 0) {
				AdaptiveSitesAdmin._installAllPlugins(not_installed);
			}

			// Second Activate Bulk.
			if (activate_plugins.length > 0) {
				AdaptiveSitesAdmin._activateAllPlugins(activate_plugins);
			}

			if (activate_plugins.length <= 0 && not_installed.length <= 0) {
				AdaptiveSitesAdmin._enable_demo_import_button();
			}

		},

		/**
		 * Activate All Plugins.
		 */
		_activateAllPlugins: function (activate_plugins) {

			AdaptiveSitesAdmin.remaining_activate_plugins = activate_plugins.length;

			$.each(activate_plugins, function (index, single_plugin) {

				AdaptiveSitesAjaxQueue.add({
					url: adaptiveSitesVars.ajaxurl,
					type: 'POST',
					data: {
						'action': 'adaptive-required-plugin-activate',
						'init': single_plugin.init,
						'options': AdaptiveSitesAdmin.options_data,
						'enabledExtensions': AdaptiveSitesAdmin.enabled_extensions,
						'_ajax_nonce': adaptiveSitesVars._ajax_nonce,
					},
					beforeSend: function () {
						console.groupCollapsed('正在激活插件 "' + single_plugin.name + '"');
						AdaptiveSitesAdmin._log_title('正在激活插件 "' + single_plugin.name + '"');
					},
					success: function (result) {
						console.log(result);
						console.groupEnd('正在激活插件 "' + single_plugin.name + '"');

						if (result.success) {
							var pluginsList = adaptiveSitesVars.requiredPlugins.inactive;

							// Reset not installed plugins list.
							adaptiveSitesVars.requiredPlugins.inactive = AdaptiveSitesAdmin._removePluginFromQueue(single_plugin.slug, pluginsList);

							// Enable Demo Import Button
							AdaptiveSitesAdmin._enable_demo_import_button();
						}

						AdaptiveSitesAdmin.remaining_activate_plugins -= 1;

						if (0 === AdaptiveSitesAdmin.remaining_activate_plugins) {
							console.groupEnd('激活所需的插件..');
						}
					}
				});
			});
			AdaptiveSitesAjaxQueue.run();
		},

		/**
		 * Install All Plugins.
		 */
		_installAllPlugins: function (not_installed) {

			$.each(not_installed, function (index, single_plugin) {

				// Add each plugin activate request in Ajax queue.
				// @see gc-admin/js/updates.js
				gc.updates.queue.push({
					action: 'install-plugin', // Required action.
					data: {
						slug: single_plugin.slug,
						init: single_plugin.init,
						name: single_plugin.name,
						success: function () {
							$(document).trigger('gc-plugin-install-success', [single_plugin]);
						},
						error: function () {
							$(document).trigger('gc-plugin-install-error', [single_plugin]);
						},
					}
				});
			});

			// Required to set queue.
			gc.updates.queueChecker();
		},

		/**
		 * Bulk Theme Active & Install
		 */
		_bulkThemeInstallActivate: function () {

			var not_installed = [];
			var activate_themes = [];
			if( adaptiveSitesVars.requiredThemes ) {
				activate_themes = adaptiveSitesVars.requiredThemes.inactive || [];
				not_installed = adaptiveSitesVars.requiredThemes.notinstalled || [];
			}

			// If has class the skip-themes then,
			// Avoid installing 3rd party themes.
			if ($('.adaptive-sites-result-preview').hasClass('skip-themes')) {
				not_installed = [];
			}

			// First Install Bulk.
			if (not_installed.length > 0) {
				AdaptiveSitesAdmin._installAllThemes(not_installed);
			}

			// Second Activate Bulk.
			if (activate_themes.length > 0) {
				AdaptiveSitesAdmin._activateAllThemes(activate_themes);
			}

			if (activate_themes.length <= 0 && not_installed.length <= 0) {
				AdaptiveSitesAdmin._enable_demo_import_button();
			}

		},

		/**
		 * Activate All Themes.
		 */
		_activateAllThemes: function (activate_themes) {

			AdaptiveSitesAdmin.remaining_activate_themes = activate_themes.length;

			$.each(activate_themes, function (index, single_theme) {

				AdaptiveSitesAjaxQueue.add({
					url: adaptiveSitesVars.ajaxurl,
					type: 'POST',
					data: {
						'action': 'adaptive-required-theme-activate',
						'slug': single_theme.slug,
						'options': AdaptiveSitesAdmin.options_data,
						'enabledExtensions': AdaptiveSitesAdmin.enabled_extensions,
						'_ajax_nonce': adaptiveSitesVars._ajax_nonce,
					},
					beforeSend: function () {
						console.groupCollapsed('正在激活主题 "' + single_theme.name + '"');
						AdaptiveSitesAdmin._log_title('正在激活主题 "' + single_theme.name + '"');
					},
					success: function (result) {
						console.log(result);
						console.groupEnd('正在激活主题 "' + single_theme.name + '"');

						if (result.success) {
							var themesList = adaptiveSitesVars.requiredThemes.inactive;

							// Reset not installed themes list.
							adaptiveSitesVars.requiredThemes.inactive = AdaptiveSitesAdmin._removeThemeFromQueue(single_theme.slug, themesList);

							// Enable Demo Import Button
							AdaptiveSitesAdmin._enable_demo_import_button();
						}

						AdaptiveSitesAdmin.remaining_activate_themes -= 1;

						if (0 === AdaptiveSitesAdmin.remaining_activate_themes) {
							console.groupEnd('激活所需的主题..');
						}
					}
				});
			});
			AdaptiveSitesAjaxQueue.run();
		},

		/**
		 * Install All Themes.
		 */
		_installAllThemes: function (not_installed) {

			$.each(not_installed, function (index, single_theme) {

				// Add each theme activate request in Ajax queue.
				// @see gc-admin/js/updates.js
				gc.updates.queue.push({
					action: 'install-theme', // Required action.
					data: {
						slug: single_theme.slug,
						init: single_theme.init,
						name: single_theme.name,
						success: function () {
							$(document).trigger('gc-theme-install-success', [single_theme]);
						},
						error: function () {
							$(document).trigger('gc-theme-install-error', [single_theme]);
						},
					}
				});
			});

			// Required to set queue.
			gc.updates.queueChecker();
		},

		_show_get_agency_bundle_notice: function (event) {
			event.preventDefault();
			$('.adaptive-sites-result-preview')
				.removeClass('adaptive-sites-activate-license adaptive-sites-site-import-popup adaptive-sites-page-import-popup')
				.addClass('adaptive-sites-get-agency-bundle')
				.show();

			var template = gc.template('adaptive-sites-pro-site-description');
			var output = '<div class="overlay"></div>';
			output += '<div class="inner"><div class="heading"><h3>Liked This demo?</h3></div><span class="dashicons close dashicons-no-alt"></span><div class="adaptive-sites-import-content">';
			output += '</div></div>';
			$('.adaptive-sites-result-preview').html(output);
			$('.adaptive-sites-import-content').html(template);
		},

		_show_activate_license_notice: function (event) {

			if (event) {
				event.preventDefault();
			}

			if (!AdaptiveSitesAdmin._getParamFromURL('license')) {
				var url_params = {
					'license': 'show'
				};
				AdaptiveSitesAdmin._changeAndSetURL(url_params);
			}

			$('.adaptive-sites-result-preview')
				.removeClass('adaptive-sites-site-import-popup adaptive-sites-skip-templates adaptive-sites-page-import-popup')
				.addClass('adaptive-sites-activate-license')
				.show();

			var template = gc.template('adaptive-sites-activate-license');
			var output = '<div class="overlay"></div>';
			output += '<div class="inner"><div class="heading"><h3>Activate License for Premium Templates</h3></div><span class="dashicons close dashicons-no-alt"></span><div class="adaptive-sites-import-content">';
			output += '</div></div>';
			$('.adaptive-sites-result-preview').html(output);
			$('.adaptive-sites-import-content').html(template);
		},

		_show_invalid_mini_agency_license: function (event) {
			event.preventDefault();
			$('.adaptive-sites-result-preview')
				.removeClass('adaptive-sites-activate-license adaptive-sites-site-import-popup adaptive-sites-skip-templates adaptive-sites-page-import-popup')
				.addClass('adaptive-sites-invalid-mini-agency-license')
				.show();

			var template = gc.template('adaptive-sites-invalid-mini-agency-license');
			var output = '<div class="overlay"></div>';
			output += '<div class="inner"><div class="heading"><h3>Not Valid License</h3></div><span class="dashicons close dashicons-no-alt"></span><div class="adaptive-sites-import-content">';
			output += '</div></div>';
			$('.adaptive-sites-result-preview').html(output);
			$('.adaptive-sites-import-content').html(template);
		},

		_get_id: function (site_id) {
			return site_id.replace('id-', '');
		},

		/**
		 * Fires when a nav item is clicked.
		 *
		 * @since 1.0
		 * @access private
		 * @method _show_site_popup
		 */
		_show_site_popup: function (event) {
			event.preventDefault();

			if ($(this).hasClass('updating-message')) {
				return;
			}

			var delay = 1;
			var retry_str = '';
			if ( AdaptiveSitesAdmin.delay_in_request ) {
				delay = AdaptiveSitesAdmin.delay_value;
				retry_str = '<p class="ast-retry-text">Retrying in <span class="ast-retry-sec">10</span>...</p>';

				var timeleft = AdaptiveSitesAdmin.delay_value / 1000;
				var countdown = setInterval(function() {
					timeleft -= 1;
					$(".ast-retry-sec").html( timeleft );
					if ( timeleft <= 0 ) {
						clearInterval(countdown);
						$(".ast-retry-sec").html( "0" );
					}
				}, 1000);
			}

			$('.adaptive-sites-result-preview').addClass('import-site').removeClass('import-page');

			$('.adaptive-sites-result-preview')
				.removeClass('adaptive-sites-get-agency-bundle preview-page-from-search-result adaptive-sites-page-import-popup adaptive-sites-activate-license')
				.addClass('adaptive-sites-site-import-popup')
				.show();

			var template = gc.template('adaptive-sites-result-preview');
			$('.adaptive-sites-result-preview').html(template('adaptive-sites')).addClass('preparing');
			$('.adaptive-sites-import-content').append('<div class="adaptive-loading-wrap"><div class="adaptive-loading-icon"></div></div>'+retry_str);

			// .attr('data-slug', 'adaptive-sites');
			AdaptiveSitesAdmin.action_slug = 'adaptive-sites';
			adaptiveSitesVars.cpt_slug = 'adaptive-sites';

			var site_id = $('#site-pages').attr('data-site-id') || '';
			site_id = AdaptiveSitesAdmin._get_id(site_id);

			// AdaptiveSitesAdmin.required_plugins_list_markup();
			if (AdaptiveSitesAdmin.visited_sites_and_pages[site_id]) {

				AdaptiveSitesAdmin.templateData = AdaptiveSitesAdmin.visited_sites_and_pages[site_id];

				AdaptiveSitesAdmin.process_site_data(AdaptiveSitesAdmin.templateData);
			} else {


				// AdaptiveSitesAdmin.templateData
				// Add Params for API request.
				AdaptiveSitesAdmin._api_params = {};

				AdaptiveSitesAdmin._apiAddParam_status();
				AdaptiveSitesAdmin._apiAddParam_adaptive_site_category();
				AdaptiveSitesAdmin._apiAddParam_adaptive_site_page_builder();
				AdaptiveSitesAdmin._apiAddParam_adaptive_page_parent_category();
				AdaptiveSitesAdmin._apiAddParam_site_url();
				AdaptiveSitesAdmin._apiAddParam_purchase_key();
				var api_post = {
					id: adaptiveSitesVars.cpt_slug,
					slug: adaptiveSitesVars.cpt_slug + '/' + site_id + '?' + decodeURIComponent($.param(AdaptiveSitesAdmin._api_params)),
				};

				setTimeout(function() {
					$.ajax({
						url: adaptiveSitesVars.ajaxurl,
						type: 'POST',
						data: {
							action: 'adaptive-sites-api-request',
							url: adaptiveSitesVars.cpt_slug + '/' + site_id + '?' + decodeURIComponent($.param(AdaptiveSitesAdmin._api_params)),
						},
						beforeSend: function () {
							console.groupCollapsed('Requesting API');
						}
					})
					.fail(function (jqXHR) {
						AdaptiveSitesAdmin._log(jqXHR);
						AdaptiveSitesAdmin._importFailMessage( jqXHR.status + ' ' + jqXHR.statusText, '', jqXHR, adaptiveSitesVars.ajax_request_failed_primary, adaptiveSitesVars.ajax_request_failed_secondary );
						console.groupEnd();
						AdaptiveSitesAdmin.delay_in_request = false;
					})
					.done(function (response) {
						console.log('Template API Response:');
						AdaptiveSitesAdmin.delay_in_request = false;
						AdaptiveSitesAdmin._log(response);
						console.groupEnd();
						if (response.success) {
							AdaptiveSitesAdmin.visited_sites_and_pages[response.data.id] = response.data;

							AdaptiveSitesAdmin.templateData = response.data;

							AdaptiveSitesAdmin.process_site_data(AdaptiveSitesAdmin.templateData);
						} else {
							AdaptiveSitesAdmin.handle_error( response, site_id );
						}
					});

				}, delay);

			}

		},

		handle_error: function( response, id ) {
			var template = 'adaptive-sites-request-failed-user';
			var template_data = {
				'primary' : '',
				'secondary' : '',
				'error' : response.data,
				'id' : id
			};
			if ( undefined !== response.data.code ) {
				var code = response.data.code.toString();
				switch( code ) {
					case '401':
					case '404':
					case '500':
						template_data.primary = adaptiveSitesVars.server_import_primary_error;
						break;

					case 'GC_Error':
						template_data.primary = adaptiveSitesVars.client_import_primary_error;
						break;

					case 'Cloudflare':
						template_data.primary = adaptiveSitesVars.cloudflare_import_primary_error;
						break;

					default:
						template = 'adaptive-sites-request-failed';
						break;
				}
			}
			let err_template = gc.template( template );
			$('.adaptive-sites-result-preview .heading > h3').text('Import Process Interrupted');
			$('.adaptive-sites-import-content').find('.adaptive-loading-wrap').remove();
			$('.adaptive-sites-result-preview').removeClass('preparing');
			$('.adaptive-sites-import-content').html( err_template( template_data ) );
			$('.adaptive-demo-import').removeClass('updating-message installing button-primary').addClass('disabled').text('Import Failed!');
		},

		show_popup: function (heading, content, actions, classes) {
			if (classes) {
				$('.adaptive-sites-popup').addClass(classes);
			}
			if (heading) {
				$('.adaptive-sites-popup .heading h3').html(heading);
			}
			if (content) {
				$('.adaptive-sites-popup .adaptive-sites-import-content').html(content);
			}
			if (actions) {
				$('.adaptive-sites-popup .ast-actioms-wrap').html(actions);
			}

			$('.adaptive-sites-popup').show();
		},

		hide_popup: function () {
			$('.adaptive-sites-popup').hide();
		},

		show_page_popup: function () {

			AdaptiveSitesAdmin.process_import_page();
		},

		process_import_page: function () {
			AdaptiveSitesAdmin.hide_popup();

			var page_id = AdaptiveSitesAdmin._get_id($('#single-pages').find('.current_page').attr('data-page-id')) || '';
			var site_id = AdaptiveSitesAdmin._get_id($('#site-pages').attr('data-site-id')) || '';

			var delay = 1;
			var retry_str = '';
			if ( AdaptiveSitesAdmin.delay_in_request ) {
				delay = AdaptiveSitesAdmin.delay_value;
				retry_str = '<p class="ast-retry-text">Retrying in <span class="ast-retry-sec">10</span>...</p>';

				var timeleft = AdaptiveSitesAdmin.delay_value / 1000;
				var countdown = setInterval(function() {
					timeleft -= 1;
					$(".ast-retry-sec").html( timeleft );
					if ( timeleft <= 0 ) {
						clearInterval(countdown);
						$(".ast-retry-sec").html( "0" );
					}
				}, 1000);
			}

			$('.adaptive-sites-result-preview')
				.removeClass('adaptive-sites-subscription-popup adaptive-sites-activate-license adaptive-sites-get-agency-bundle adaptive-sites-site-import-popup adaptive-sites-page-import-popup')
				.addClass('preview-page-from-search-result')
				.show();

			$('.adaptive-sites-result-preview').html(gc.template('adaptive-sites-result-preview')).addClass('preparing');
			$('.adaptive-sites-import-content').append('<div class="adaptive-loading-wrap"><div class="adaptive-loading-icon"></div></div>' + retry_str);

			AdaptiveSitesAdmin.action_slug = 'site-pages';
			adaptiveSitesVars.cpt_slug = 'site-pages';

			if (AdaptiveSitesAdmin.visited_sites_and_pages[page_id]) {

				AdaptiveSitesAdmin.templateData = AdaptiveSitesAdmin.visited_sites_and_pages[page_id];

				AdaptiveSitesAdmin.required_plugins_list_markup(AdaptiveSitesAdmin.templateData['site-pages-required-plugins']);
			} else {

				// AdaptiveSitesAdmin.templateData
				// Add Params for API request.
				AdaptiveSitesAdmin._api_params = {};

				AdaptiveSitesAdmin._apiAddParam_status();
				AdaptiveSitesAdmin._apiAddParam_per_page();
				AdaptiveSitesAdmin._apiAddParam_adaptive_site_category();
				AdaptiveSitesAdmin._apiAddParam_adaptive_site_page_builder();
				AdaptiveSitesAdmin._apiAddParam_adaptive_page_parent_category();
				AdaptiveSitesAdmin._apiAddParam_site_url();
				AdaptiveSitesAdmin._apiAddParam_purchase_key();

				setTimeout( function() {
					// Request.
					$.ajax({
						url: adaptiveSitesVars.ajaxurl,
						type: 'POST',
						data: {
							action: 'adaptive-sites-api-request',
							url: adaptiveSitesVars.cpt_slug + '/' + page_id + '?' + decodeURIComponent($.param(AdaptiveSitesAdmin._api_params)),
						},
						beforeSend: function () {
							console.groupCollapsed('Requesting API URL');
							AdaptiveSitesAdmin._log('Requesting API URL');
						}
					})
					.fail(function (jqXHR) {
						AdaptiveSitesAdmin._log(jqXHR);
						AdaptiveSitesAdmin._importFailMessage( jqXHR.status + ' ' + jqXHR.statusText, 'Page Import API Request Failed!', jqXHR, adaptiveSitesVars.ajax_request_failed_primary, adaptiveSitesVars.ajax_request_failed_secondary );
						console.groupEnd();
						AdaptiveSitesAdmin.delay_in_request = false;
					})
					.done(function (response) {
						AdaptiveSitesAdmin._log(response);
						console.groupEnd();
						AdaptiveSitesAdmin.delay_in_request = false;

						if (response.success) {
							AdaptiveSitesAdmin.visited_sites_and_pages[response.data.id] = response.data;

							AdaptiveSitesAdmin.templateData = response.data;

							AdaptiveSitesAdmin.required_plugins_list_markup(AdaptiveSitesAdmin.templateData['site-pages-required-plugins']);
						} else {
							AdaptiveSitesAdmin.handle_error( response, page_id );
						}
					});
				}, delay );
			}
		},

		show_page_popup_from_search: function (event) {
			event.preventDefault();
			var page_id = $(this).parents('.adaptive-theme').attr('data-page-id') || '';
			var site_id = $(this).parents('.adaptive-theme').attr('data-site-id') || '';

			// $('.adaptive-sites-result-preview').show();
			$('#adaptive-sites').hide();
			$('#site-pages').hide();
			AdaptiveSitesAdmin.show_pages_by_site_id(site_id, page_id);
		},

		/**
		 * Fires when a nav item is clicked.
		 *
		 * @since 1.0
		 * @access private
		 * @method show_page_popup
		 */
		show_page_popup_from_sites: function (event) {
			event.preventDefault();

			if ($(this).hasClass('updating-message')) {
				return;
			}

			$('.adaptive-sites-result-preview').addClass('import-page').removeClass('import-site');

			AdaptiveSitesAdmin.show_page_popup();
		},

		// Returns if a value is an array
		_isArray: function (value) {
			return value && typeof value === 'object' && value.constructor === Array;
		},

		add_skip_and_import_popups: function (templates) {
			if (Object.keys(templates).length) {
				for (template_id in templates) {
					var template = gc.template(template_id);
					var template_data = templates[template_id] || '';
					$('.adaptive-sites-result-preview .inner').append(template(template_data));
				}
				$('.adaptive-sites-result-preview .inner > .default').addClass('hide-me');
				$('.adaptive-sites-result-preview .inner > .skip-and-import:not(:last-child)').addClass('hide-me');
			}
		},

		start_plugin_import: function( response ) {

			if (AdaptiveSitesAdmin.subscribe_skiped || AdaptiveSitesAdmin.subscription_form_submitted == 'yes') {
				$('.user-building-for-title').hide();
				$('.adaptive-sites-advanced-options-heading').hide();
				$('.adaptive-sites-advanced-options').show();
				$('#adaptive-sites-subscription-form-one').hide();
			}

			if (false === AdaptiveSitesAdmin.subscribe_skiped && $('.subscription-enabled').length && AdaptiveSitesAdmin.subscription_form_submitted !== 'yes') {
				$('.adaptive-sites-result-preview .heading h3').html(adaptiveSitesVars.headings.subscription);
				$('.site-import-cancel').hide();

				if ('site-pages' === AdaptiveSitesAdmin.action_slug) {
					$('#adaptive-sites-subscription-form-two').html(gc.template('adaptive-sites-subscription-form-one'));
					$('#adaptive-sites-subscription-form-two').append(gc.template('adaptive-sites-subscription-form-two'));
				} else {
					$('#adaptive-sites-subscription-form-one').html(gc.template('adaptive-sites-subscription-form-one'));
					$('#adaptive-sites-subscription-form-two').html(gc.template('adaptive-sites-subscription-form-two'));
				}
			}

			// Set compatibilities.
			AdaptiveSitesAdmin.skip_and_import_popups = [];
			var compatibilities = adaptiveSitesVars.compatibilities;
			required_plugins = [];
			if( response ) {
				required_plugins = response.data['required_plugins'];
				AdaptiveSitesAdmin.required_plugins = response.data['required_plugins'];

				if (response.data['third_party_required_plugins'].length) {
					AdaptiveSitesAdmin.skip_and_import_popups['adaptive-sites-third-party-required-plugins'] = response.data['third_party_required_plugins'];
				}
			}

			var is_dynamic_page = $('#single-pages').find('.current_page').attr('data-dynamic-page') || 'no';

			if (('yes' === is_dynamic_page) && 'site-pages' === AdaptiveSitesAdmin.action_slug) {
				AdaptiveSitesAdmin.skip_and_import_popups['adaptive-sites-dynamic-page'] = '';
			}

			// Release disabled class from import button.
			$('.adaptive-demo-import')
				.removeClass('disabled not-click-able')
				.attr('data-import', 'disabled');

			// Remove loader.
			$('.required-plugins').removeClass('loading').html('');
			$('.required-plugins-list').html('');

			var output = '';

			/**
			 * Count remaining plugins.
			 * @type number
			 */
			var remaining_plugins = 0;
			var required_plugins_markup = '';

			/**
			 * Not Installed
			 *
			 * List of not installed required plugins.
			 */
			if ( required_plugins && typeof required_plugins.notinstalled !== 'undefined') {

				// Add not have installed plugins count.
				remaining_plugins += parseInt(required_plugins.notinstalled.length);

				$(required_plugins.notinstalled).each(function (index, plugin) {
					output += '<li class="plugin-card plugin-card-' + plugin.slug + '" data-slug="' + plugin.slug + '" data-init="' + plugin.init + '" data-name="' + plugin.name + '">' + plugin.name + '</li>';
				});
			}

			/**
			 * Inactive
			 *
			 * List of not inactive required plugins.
			 */
			if ( required_plugins && typeof required_plugins.inactive !== 'undefined') {

				// Add inactive plugins count.
				remaining_plugins += parseInt(required_plugins.inactive.length);

				$(required_plugins.inactive).each(function (index, plugin) {
					output += '<li class="plugin-card plugin-card-' + plugin.slug + '" data-slug="' + plugin.slug + '" data-init="' + plugin.init + '" data-name="' + plugin.name + '">' + plugin.name + '</li>';
				});
			}

			if ('' == output) {
				$('.adaptive-sites-result-preview').find('.adaptive-sites-import-plugins').hide();
			} else {
				$('.adaptive-sites-result-preview').find('.adaptive-sites-import-plugins').show();
				$('.adaptive-sites-result-preview').find('.required-plugins-list').html(output);
			}
			//启用 "删除以前导入的站点" 功能
			if ('yes' === AdaptiveSitesAdmin.first_import_complete && !$('.adaptive-sites-result-preview').hasClass('import-page')) {
				$('.adaptive-sites-advanced-options').find('.adaptive-site-contents').prepend(gc.template('adaptive-sites-delete-previous-site'));
			}

			/**
			 * Enable Demo Import Button
			 * @type number
			 */
			adaptiveSitesVars.requiredPlugins = required_plugins;

			$('.adaptive-sites-import-content').find('.adaptive-loading-wrap').remove();
			$('.adaptive-sites-result-preview').removeClass('preparing');

			// Compatibility.
			if (Object.keys(compatibilities.errors).length || Object.keys(compatibilities.warnings).length || Object.keys(AdaptiveSitesAdmin.skip_and_import_popups).length) {

				if (Object.keys(compatibilities.errors).length || Object.keys(compatibilities.warnings).length) {
					AdaptiveSitesAdmin.skip_and_import_popups['adaptive-sites-compatibility-messages'] = compatibilities;
				}

				if (Object.keys(AdaptiveSitesAdmin.skip_and_import_popups).length) {
					AdaptiveSitesAdmin.add_skip_and_import_popups(AdaptiveSitesAdmin.skip_and_import_popups);
				}

			} else {

				// Avoid plugin activation, for pages only.
				if ('site-pages' === AdaptiveSitesAdmin.action_slug) {

					var notinstalled = [];
					if( adaptiveSitesVars && adaptiveSitesVars.requiredPlugins && adaptiveSitesVars.requiredPlugins.notinstalled ) {
						notinstalled = adaptiveSitesVars.requiredPlugins.notinstalled;
					}

					if (!notinstalled.length) {
						AdaptiveSitesAdmin.import_page_process();
					}
				}
			}
		},

		start_theme_import: function( response ) {

			// if (AdaptiveSitesAdmin.subscribe_skiped || AdaptiveSitesAdmin.subscription_form_submitted == 'yes') {
			// 	$('.user-building-for-title').hide();
			// 	$('.adaptive-sites-advanced-options-heading').hide();
			// 	$('.adaptive-sites-advanced-options').show();
			// 	$('#adaptive-sites-subscription-form-one').hide();
			// }

			// if (false === AdaptiveSitesAdmin.subscribe_skiped && $('.subscription-enabled').length && AdaptiveSitesAdmin.subscription_form_submitted !== 'yes') {
			// 	$('.adaptive-sites-result-preview .heading h3').html(adaptiveSitesVars.headings.subscription);
			// 	$('.site-import-cancel').hide();

			// 	if ('site-pages' === AdaptiveSitesAdmin.action_slug) {
			// 		$('#adaptive-sites-subscription-form-two').html(gc.template('adaptive-sites-subscription-form-one'));
			// 		$('#adaptive-sites-subscription-form-two').append(gc.template('adaptive-sites-subscription-form-two'));
			// 	} else {
			// 		$('#adaptive-sites-subscription-form-one').html(gc.template('adaptive-sites-subscription-form-one'));
			// 		$('#adaptive-sites-subscription-form-two').html(gc.template('adaptive-sites-subscription-form-two'));
			// 	}
			// }

			// Set compatibilities.
			AdaptiveSitesAdmin.skip_and_import_popups = [];
			var compatibilities = adaptiveSitesVars.compatibilities;
			required_themes = [];
			if( response ) {
				required_themes = response.data['required_themes'];
				AdaptiveSitesAdmin.required_themes = response.data['required_themes'];
			}

			// var is_dynamic_page = $('#single-pages').find('.current_page').attr('data-dynamic-page') || 'no';

			// if (('yes' === is_dynamic_page) && 'site-pages' === AdaptiveSitesAdmin.action_slug) {
			// 	AdaptiveSitesAdmin.skip_and_import_popups['adaptive-sites-dynamic-page'] = '';
			// }

			// Release disabled class from import button.
			// $('.adaptive-demo-import')
			// 	.removeClass('disabled not-click-able')
			// 	.attr('data-import', 'disabled');

			// Remove loader.
			$('.required-themes').removeClass('loading').html('');
			$('.required-themes-list').html('');

			var output = '';

			/**
			 * Count remaining themes.
			 * @type number
			 */
			var remaining_themes = 0;
			var required_themes_markup = '';

			/**
			 * Not Installed
			 *
			 * List of not installed required themes.
			 */
			if ( required_themes && typeof required_themes.notinstalled !== 'undefined') {

				// Add not have installed themes count.
				remaining_themes += parseInt(required_themes.notinstalled.length);

				$(required_themes.notinstalled).each(function (index, theme) {
					output += '<li class="theme-card theme-card-' + theme.slug + '" data-slug="' + theme.slug + '" data-init="' + theme.init + '" data-name="' + theme.name + '">' + theme.name + '</li>';
				});
			}

			/**
			 * Inactive
			 *
			 * List of not inactive required themes.
			 */
			if ( required_themes && typeof required_themes.inactive !== 'undefined') {

				// Add inactive themes count.
				remaining_themes += parseInt(required_themes.inactive.length);

				$(required_themes.inactive).each(function (index, theme) {
					output += '<li class="theme-card theme-card-' + theme.slug + '" data-slug="' + theme.slug + '" data-init="' + theme.init + '" data-name="' + theme.name + '">' + theme.name + '</li>';
				});
			}

			if ('' == output) {
				$('.adaptive-sites-result-preview').find('.adaptive-sites-import-themes').hide();
			} else {
				$('.adaptive-sites-result-preview').find('.adaptive-sites-import-themes').show();
				$('.adaptive-sites-result-preview').find('.required-themes-list').html(output);
			}
			// if ('yes' === AdaptiveSitesAdmin.first_import_complete && !$('.adaptive-sites-result-preview').hasClass('import-page')) {
			// 	$('.adaptive-sites-advanced-options').find('.adaptive-site-contents').prepend(gc.template('adaptive-sites-delete-previous-site'));
			// }

			/**
			 * Enable Demo Import Button
			 * @type number
			 */
			adaptiveSitesVars.requiredThemes = required_themes;

			// $('.adaptive-sites-import-content').find('.adaptive-loading-wrap').remove();
			// $('.adaptive-sites-result-preview').removeClass('preparing');

			// // Compatibility.
			// if (Object.keys(compatibilities.errors).length || Object.keys(compatibilities.warnings).length || Object.keys(AdaptiveSitesAdmin.skip_and_import_popups).length) {

			// 	if (Object.keys(compatibilities.errors).length || Object.keys(compatibilities.warnings).length) {
			// 		AdaptiveSitesAdmin.skip_and_import_popups['adaptive-sites-compatibility-messages'] = compatibilities;
			// 	}

			// 	if (Object.keys(AdaptiveSitesAdmin.skip_and_import_popups).length) {
			// 		AdaptiveSitesAdmin.add_skip_and_import_popups(AdaptiveSitesAdmin.skip_and_import_popups);
			// 	}

			// } else {

			// 	// Avoid theme activation, for pages only.
			// 	if ('site-pages' === AdaptiveSitesAdmin.action_slug) {

			// 		var notinstalled = [];
			// 		if( adaptiveSitesVars && adaptiveSitesVars.requiredThemes && adaptiveSitesVars.requiredThemes.notinstalled ) {
			// 			notinstalled = adaptiveSitesVars.requiredThemes.notinstalled;
			// 		}

			// 		if (!notinstalled.length) {
			// 			AdaptiveSitesAdmin.import_page_process();
			// 		}
			// 	}
			// }
		},

		required_plugins_list_markup: function (requiredPlugins) {

			// Add disabled class from import button.
			$('.adaptive-demo-import')
				.addClass('disabled not-click-able')
				.removeAttr('data-import');

			if( '' === requiredPlugins ) {
				AdaptiveSitesAdmin.start_plugin_import();
			} else {

				$('.required-plugins').addClass('loading').html('<span class="spinner is-active"></span>');

				// Required Required.
				$.ajax({
					url: adaptiveSitesVars.ajaxurl,
					type: 'POST',
					data: {
						action: 'adaptive-required-plugins',
						_ajax_nonce: adaptiveSitesVars._ajax_nonce,
					},
					beforeSend: function () {
						console.groupCollapsed('所需插件');
						console.log('模板所需插件:');
						console.log(requiredPlugins);
					}
				})
				.fail(function (jqXHR) {
					AdaptiveSitesAdmin._log(jqXHR);

					// Remove loader.
					$('.required-plugins').removeClass('loading').html('');
					AdaptiveSitesAdmin._failed( jqXHR.status + ' ' + jqXHR.statusText, '所需插件失败!' );
					console.groupEnd();
				})
				.done(function (response) {
					console.log('网站上必需的插件状态:');
					AdaptiveSitesAdmin._log(response);
					console.groupEnd();

					if (false === response.success) {
						AdaptiveSitesAdmin._failed( response.data, '所需插件失败!' );
					} else {
						AdaptiveSitesAdmin.start_plugin_import( response );
					}
				});
			}
		},

		required_themes_list_markup: function (requiredThemes) {

			// Add disabled class from import button.
			$('.adaptive-demo-import')
				.addClass('disabled not-click-able')
				.removeAttr('data-import');

			if( '' === requiredThemes ) {
				AdaptiveSitesAdmin.start_theme_import();
			} else {

				$('.required-themes').addClass('loading').html('<span class="spinner is-active"></span>');

				// Required Required.
				$.ajax({
					url: adaptiveSitesVars.ajaxurl,
					type: 'POST',
					data: {
						action: 'adaptive-required-themes',
						_ajax_nonce: adaptiveSitesVars._ajax_nonce,
					},
					beforeSend: function () {
						console.groupCollapsed('所需主题');
						console.log('模板所需主题:');
						console.log(requiredThemes);
					}
				})
				.fail(function (jqXHR) {
					AdaptiveSitesAdmin._log(jqXHR);

					// Remove loader.
					$('.required-themes').removeClass('loading').html('');
					AdaptiveSitesAdmin._failed( jqXHR.status + ' ' + jqXHR.statusText, '所需主题失败!' );
					console.groupEnd();
				})
				.done(function (response) {
					console.log('网站上必需的主题状态:');
					AdaptiveSitesAdmin._log(response);
					console.groupEnd();

					if (false === response.success) {
						AdaptiveSitesAdmin._failed( response.data, '所需主题失败!' );
					} else {
						AdaptiveSitesAdmin.start_theme_import( response );
					}
				});
			}
		},

		import_page_process: function () {

			if ($('.adaptive-sites-page-import-popup .site-install-site-button, .preview-page-from-search-result .site-install-site-button').hasClass('updating-message')) {
				return;
			}

			if (false === AdaptiveSitesAdmin.subscribe_skiped && $('.subscription-enabled').length && AdaptiveSitesAdmin.subscription_form_submitted !== 'yes') {
				$('.subscription-popup').show();
				$('.adaptive-sites-result-preview .default').hide();
			} else {
				AdaptiveSitesAdmin.subscribe_status = true;
			}

			$('.adaptive-sites-page-import-popup .site-install-site-button, .preview-page-from-search-result .site-install-site-button').addClass('updating-message installing').text('导入中..');

			AdaptiveSitesAdmin.import_start_time = new Date();

			$('.adaptive-sites-result-preview .inner > h3').text('正在导入您的网站。');
			$('.install-theme-info').hide();
			$('.ast-importing-wrap').show();
			var output = '<div class="current-importing-status-title"></div><div class="current-importing-status-description"></div>';
			$('.current-importing-status').html(output);

			// Process Bulk Plugin Install & Activate.
			AdaptiveSitesAdmin._bulkPluginInstallActivate();
			// Process Bulk Theme Install & Activate.
			AdaptiveSitesAdmin._bulkThemeInstallActivate();
		},

		// _installAdaptive: function (status) {
		// 	var theme_slug = 'adaptive';
		// 	// 获取主题信息
		// 	var requiredThemes = AdaptiveSitesAdmin.templateData['required-themes'];
		// 	if( '' === requiredThemes ) {
		// 		AdaptiveSitesAdmin.start_theme_import();
		// 	} else {
		// 		theme_slug = requiredThemes[0].slug;
		// 	}

		// 	AdaptiveSitesAdmin._log_title(adaptiveSitesVars.log.themeInstall);
		// 	AdaptiveSitesAdmin._log(adaptiveSitesVars.log.themeInstall);

		// 	if (status == 'not-installed') {
		// 		// 未安装
		// 		if (gc.updates.shouldRequestFilesystemCredentials && !gc.updates.ajaxLocked) {
		// 			gc.updates.requestFilesystemCredentials();
		// 		}
		// 		gc.updates.installTheme({
		// 			slug: theme_slug
		// 		});

		// 	} else if (status == 'installed-but-inactive') {
		// 		// 已安装但未激活
		// 		// 激活主题
		// 		AdaptiveSitesAdmin._activateTheme();
		// 	}

		// },

		// _activateTheme: function (event, response) {

		// 	// GeChiUI adds "Activate" button after waiting for 1000ms. So we will run our activation after that.
		// 	setTimeout(function () {

		// 		$.ajax({
		// 			url: adaptiveSitesVars.ajaxurl,
		// 			type: 'POST',
		// 			data: {
		// 				'action': 'adaptive-sites-activate-theme',
		// 				'_ajax_nonce': adaptiveSitesVars._ajax_nonce,
		// 			},
		// 			beforeSend: function () {
		// 				console.log('激活 Adaptive 主题..');
		// 			}
		// 		})
		// 			.done(function (result) {
		// 				AdaptiveSitesAdmin._log(result);
		// 				if (result.success) {
		// 					AdaptiveSitesAdmin._log_title(result.data.message);
		// 					AdaptiveSitesAdmin._log(result.data.message);
		// 				}
		// 			});

		// 	}, 3000);
		// },

		_close_popup_by_overlay: function (event) {
			if (this === event.target) {
				// Import process is started?
				// And Closing the window? Then showing the warning confirm message.
				if ($('body').hasClass('importing-site') && !confirm(adaptiveSitesVars.strings.warningBeforeCloseWindow)) {
					return;
				}

				$('body').removeClass('importing-site');
				$('html').removeClass('adaptive-site-preview-on');

				AdaptiveSitesAdmin._close_popup();
				AdaptiveSitesAdmin.hide_popup();
			}
		},

		/**
		 * Close Popup
		 *
		 * @since 1.0
		 * @access private
		 * @method _importDemo
		 */
		_close_popup: function () {
			AdaptiveSitesAdmin._clean_url_params('adaptive-site');
			AdaptiveSitesAdmin._clean_url_params('adaptive-page');
			AdaptiveSitesAdmin._clean_url_params('license');
			$('.adaptive-sites-result-preview').html('').hide();

			AdaptiveSitesAdmin.hide_popup();
		},

		_page_api_call: function () {

			// Have any skip and import popup in queue then return.
			if (Object.keys(AdaptiveSitesAdmin.skip_and_import_popups).length) {
				return;
			}

			// Has API data of pages.
			if (null == AdaptiveSitesAdmin.templateData) {
				return;
			}

			AdaptiveSitesAdmin.import_gcform(AdaptiveSitesAdmin.templateData['adaptive-site-gcforms-path'], function (form_response) {

				$('body').addClass('importing-site');

				// Import Page Content
				$('.current-importing-status-wrap').remove();
				$('.adaptive-sites-result-preview .inner > h3').text('正在导入页面！');

				fetch(AdaptiveSitesAdmin.templateData['adaptive-page-api-url'] + '?&site_url=' + adaptiveSitesVars.siteURL).then(response => {
					return response.json();
				}).then(data => {

					// 导入单页
					$.ajax({
						url: adaptiveSitesVars.ajaxurl,
						type: 'POST',
						dataType: 'json',
						data: {
							'action': 'adaptive-sites-create-page',
							'_ajax_nonce': adaptiveSitesVars._ajax_nonce,
							'page_settings_flag': AdaptiveSitesAdmin.page_settings_flag,
							'data': data,
						},
						success: function (response) {
							if (response.success) {
								AdaptiveSitesAdmin.page_import_status = true;
								AdaptiveSitesAdmin.imported_page_data = response.data
								AdaptiveSitesAdmin.page_import_complete();
							} else {
								AdaptiveSitesAdmin._importFailMessage(response.data, '页面 Rest API 请求失败!');
							}
						}
					});

				}).catch(err => {
					AdaptiveSitesAdmin._log(err);
					AdaptiveSitesAdmin._importFailMessage(response.data, '页面 Rest API 请求失败!');
				});
			});
		},

		import_gcform: function (gcforms_url, callback) {

			if ('' == gcforms_url) {
				if (callback && typeof callback == "function") {
					callback('');
				}
				return;
			}

			$.ajax({
				url: adaptiveSitesVars.ajaxurl,
				type: 'POST',
				dataType: 'json',
				data: {
					action: 'adaptive-sites-import-gcforms',
					gcforms_url: gcforms_url,
					_ajax_nonce: adaptiveSitesVars._ajax_nonce,
				},
				beforeSend: function () {
					console.groupCollapsed('正在导入表单');
					AdaptiveSitesAdmin._log_title('正在导入表单..');
				},
			})
				.fail(function (jqXHR) {
					AdaptiveSitesAdmin._log(jqXHR);
					AdaptiveSitesAdmin._failed( jqXHR.status + ' ' + jqXHR.statusText, '导入表单失败' );
					console.groupEnd();
				})
				.done(function (response) {
					AdaptiveSitesAdmin._log(response);
					console.groupEnd();

					// 1. Fail - Import GCForms Options.
					if (false === response.success) {
						AdaptiveSitesAdmin._failed( response.data, '导入表单失败' );
					} else {
						if (callback && typeof callback == "function") {
							callback(response);
						}
					}
				});
		},

		process_site_data: function (data) {

			if ('log_file' in data) {
				AdaptiveSitesAdmin.log_file_url = decodeURIComponent(data.log_file) || '';
			}

			// 1. Pass - Request Site Import
			AdaptiveSitesAdmin.customizer_data = JSON.stringify(data['adaptive-site-customizer-data']) || '';
			AdaptiveSitesAdmin.wxr_url = encodeURI(data['adaptive-site-wxr-path']) || '';
			AdaptiveSitesAdmin.gcforms_url = encodeURI(data['adaptive-site-gcforms-path']) || '';
			AdaptiveSitesAdmin.cartflows_url = encodeURI(data['adaptive-site-cartflows-path']) || '';
			AdaptiveSitesAdmin.options_data = JSON.stringify(data['adaptive-site-options-data']) || '';
			AdaptiveSitesAdmin.enabled_extensions = JSON.stringify(data['adaptive-enabled-extensions']) || '';
			AdaptiveSitesAdmin.widgets_data = data['adaptive-site-widgets-data'] || '';

			// Elementor Template Kit Markup.
			AdaptiveSitesAdmin.template_kit_markup(data);

			// Required Plugins.
			AdaptiveSitesAdmin.required_plugins_list_markup(data['required-plugins']);
			// Required Themes.
			AdaptiveSitesAdmin.required_themes_list_markup(data['required-themes']);
		},

		template_kit_markup: function (data) {
			if ('elementor' != adaptiveSitesVars.default_page_builder) {
				return;
			}
		},

		/**
		 * Enable Demo Import Button.
		 */
		_enable_demo_import_button: function (type) {

			type = (undefined !== type) ? type : 'free';

			$('.install-theme-info .theme-details .site-description').remove();

			switch (type) {

				case 'free':

					var notinstalled = [];
					var inactive = [];
					if( adaptiveSitesVars.requiredPlugins ) {
						notinstalled = adaptiveSitesVars.requiredPlugins.notinstalled || [];
						inactive = adaptiveSitesVars.requiredPlugins.inactive || [];
					}
					if ($('.adaptive-sites-result-preview').hasClass('skip-plugins')) {
						notinstalled = [];
					}

					if (notinstalled.length === inactive.length) {
						$(document).trigger('adaptive-sites-after-' + AdaptiveSitesAdmin.action_slug + '-required-plugins');
					}
					break;

				case 'upgrade':
					var demo_slug = $('.gc-full-overlay-header').attr('data-demo-slug');

					$('.adaptive-demo-import')
						.addClass('go-pro button-primary')
						.removeClass('adaptive-demo-import')
						.attr('target', '_blank')
						.attr('href', adaptiveSitesVars.getUpgradeURL + demo_slug)
						.text(adaptiveSitesVars.getUpgradeText)
						.append('<i class="dashicons dashicons-external"></i>');
					break;

				default:
					var demo_slug = $('.gc-full-overlay-header').attr('data-demo-slug');

					$('.adaptive-demo-import')
						.addClass('go-pro button-primary')
						.removeClass('adaptive-demo-import')
						.attr('target', '_blank')
						.attr('href', adaptiveSitesVars.getProURL)
						.text(adaptiveSitesVars.getProText)
						.append('<i class="dashicons dashicons-external"></i>');

					$('.gc-full-overlay-header').find('.go-pro').remove();

					if (false == adaptiveSitesVars.isWhiteLabeled) {
						if (adaptiveSitesVars.isPro) {
							$('.install-theme-info .theme-details').prepend(gc.template('adaptive-sites-pro-inactive-site-description'));
						} else {
							$('.install-theme-info .theme-details').prepend(gc.template('adaptive-sites-pro-site-description'));
						}
					}

					break;
			}

		},

		/**
		 * Update Page Count.
		 */

		/**
		 * Remove plugin from the queue.
		 */
		_removePluginFromQueue: function (removeItem, pluginsList) {
			return jQuery.grep(pluginsList, function (value) {
				return value.slug != removeItem;
			});
		},

		/**
		 * Update Page Count.
		 */

		/**
		 * Remove theme from the queue.
		 */
		_removeThemeFromQueue: function (removeItem, themesList) {
			return jQuery.grep(themesList, function (value) {
				return value.slug != removeItem;
			});
		}

	};

	/**
	 * Initialize AdaptiveSitesAdmin
	 */
	$(function () {
		AdaptiveSitesAdmin.init();
	});

})(jQuery);
