(function($){

	AdaptiveSitesInstallTheme = {

		/**
		 * Init
		 */
		init: function() {
			this._auto_close_notice();
			this._bind();
		},

		/**
		 * Binds events for the Adaptive Sites.
		 *
		 * @since 1.3.2
		 *
		 * @access private
		 * @method _bind
		 */
		_bind: function()
		{
			$( document ).on( 'click', '.adaptive-sites-theme-not-installed', AdaptiveSitesInstallTheme._install_and_activate );
			$( document ).on( 'click', '.adaptive-sites-theme-installed-but-inactive', AdaptiveSitesInstallTheme._activateTheme );
			$( document ).on( 'gc-theme-install-success' , AdaptiveSitesInstallTheme._activateTheme);
		},

		/**
		 * Close Getting Started Notice
		 *
		 * @param  {object} event
		 * @return void
		 */
		_auto_close_notice: function() {

			if( $( '.adaptive-sites-getting-started-btn' ).length ) {
				$.ajax({
					url: AdaptiveSitesInstallThemeVars.ajaxurl,
					type: 'POST',
					data: {
						'action' : 'adaptive-sites-getting-started-notice',
						'_ajax_nonce' : AdaptiveSitesInstallThemeVars._ajax_nonce,
					},
				})
				.done(function (result) {
				});
			}

		},

		/**
		 * Activate Theme
		 *
		 * @since 1.3.2
		 */
		_activateTheme: function( event, response ) {
			event.preventDefault();

			$('#adaptive-theme-activation-nag a').addClass('processing');

			if( response ) {
				$('#adaptive-theme-activation-nag a').text( AdaptiveSitesInstallThemeVars.installed );
			} else {
				$('#adaptive-theme-activation-nag a').text( AdaptiveSitesInstallThemeVars.activating );
			}

			// GeChiUI adds "Activate" button after waiting for 1000ms. So we will run our activation after that.
			setTimeout( function() {

				$.ajax({
					url: AdaptiveSitesInstallThemeVars.ajaxurl,
					type: 'POST',
					data: {
						'action' : 'adaptive-sites-activate-theme',
						'_ajax_nonce' : AdaptiveSitesInstallThemeVars._ajax_nonce,
					},
				})
				.done(function (result) {
					if( result.success ) {
						$('.adaptive-sites-theme-action-link').parent().html( AdaptiveSitesInstallThemeVars.activated + ' ?' );
					}

				});

			}, 3000 );

		},

		/**
		 * Install and activate
		 *
		 * @since 1.3.2
		 *
		 * @param  {object} event Current event.
		 * @return void
		 */
		_install_and_activate: function(event ) {
			event.preventDefault();
			var theme_slug = $(this).data('theme-slug') || '';
			var btn = $( event.target );

			if ( btn.hasClass( 'processing' ) ) {
				return;
			}

			btn.text( AdaptiveSitesInstallThemeVars.installing ).addClass('processing');

			if ( gc.updates.shouldRequestFilesystemCredentials && ! gc.updates.ajaxLocked ) {
				gc.updates.requestFilesystemCredentials( event );
			}

			gc.updates.installTheme( {
				slug: theme_slug
			});
		}

	};

	/**
	 * Initialize
	 */
	$(function(){
		AdaptiveSitesInstallTheme.init();
	});

})(jQuery);
