(function($){

	AdaptiveRender = {

		_ref			: null,

		/**
		 * _api_params = {
		 * 		'search'                  : '',
		 * 		'per_page'                : '',
		 * 		'adaptive-site-category'     : '',
		 * 		'adaptive-site-page-builder' : '',
		 * 		'page'                    : '',
		 *   };
		 *
		 * E.g. per_page=<page-id>&adaptive-site-category=<category-ids>&adaptive-site-page-builder=<page-builder-ids>&page=<page>
		 */
		_api_params		: {},
		_breakpoint		: 768,
		_has_default_page_builder : false,
		_first_time_loaded : true,
	
		init: function()
		{
			this._resetPagedCount();
			this._bind();
			this._loadPageBuilders();
		},

		/**
		 * Binds events for the Adaptive Sites.
		 *
		 * @since 1.0.0
		 * @access private
		 * @method _bind
		 */
		_bind: function()
		{
			$( document ).on('adaptive-sites-api-request-error'   , AdaptiveRender._addSuggestionBox );
			$( document ).on('adaptive-sites-api-request-fail'    , AdaptiveRender._addSuggestionBox );
			$( document ).on('adaptive-api-post-loaded-on-scroll' , AdaptiveRender._reinitGridScrolled );
			$( document ).on('adaptive-api-post-loaded'           , AdaptiveRender._reinitGrid );
			$( document ).on('adaptive-api-page-builder-loaded'       , AdaptiveRender._addPageBuilders );
			$( document ).on('adaptive-api-category-loaded'   			, AdaptiveRender._loadFirstGrid );
			
			// Event's for API request.
			$( document ).on('click'                           , '.filter-links a', AdaptiveRender._filterClick );
			$( document ).on('keyup input'                     , '#gc-filter-search-input', AdaptiveRender._search );
			$( document ).on('scroll'                          , AdaptiveRender._scroll );
			$( document ).on('adaptive-sites-api-request-fail', AdaptiveRender._site_unreachable );
		},

		/**
		 * Website is Down
		 *
		 * @since 1.2.11
		 * @return null
		 */
		_site_unreachable: function( event, jqXHR, textStatus, args ) {
			event.preventDefault();
			if ( 'adaptive-site-page-builder' === args.id ) {
				$('#adaptive-sites-admin').html( gc.template('adaptive-site-down') )
			}
		},

		/**
		 * On Filter Clicked
		 *
		 * Prepare Before API Request:
		 * - Empty search input field to avoid search term on filter click.
		 * - Remove Inline Height
		 * - Added 'hide-me' class to hide the 'No more sites!' string.
		 * - Added 'loading-content' for body.
		 * - Show spinner.
		 */
		_filterClick: function( event ) {

			event.preventDefault();

			if( $( this ).parents('.adaptive-site-category').length && ! $('body').hasClass('page-builder-selected') ) {
				return;
			}

			$(this).parents('.filter-links').find('a').removeClass('current');
			$(this).addClass('current');

			// Prepare Before Search.
			$('.no-more-demos').addClass('hide-me');
			$('.adaptive-sites-suggestions').remove();

			// Empty the search input only click on category filter not on page builder filter.
			if( $(this).parents('.filter-links').hasClass('adaptive-site-category') ) {
				$('#gc-filter-search-input').val('');
			}
			$('#adaptive-sites').hide().css('height', '');

			$('body').addClass('loading-content');
			$('#adaptive-sites-admin').find('.spinner').removeClass('hide-me');

	        // Show sites.
			AdaptiveRender._showSites();
		},

		/**
		 * Search Site.
		 *
		 * Prepare Before API Request:
		 * - Remove Inline Height
		 * - Added 'hide-me' class to hide the 'No more sites!' string.
		 * - Added 'loading-content' for body.
		 * - Show spinner.
		 */
		_search: function() {

			if( ! $('body').hasClass('page-builder-selected') ) {
				return;
			}

			$this = jQuery('#gc-filter-search-input').val();

			// Prepare Before Search.
			$('#adaptive-sites').hide().css('height', '');			
			$('.no-more-demos').addClass('hide-me');
			$('.adaptive-sites-suggestions').remove();

			$('body').addClass('loading-content');
			$('#adaptive-sites-admin').find('.spinner').removeClass('hide-me');

			window.clearTimeout(AdaptiveRender._ref);
			AdaptiveRender._ref = window.setTimeout(function () {
				AdaptiveRender._ref = null;

				AdaptiveRender._resetPagedCount();
				jQuery('body').addClass('loading-content');
				jQuery('body').attr('data-adaptive-demo-search', $this);

				AdaptiveRender._showSites();

			}, 500);

		},

		/**
		 * On Scroll
		 */
		_scroll: function(event) {

			if( ! $('body').hasClass('page-builder-selected') ) {
				return;
			}

			if( ! $('body').hasClass('listed-all-sites') ) {

				var scrollDistance = jQuery(window).scrollTop();

				var themesBottom = Math.abs(jQuery(window).height() - jQuery('#adaptive-sites').offset().top - jQuery('#adaptive-sites').height());
				themesBottom = themesBottom - 100;

				ajaxLoading = jQuery('body').data('scrolling');

				if (scrollDistance > themesBottom && ajaxLoading == false) {
					AdaptiveRender._updatedPagedCount();

					if( ! $('#adaptive-sites .no-themes').length ) {
						$('#adaptive-sites-admin').find('.spinner').addClass('is-active');
					}

					jQuery('body').data('scrolling', true);
					
					/**
					 * @see _reinitGridScrolled() which called in trigger 'adaptive-api-post-loaded-on-scroll'
					 */
					AdaptiveRender._showSites( false, 'adaptive-api-post-loaded-on-scroll' );
				}
			}
		},

		_apiAddParam_status: function() {
			if( adaptiveRenderGrid.sites && adaptiveRenderGrid.sites.status ) {
				AdaptiveRender._api_params['status'] = adaptiveRenderGrid.sites.status;
			}
		},

		// Add 'search'
		_apiAddParam_search: function() {
			var search_val = jQuery('#gc-filter-search-input').val() || '';
			if( '' !== search_val ) {
				AdaptiveRender._api_params['search'] = search_val;
			}
		},

		_apiAddParam_per_page: function() {
			// Add 'per_page'
			var per_page_val = 30;
			if( adaptiveRenderGrid.sites && adaptiveRenderGrid.sites["par-page"] ) {
				per_page_val = parseInt( adaptiveRenderGrid.sites["par-page"] );
			}
			AdaptiveRender._api_params['per_page'] = per_page_val;
		},

		_apiAddParam_adaptive_site_category: function() {
			// Add 'adaptive-site-category'
			var selected_category_id = jQuery('.filter-links.adaptive-site-category').find('.current').data('group') || '';
			if( '' !== selected_category_id && 'all' !== selected_category_id ) {
				AdaptiveRender._api_params['adaptive-site-category'] =  selected_category_id;
			} else if( adaptiveRenderGrid.sites && adaptiveRenderGrid['categories'].include ) {
				if( AdaptiveRender._isArray( adaptiveRenderGrid['categories'].include ) ) {
					AdaptiveRender._api_params['adaptive-site-category'] = adaptiveRenderGrid['categories'].include.join(',');
				} else {
					AdaptiveRender._api_params['adaptive-site-category'] = adaptiveRenderGrid['categories'].include;
				}
			}
		},

		_apiAddParam_siteground: function() {
			if( adaptiveRenderGrid['siteground'] ) {
				AdaptiveRender._api_params['siteground'] = adaptiveRenderGrid['siteground'];
			}
		},

		_apiAddParam_adaptive_site_page_builder: function() {
			// Add 'adaptive-site-page-builder'
			var selected_page_builder_id = jQuery('.filter-links.adaptive-site-page-builder').find('.current').data('group') || '';
			if( '' !== selected_page_builder_id && 'all' !== selected_page_builder_id ) {
				AdaptiveRender._api_params['adaptive-site-page-builder'] =  selected_page_builder_id;
			} else if( adaptiveRenderGrid.sites && adaptiveRenderGrid['page-builders'].include ) {
				if( AdaptiveRender._isArray( adaptiveRenderGrid['page-builders'].include ) ) {
					AdaptiveRender._api_params['adaptive-site-page-builder'] = adaptiveRenderGrid['page-builders'].include.join(',');
				} else {
					AdaptiveRender._api_params['adaptive-site-page-builder'] = adaptiveRenderGrid['page-builders'].include;
				}
			}
		},

		_apiAddParam_page: function() {
			// Add 'page'
			var page_val = parseInt(jQuery('body').attr('data-adaptive-demo-paged')) || 1;
			AdaptiveRender._api_params['page'] = page_val;
		},

		_apiAddParam_purchase_key: function() {
			if( adaptiveRenderGrid.sites && adaptiveRenderGrid.sites.purchase_key ) {
				AdaptiveRender._api_params['purchase_key'] = adaptiveRenderGrid.sites.purchase_key;
			}
		},

		_apiAddParam_site_url: function() {
			if( adaptiveRenderGrid.sites && adaptiveRenderGrid.sites.site_url ) {
				AdaptiveRender._api_params['site_url'] = adaptiveRenderGrid.sites.site_url;
			}
		},

		/**
		 * Show Sites
		 * 
		 * 	Params E.g. per_page=<page-id>&adaptive-site-category=<category-ids>&adaptive-site-page-builder=<page-builder-ids>&page=<page>
		 *
		 * @param  {Boolean} resetPagedCount Reset Paged Count.
		 * @param  {String}  trigger         Filtered Trigger.
		 */
		_showSites: function( resetPagedCount, trigger ) {

			if( undefined === resetPagedCount ) {
				resetPagedCount = true
			}

			if( undefined === trigger ) {
				trigger = 'adaptive-api-post-loaded';
			}

			if( resetPagedCount ) {
				AdaptiveRender._resetPagedCount();
			}

			// Add Params for API request.
			AdaptiveRender._api_params = {};

			AdaptiveRender._apiAddParam_status();
			AdaptiveRender._apiAddParam_search();
			AdaptiveRender._apiAddParam_per_page();
			AdaptiveRender._apiAddParam_adaptive_site_category();
			AdaptiveRender._apiAddParam_siteground();
			AdaptiveRender._apiAddParam_page();
			AdaptiveRender._apiAddParam_adaptive_site_page_builder();
			AdaptiveRender._apiAddParam_site_url();
			AdaptiveRender._apiAddParam_purchase_key();

			// API Request.
			var api_post = {
				id: 'adaptive-sites',
				slug: 'adaptive-sites?' + decodeURIComponent( $.param( AdaptiveRender._api_params ) ),
				trigger: trigger,
			};

			AdaptiveSitesAPI._api_request( api_post );

		},

		/**
		 * Get Category Params
		 *
		 * @since 1.2.4
		 * @param  {string} category_slug Category Slug.
		 * @return {mixed}               Add `include=<category-ids>` in API request.
		 */
		_getPageBuilderParams: function()
		{
			var _params = {};

			if( adaptiveRenderGrid.default_page_builder ) {
				_params['search'] = adaptiveRenderGrid.default_page_builder;
			}

			if( adaptiveRenderGrid.sites && adaptiveRenderGrid.sites.purchase_key ) {
				_params['purchase_key'] = adaptiveRenderGrid.sites.purchase_key;
			}

			if( adaptiveRenderGrid.sites && adaptiveRenderGrid.sites.site_url ) {
				_params['site_url'] = adaptiveRenderGrid.sites.site_url;
			}

			if( adaptiveRenderGrid.sites && adaptiveRenderGrid['page-builders'].include ) {
				if( AdaptiveRender._isArray( adaptiveRenderGrid['page-builders'].include ) ) {
					_params['include'] = adaptiveRenderGrid['page-builders'].include.join(',');
				} else {
					_params['include'] = adaptiveRenderGrid['page-builders'].include;
				}
			}

			var decoded_params = decodeURIComponent( $.param( _params ) );

			if( decoded_params.length ) {
				return '/?' + decoded_params;
			}

			return '/';
		},

		/**
		 * Get Category Params
		 * 
		 * @param  {string} category_slug Category Slug.
		 * @return {mixed}               Add `include=<category-ids>` in API request.
		 */
		_getCategoryParams: function( category_slug ) {

			var _params = {};

			if( adaptiveRenderGrid.sites && adaptiveRenderGrid['categories'].include ) {
				if( AdaptiveRender._isArray( adaptiveRenderGrid['categories'].include ) ) {
					_params['include'] = adaptiveRenderGrid['categories'].include.join(',');
				} else {
					_params['include'] = adaptiveRenderGrid['categories'].include;
				}
			}

			var decoded_params = decodeURIComponent( $.param( _params ) );

			if( decoded_params.length ) {
				return '/?' + decoded_params;
			}

			return '/';
		},

		/**
		 * Get All Select Status
		 * 
		 * @param  {string} category_slug Category Slug.
		 * @return {boolean}              Return true/false.
		 */
		_getCategoryAllSelectStatus: function( category_slug ) {	

			// Has category?
			if( category_slug in adaptiveRenderGrid.settings ) {

				// Has `all` in stored list?
				if( $.inArray('all', adaptiveRenderGrid.settings[ category_slug ]) === -1 ) {
					return false;
				}
			}

			return true;
		},

		/**
		 * Show Filters
		 */
		_loadPageBuilders: function() {

			// Is Welcome screen?
			// Then pre-send the API request to avoid the loader.
			if( $('.adaptive-sites-welcome').length ) {

				var plugins = $('.adaptive-sites-welcome').attr( 'data-plugins' ) || '';
				var plugins = plugins.split(",");

				// Also, Send page builder request with `/?search=` parameter. Because, We send the selected page builder request 
				// Which does not cached due to extra parameter `/?search=`. For that we initially send all these requests.
				$.each(plugins, function( key, plugin) {
					var category_slug = 'adaptive-site-page-builder';
					var category = {
						slug          : category_slug + '/?search=' + plugin,
						id            : category_slug,
						class         : category_slug,
						trigger       : '',
						wrapper_class : 'filter-links',
						show_all      : false,
					};

					// Pre-Send `sites` request for each active page builder to avoid the loader.
					AdaptiveSitesAPI._api_request( category, function( data ) {
						if( data.items ) {

							var per_page_val = 30;
							if( adaptiveRenderGrid.sites && adaptiveRenderGrid.sites["par-page"] ) {
								per_page_val = parseInt( adaptiveRenderGrid.sites["par-page"] );
							}
							
							var api_params = {
												per_page : per_page_val,
												page : 1,
											};
							// Load `all` sites from each page builder.
							$.each(data.items, function(index, item) {

								if( item.id ) {
									api_params['adaptive-site-page-builder'] =  item.id;

									// API Request.
									var api_post = {
										id: 'adaptive-sites',
										slug: 'adaptive-sites?' + decodeURIComponent( $.param( api_params ) ),
									};

									AdaptiveSitesAPI._api_request( api_post );
								}
							});
						}
					});

				} );

				// Pre-Send `category` request to avoid the loader.
				var category_slug = 'adaptive-site-category';
				var category = {
					slug          : category_slug + '/',
					id            : category_slug,
					class         : category_slug,
					trigger       : '',
					wrapper_class : 'filter-links',
					show_all      : false,
				};
				AdaptiveSitesAPI._api_request( category );

			// Load `sites` from selected page builder.
			} else {
				var category_slug = 'adaptive-site-page-builder';
				var category = {
					slug          : category_slug + AdaptiveRender._getPageBuilderParams(),
					id            : category_slug,
					class         : category_slug,
					trigger       : 'adaptive-api-page-builder-loaded',
					wrapper_class : 'filter-links',
					show_all      : false,
				};

				AdaptiveSitesAPI._api_request( category );
			}
		},

		/**
		 * Load First Grid.
		 *
		 * This is triggered after all category loaded.
		 * 
		 * @param  {object} event Event Object.
		 */
		_loadFirstGrid: function( event, data ) {
			
			event.preventDefault();

			if( $('#' + data.args.id).length ) {
				var template = gc.template('adaptive-site-filters');
				$('#' + data.args.id).html(template( data ));

				if( 'true' === $('body').attr( 'data-default-page-builder-selected' ) ) {
					$('#' + data.args.id).find('li:first a').addClass('current');
					AdaptiveRender._showSites();
				} else {
					$('body').removeClass('loading-content');
					if( ! $('#adaptive-sites-admin .adaptive-site-select-page-builder').length ) {
						$('#adaptive-sites-admin').append( gc.template( 'adaptive-site-select-page-builder' ) );
					}
				}
			} else {
				AdaptiveRender._showSites();
			}

		},

		/**
		 * Append filters.
		 * 
		 * @param  {object} event Object.
		 * @param  {object} data  API response data.
		 */
		_addPageBuilders: function( event, data ) {
			event.preventDefault();

			if( $('#' + data.args.id).length ) {
				var template = gc.template('adaptive-site-filters');
				$('#' + data.args.id).html(template( data ));

				if( 1 === parseInt( data.items_count ) ) {
					$('body').attr( 'data-default-page-builder-selected', true );
					$('#' + data.args.id).find('li:first a').addClass('current');
				}
			}

			/**
			 * Categories
			 */
			var category_slug = 'adaptive-site-category';
			var category = {
				slug          : category_slug + AdaptiveRender._getCategoryParams( category_slug ),
				id            : category_slug,
				class         : category_slug,
				trigger       : 'adaptive-api-category-loaded',
				wrapper_class : 'filter-links',
				show_all      : AdaptiveRender._getCategoryAllSelectStatus( category_slug ),
			};

			AdaptiveSitesAPI._api_request( category );

		},
		

		/**
		 * Append sites on scroll.
		 * 
		 * @param  {object} event Object.
		 * @param  {object} data  API response data.
		 */
		_reinitGridScrolled: function( event, data ) {

			var template = gc.template('adaptive-sites-list');

			if( data.items.length > 0 ) {

				$('body').removeClass( 'loading-content' );
				$('.filter-count .count').text( data.items_count );

				setTimeout(function() {
					jQuery('#adaptive-sites').append(template( data ));

					AdaptiveRender._imagesLoaded();
				}, 800);
			} else {
				$('body').addClass('listed-all-sites');
			}

		},

		/**
		 * Update Adaptive sites list.
		 * 
		 * @param  {object} event Object.
		 * @param  {object} data  API response data.
		 */
		_reinitGrid: function( event, data ) {

			var template = gc.template('adaptive-sites-list');

			$('body').addClass( 'page-builder-selected' );
			$('body').removeClass( 'loading-content' );
			$('.filter-count .count').text( data.items_count );

			jQuery('body').attr('data-adaptive-demo-last-request', data.items_count);

			jQuery('#adaptive-sites').show().html(template( data ));

			AdaptiveRender._imagesLoaded();

			$('#adaptive-sites-admin').find('.spinner').removeClass('is-active');

			if( data.items_count <= 0 ) {
				$('#adaptive-sites-admin').find('.spinner').removeClass('is-active');
				$('.no-more-demos').addClass('hide-me');
				$('.adaptive-sites-suggestions').remove();

			} else {
				$('body').removeClass('listed-all-sites');
			}

			// Re-Send `categories` sites request to avoid the loader.
			var categories = AdaptiveSitesAPI._stored_data['adaptive-site-category'];
			if( categories && AdaptiveRender._first_time_loaded ) {
				
				var per_page_val = 30;
				if( adaptiveRenderGrid.sites && adaptiveRenderGrid.sites["par-page"] ) {
					per_page_val = parseInt( adaptiveRenderGrid.sites["par-page"] );
				}
				
				var api_params = {
					per_page : per_page_val,
				};

				var page_builder_id = $('#adaptive-site-page-builder').find('.current').data('group') || '';

				$.each( categories, function( index, category ) {

					api_params['adaptive-site-category'] =  category.id;

					api_params['page'] = 1;

					if( page_builder_id ) {
						api_params['adaptive-site-page-builder'] = page_builder_id;
					}

					if( adaptiveRenderGrid.sites && adaptiveRenderGrid.sites.site_url ) {
						api_params['site_url'] = adaptiveRenderGrid.sites.site_url;
					}
					if( adaptiveRenderGrid.sites && adaptiveRenderGrid.sites.purchase_key ) {
						api_params['purchase_key'] = adaptiveRenderGrid.sites.purchase_key;
					}

					// API Request.
					var api_post = {
						id: 'adaptive-sites',
						slug: 'adaptive-sites?' + decodeURIComponent( $.param( api_params ) ),
					};

					AdaptiveSitesAPI._api_request( api_post );
				} );

				AdaptiveRender._first_time_loaded = false;
			}

		},

		/**
		 * Check image loaded with function `imagesLoaded()`
		 */
		_imagesLoaded: function() {

			var self = jQuery('#sites-filter.execute-only-one-time a');
			
			$('.adaptive-sites-grid').imagesLoaded()
			.always( function( instance ) {
				if( jQuery( window ).outerWidth() > AdaptiveRender._breakpoint ) {
					// $('#adaptive-sites').masonry('reload');
				}

				$('#adaptive-sites-admin').find('.spinner').removeClass('is-active');
			})
			.progress( function( instance, image ) {
				var result = image.isLoaded ? 'loaded' : 'broken';
			});

		},

		/**
		 * Add Suggestion Box
		 */
		_addSuggestionBox: function() {
			$('#adaptive-sites-admin').find('.spinner').removeClass('is-active').addClass('hide-me');

			$('#adaptive-sites-admin').find('.no-more-demos').removeClass('hide-me');
			var template = gc.template('adaptive-sites-suggestions');
			if( ! $( '.adaptive-sites-suggestions').length ) {
				$('#adaptive-sites').append( template );
			}
		},

		/**
		 * Update Page Count.
		 */
		_updatedPagedCount: function() {
			paged = parseInt(jQuery('body').attr('data-adaptive-demo-paged'));
			jQuery('body').attr('data-adaptive-demo-paged', paged + 1);
			window.setTimeout(function () {
				jQuery('body').data('scrolling', false);
			}, 800);
		},

		/**
		 * Reset Page Count.
		 */
		_resetPagedCount: function() {

			jQuery('body').attr('data-adaptive-demo-last-request', '1');
			jQuery('body').attr('data-adaptive-demo-paged', '1');
			jQuery('body').attr('data-adaptive-demo-search', '');
			jQuery('body').attr('data-scrolling', false);

		},

		// Returns if a value is an array
		_isArray: function(value) {
			return value && typeof value === 'object' && value.constructor === Array;
		}

	};

	/**
	 * Initialize AdaptiveRender
	 */
	$(function(){
		AdaptiveRender.init();
	});

})(jQuery);