<?php
/**
 * Functions
 *
 * @since  2.0.0
 * @package Adaptive Sites
 */

if ( ! function_exists( 'adaptive_sites_error_log' ) ) :

	/**
	 * Error Log
	 *
	 * A wrapper function for the error_log() function.
	 *
	 * @since 2.0.0
	 *
	 * @param  mixed $message Error message.
	 * @return void
	 */
	function adaptive_sites_error_log( $message = '' ) {
		if ( defined( 'GC_DEBUG_LOG' ) && GC_DEBUG_LOG ) {
			if ( is_array( $message ) ) {
				$message = gc_json_encode( $message );
			}

			if ( apply_filters( 'adaptive_sites_debug_logs', false ) ) {
				error_log( $message ); // phpcs:ignore GeChiUI.PHP.DevelopmentFunctions.error_log_error_log
			}
		}
	}

endif;

if ( ! function_exists( 'adaptive_sites_get_suggestion_link' ) ) :
	/**
	 *
	 * Get suggestion link.
	 *
	 * @since 2.6.1
	 *
	 * @return suggestion link.
	 */
	function adaptive_sites_get_suggestion_link() {
		$white_label_link = Adaptive_Sites_White_Label::get_option( 'adaptive-agency', 'licence' );

		if ( empty( $white_label_link ) ) {
			$white_label_link = 'https://www.gechiui.com/shop/pro/adaptive-sites/';
		}
		return apply_filters( 'adaptive_sites_suggestion_link', $white_label_link );
	}
endif;

if ( ! function_exists( 'adaptive_sites_is_valid_image' ) ) :
	/**
	 * Check for the valid image
	 *
	 * @param string $link  The Image link.
	 *
	 * @since 2.6.2
	 * @return boolean
	 */
	function adaptive_sites_is_valid_image( $link = '' ) {
		return preg_match( '/^((https?:\/\/)|(www\.))([a-z0-9-].?)+(:[0-9]+)?\/[\w\-]+\.(jpg|png|gif|jpeg|svg)\/?$/i', $link );
	}
endif;

if ( ! function_exists( 'adaptive_get_site_data' ) ) :
	/**
	 * Returns the value of the index for the Site Data
	 *
	 * @param string $index  The index value of the data.
	 *
	 * @since 2.6.14
	 * @return mixed
	 */
	function adaptive_get_site_data( $index = '' ) {
		$demo_data = get_option( 'adaptive_sites_import_data', array() );
		if ( ! empty( $demo_data ) && isset( $demo_data[ $index ] ) ) {
			return $demo_data[ $index ];
		}
		return '';
	}
endif;
