<?php
/**
 * Single Page Background Process
 *
 * @package Adaptive Sites
 * @since 2.0.0
 */

if ( class_exists( 'GC_Background_Process' ) ) :

	/**
	 * Image Background Process
	 *
	 * @since 2.0.0
	 */
	class GC_Background_Process_Adaptive_Site_Importer extends GC_Background_Process {

		/**
		 * Image Process
		 *
		 * @var string
		 */
		protected $action = 'adaptive_site_importer';

		/**
		 * Task
		 *
		 * Override this method to perform any actions required on each
		 * queue item. Return the modified item for further processing
		 * in the next pass through. Or, return false to remove the
		 * item from the queue.
		 *
		 * @since 2.0.0
		 *
		 * @param object $object Queue item object.
		 * @return mixed
		 */
		protected function task( $object ) {

			$process = $object['instance'];
			$method  = $object['method'];

			if ( 'import_page_builders' === $method ) {
				adaptive_sites_error_log( '-------- Importing Page Builders --------' );
				update_site_option( 'adaptive-sites-batch-status-string', 'Importing Page Builders', 'no' );
				$process->import_page_builders();
			} elseif ( 'import_categories' === $method ) {
				adaptive_sites_error_log( '-------- Importing Tags --------' );
				update_site_option( 'adaptive-sites-batch-status-string', 'Importing Tags', 'no' );
				$process->import_categories();
			} elseif ( 'import_sites' === $method ) {
				adaptive_sites_error_log( '-------- Importing Sites --------' );
				$page = $object['page'];
				adaptive_sites_error_log( 'Inside Batch ' . $page );
				update_site_option( 'adaptive-sites-batch-status-string', 'Inside Batch ' . $page, 'no' );
				$process->import_sites( $page );
			} elseif ( 'import_blocks' === $method ) {
				adaptive_sites_error_log( '-------- Importing Blocks --------' );
				$page = $object['page'];
				adaptive_sites_error_log( 'Inside Batch ' . $page );
				update_site_option( 'adaptive-sites-batch-status-string', 'Inside Batch ' . $page, 'no' );
				$process->import_blocks( $page );
			} elseif ( 'import_block_categories' === $method ) {
				adaptive_sites_error_log( '-------- Importing Blocks Categories --------' );
				update_site_option( 'adaptive-sites-batch-status-string', 'Importing Blocks Categories', 'no' );
				$process->import_block_categories();
			} elseif ( 'import_site_categories' === $method ) {
				adaptive_sites_error_log( '-------- Importing Site Categories --------' );
				update_site_option( 'adaptive-sites-batch-status-string', 'Importing Site Categories', 'no' );
				$process->import_site_categories();
			}

			return false;
		}

		/**
		 * Complete
		 *
		 * Override if applicable, but ensure that the below actions are
		 * performed, or, call parent::complete().
		 *
		 * @since 2.0.0
		 */
		protected function complete() {
			parent::complete();

			adaptive_sites_error_log( esc_html__( '所有流程都已完成', 'adaptive-sites' ) );
			update_site_option( 'adaptive-sites-batch-status-string', '所有流程都已完成', 'no' );
			delete_site_option( 'adaptive-sites-batch-status' );
			update_site_option( 'adaptive-sites-batch-is-complete', 'yes', 'no' );

			do_action( 'adaptive_sites_site_import_batch_complete' );
		}

	}

endif;
