<?php
/**
 * Image Background Process
 *
 * @package Adaptive Sites
 * @since 1.0.11
 */

if ( class_exists( 'GC_Background_Process' ) ) :

	/**
	 * Image Background Process
	 *
	 * @since 1.0.11
	 */
	class GC_Background_Process_Adaptive extends GC_Background_Process {

		/**
		 * Image Process
		 *
		 * @var string
		 */
		protected $action = 'image_process';

		/**
		 * Task
		 *
		 * Override this method to perform any actions required on each
		 * queue item. Return the modified item for further processing
		 * in the next pass through. Or, return false to remove the
		 * item from the queue.
		 *
		 * @since 1.0.11
		 *
		 * @param object $process Queue item object.
		 * @return mixed
		 */
		protected function task( $process ) {

			if ( method_exists( $process, 'import' ) ) {
				$process->import();
			}

			return false;
		}

		/**
		 * Complete
		 *
		 * Override if applicable, but ensure that the below actions are
		 * performed, or, call parent::complete().
		 *
		 * @since 1.0.11
		 */
		protected function complete() {

			parent::complete();

			Adaptive_Sites_Importer_Log::add( 'Batch Process Complete!' );

			// Delete Log file.
			delete_option( 'adaptive_sites_recent_import_log_file' );

			do_action( 'adaptive_sites_image_import_complete' );
		}

	}

endif;
