<?php
/**
 * Customizer Data importer class.
 *
 * @since  1.0.0
 * @package Adaptive Addon
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Customizer Data importer class.
 *
 * @since  1.0.0
 */
class Adaptive_Customizer_Import {

	/**
	 * Instance of Adaptive_Customizer_Import
	 *
	 * @since  1.0.0
	 * @var Adaptive_Customizer_Import
	 */
	private static $instance = null;

	/**
	 * Instantiate Adaptive_Customizer_Import
	 *
	 * @since  1.0.0
	 * @return (Object) Adaptive_Customizer_Import
	 */
	public static function instance() {

		if ( ! isset( self::$instance ) ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	/**
	 * Import customizer options.
	 *
	 * @since  1.0.0
	 *
	 * @param  (Array) $options customizer options from the demo.
	 */
	public function import( $options ) {

		// Update Adaptive Theme customizer settings.
		if ( isset( $options['adaptive-settings'] ) ) {
			self::import_settings( $options['adaptive-settings'] );
		}

		// Add Custom CSS.
		if ( isset( $options['custom-css'] ) ) {
			gc_update_custom_css_post( $options['custom-css'] );
		}

	}

	/**
	 * Import Adaptive Setting's
	 *
	 * Download & Import images from Adaptive Customizer Settings.
	 *
	 * @since 1.0.10
	 *
	 * @param  array $options Adaptive Customizer setting array.
	 * @return void
	 */
	public static function import_settings( $options = array() ) {

		array_walk_recursive(
			$options,
			function ( &$value ) {
				if ( ! is_array( $value ) && adaptive_sites_is_valid_image( $value ) ) {
					$downloaded_image = Adaptive_Sites_Image_Importer::get_instance()->import(
						array(
							'url' => $value,
							'id'  => 0,
						)
					);
					$value            = $downloaded_image['url'];
				}
			}
		);

		// Updated settings.
		update_option( 'adaptive-settings', $options );
	}
}
