<?php
/**
 * Shortcode Markup
 *
 * TMPL - Single Demo Preview
 * TMPL - No more demos
 * TMPL - Filters
 * TMPL - List
 *
 * @package Adaptive Sites
 * @since 1.0.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

$subscription_status = Adaptive_Sites::get_instance()->should_display_subscription_form();
$subscription_class  = true === $subscription_status ? 'subscription-enabled' : '';

$site_import_options = apply_filters(
	'adaptive_sites_site_import_options',
	array(
		'activate-theme' => true,
		'reset'          => false,
		'customizer'     => true,
		'widgets'        => true,
		'themes'        => true,
		'plugins'        => true,
		'xml'            => true,
	)
);

?>

<div class="wrap" id="adaptive-sites-admin" data-slug="<?php echo esc_html( $global_cpt_meta['cpt_slug'] ); ?>">

	<?php
	if ( ! empty( $_GET['debug'] ) && 'yes' === $_GET['debug'] ) { // phpcs:ignore GeChiUI.Security.NonceVerification.Recommended
		?>
		<div class="adaptive-sites-log">
			<?php Adaptive_Sites_Importer_Log::get_instance()->display_data(); ?>
		</div>

	<?php } else { ?>

		<?php do_action( 'adaptive_sites_before_site_grid' ); ?>

		<div class="theme-browser rendered">
			<div id="adaptive-sites" class="themes gc-clearfix"></div>
			<div id="site-pages" class="themes gc-clearfix"></div>
			<div class="adaptive-sites-result-preview" style="display: none;"></div>

			<div class="adaptive-sites-popup" style="display: none;">
				<div class="overlay"></div>
				<div class="inner">
					<div class="heading">
						<h3><?php esc_html_e( '标题', 'adaptive-sites' ); ?></h3>
						<span class="dashicons close dashicons-no-alt"></span>
					</div>
					<div class="adaptive-sites-import-content"></div>
					<div class="ast-actioms-wrap"></div>
				</div>
			</div>
		</div>

		<?php do_action( 'adaptive_sites_after_site_grid' ); ?>

	<?php } ?>
</div>

<script type="text/template" id="tmpl-adaptive-sites-compatibility-messages">

	<div class="skip-and-import">
		<div class="heading">
			<h3><?php esc_html_e( '我们都在！', 'adaptive-sites' ); ?></h3>
			<span class="dashicons close dashicons-no-alt"></span>
		</div>
		<div class="adaptive-sites-import-content">

			<p><?php esc_html_e( '您即将导入模板。要完成此过程，请清除以下条件。', 'adaptive-sites' ); ?></p>

			<ul class="adaptive-site-contents">

				<# for ( code in data ) { #>
					<# if( Object.keys( data[ code ] ).length ) { #>

						<# for ( id in data[ code ] ) { #>
							<li>
								{{{ data[ code ][id].title }}}

								<# if ( data[ code ][id].tooltip ) { #>
									<span class="adaptive-sites-tooltip-icon" data-tip-id="adaptive-sites-skip-and-import-notice-{{id}}">
										<span class="dashicons dashicons-editor-help"></span>
									</span>
									<div class="adaptive-sites-tooltip-message" id="adaptive-sites-skip-and-import-notice-{{id}}" style="display: none;">
										{{{data[ code ][id].tooltip}}}
									</div>
								<# } #>
							</li>
						<# } #>

					<# } #>
				<# } #>

			</ul>

		</div>
		<div class="ast-actioms-wrap">
			<# if( Object.keys( data['errors'] ).length ) { #>
				<a href="#" class="button button-hero button-primary adaptive-demo-import disabled site-install-site-button"><?php esc_html_e( '跳过&导入', 'adaptive-sites' ); ?></a>
				<div class="button button-hero site-import-cancel"><?php esc_html_e( '取消', 'adaptive-sites' ); ?></div>
			<# } else {
				var plugin_update = data['warnings']['update-available'] || 0;
				if( plugin_update ) { #>
					<a href="<?php echo esc_url( network_admin_url( 'update-core.php' ) ); ?>" class="button button-hero button-primary" target="_blank"><?php esc_html_e( '更新', 'adaptive-sites' ); ?></a>
					<a href="#" class="button button-hero button-primary adaptive-sites-skip-and-import-step"><?php esc_html_e( '跳过&导入', 'adaptive-sites' ); ?></a>
				<# } else { #>
					<a href="#" class="button button-hero button-primary adaptive-sites-skip-and-import-step"><?php esc_html_e( '跳过&导入', 'adaptive-sites' ); ?></a>
					<div class="button button-hero site-import-cancel"><?php esc_html_e( '取消', 'adaptive-sites' ); ?></div>
				<# } #>
			<# } #>
		</div>
	</div>

</script>

<script type="text/template" id="tmpl-adaptive-sites-no-sites">
	<div class="adaptive-sites-no-sites">
		<div class="inner">
			<h3><?php esc_html_e( '抱歉没有找到。', 'adaptive-sites' ); ?></h3>
			<div class="content">
				<div class="empty-item">
					<img class="empty-collection-part" src="<?php echo esc_url( ADAPTIVE_SITES_URI . 'inc/assets/images/empty-collection.svg' ); ?>" alt="empty-collection">
				</div>
				<div class="description">
					<p>
					<?php
					/* translators: %1$s External Link */
					printf( __( 'Don\'t see a template you would like to import?<br><a target="_blank" href="%1$s">Make a Template Suggestion!</a>', 'adaptive-sites' ), esc_url( adaptive_sites_get_suggestion_link() ) ); // phpcs:ignore GeChiUI.Security.EscapeOutput.OutputNotEscaped
					?>
					</p>
					<div class="back-to-layout-button"><span class="button adaptive-sites-back"><?php esc_html_e( '回到模板', 'adaptive-sites' ); ?></span></div>
				</div>
			</div>
		</div>
	</div>
</script>

<script type="text/template" id="tmpl-adaptive-sites-no-favorites">
	<div class="adaptive-sites-no-favorites">
		<div class="inner">
			<h3><?php esc_html_e( '收藏模板列表为空。', 'adaptive-sites' ); ?></h3>
			<div class="content">
				<div class="empty-item">
					<img class="empty-collection-part" src="<?php echo esc_url( ADAPTIVE_SITES_URI . 'inc/assets/images/empty-collection.svg' ); ?>" alt="empty-collection">
				</div>
				<div class="description">
					<p>
					<?php
					/* translators: %1$s External Link */
					esc_html_e( '你会注意到每个模板卡上都有一个心形的符号。只需点击这个图标将模板标记为收藏。', 'adaptive-sites' );
					?>
					</p>
					<img src="<?php echo esc_url( ADAPTIVE_SITES_URI . 'inc/assets/images/arrow-blue.svg' ); ?>" class="arrow-img">
					<div class="back-to-layout-button"><span class="button adaptive-sites-back"><?php esc_html_e( '回到模板', 'adaptive-sites' ); ?></span></div>
				</div>
			</div>
		</div>
	</div>
</script>
<?php
/**
 * TMPL - Show Page Builder Sites
 */
?>
<script type="text/template" id="tmpl-adaptive-sites-page-builder-sites">
	<# for ( site_id in data ) { #>
	<#
		var current_site_id     = site_id;
		var type                = data[site_id]['type'] || 'site';
		var wrapper_class       = data[site_id]['class'] || '';
		var page_site_id        = data[site_id]['site_id'] || '';
		var favorite_status     = false;
		var favorite_class      = '';
		var favorite_title      = '<?php esc_html_e( '装作喜欢的', 'adaptive-sites' ); ?>';
		var featured_image_url = data[site_id]['featured-image-url'];
		var thumbnail_image_url = data[site_id]['thumbnail-image-url'] || featured_image_url;

		var site_type = data[site_id]['adaptive-sites-type'] || '';
		var page_id = '';
		if ( 'site' === type ) {
			if( Object.values( adaptiveSitesVars.favorite_data ).indexOf( String(site_id) ) >= 0 ) {
				favorite_class = 'is-favorite';
				favorite_status = true;
				favorite_title = '<?php esc_html_e( '装作不喜欢的', 'adaptive-sites' ); ?>';
			}
		} else {
			thumbnail_image_url = featured_image_url;
			current_site_id = page_site_id;
			page_id = site_id;
		}

		var title = data[site_id]['title'] || '';
		var pages_count = parseInt( data[site_id]['pages-count'] ) || 0;
		var pages_count_class = '';
		var pages_count_string = ( pages_count !== 1 ) ? pages_count + ' Templates' : pages_count + ' Template';
		if( 'site' === type ) {
			if( pages_count ) {
				pages_count_class = 'has-pages';
			} else {
				pages_count_class = 'no-pages';
			}
		}
		var site_title = data[site_id]['site-title'] || '';

	#>
	<div class="theme adaptive-theme site-single {{favorite_class}} {{pages_count_class}} adaptive-sites-previewing-{{type}} {{wrapper_class}}" data-site-id="{{current_site_id}}" data-page-id="{{page_id}}">
		<div class="inner">
			<span class="site-preview" data-title="{{{title}}}">
				<div class="theme-screenshot one loading" data-src="{{thumbnail_image_url}}" data-featured-src="{{featured_image_url}}"></div>
			</span>
			<div class="theme-id-container">
				<div class="theme-name">
					<span class="title">
						<# if ( 'site' === type ) { #>
							<div class='site-title'>{{{title}}}</div>
							<# if ( pages_count ) { #>
								<div class='pages-count'>{{{pages_count_string}}}</div>
							<# } #>
						<# } else { #>
							<div class='site-title'>{{{site_title}}}</div>
							<div class='page-title'>{{{title}}}</div>
						<# } #>
					</span>
				</div>
				<# if ( '' === type || 'site' === type ) { #>
					<div class="favorite-action-wrap" data-favorite="{{favorite_class}}" title="{{favorite_title}}">
						<i class="ast-icon-heart"></i>
					</div>
				<# } #>
			</div>
			<# if ( site_type && 'free' !== site_type ) { #>
				<?php /* translators: %s are white label strings. */ ?>
				<div class="agency-ribbons" title="<?php printf( esc_attr__( '此高级模板可使用%1$s"高级"封装。', 'adaptive-sites' ), Adaptive_Sites_White_Label::get_instance()->get_white_label_name() ); // phpcs:ignore GeChiUI.Security.EscapeOutput.OutputNotEscaped ?>"><img class="premium-crown-icon" src="<?php echo esc_url( ADAPTIVE_SITES_URI . 'inc/assets/images/premium-crown.svg' ); ?>" alt="premium-crown"><?php esc_html_e( '高级', 'adaptive-sites' ); ?></div>
			<# } #>
		</div>
	</div>
	<# } #>

</script>

<?php
/**
 * TMPL - Show Page Builder Sites
 */
?>
<script type="text/template" id="tmpl-adaptive-sites-page-builder-sites-search">
	<# var pages_list = []; #>
	<# var sites_list = []; #>
	<# var pages_list_arr = []; #>
	<# var sites_list_arr = []; #>
	<# for ( site_id in data ) {
		var type = data[site_id]['type'] || 'site';
		if ( 'site' === type ) {
			sites_list_arr.push( data[site_id] );
			sites_list[site_id] = data[site_id];
		} else {
			pages_list_arr.push( data[site_id] );
			pages_list[site_id] = data[site_id]
		}
	} #>
	<# if ( sites_list_arr.length > 0 ) { #>
		<h3 class="ast-sites__search-title"><?php esc_html_e( '站点模板', 'adaptive-sites' ); ?></h3>
		<div class="ast-sites__search-wrap">
		<# for ( site_id in sites_list ) { #>
		<#
			var current_site_id     = site_id;
			var type                = sites_list[site_id]['type'] || 'site';
			var page_site_id        = sites_list[site_id]['site_id'] || '';
			var favorite_status     = false;
			var favorite_class      = '';
			var favorite_title      = '<?php esc_html_e( '装作喜欢的', 'adaptive-sites' ); ?>';
			var featured_image_url = sites_list[site_id]['featured-image-url'];
			var thumbnail_image_url = sites_list[site_id]['thumbnail-image-url'] || featured_image_url;

			var site_type = sites_list[site_id]['adaptive-sites-type'] || '';
			var page_id = '';
			if( Object.values( adaptiveSitesVars.favorite_data ).indexOf( String(site_id) ) >= 0 ) {
				favorite_class = 'is-favorite';
				favorite_status = true;
				favorite_title = '<?php esc_html_e( '装作不喜欢的', 'adaptive-sites' ); ?>';
			}

			var title = sites_list[site_id]['title'] || '';
			var pages_count = parseInt( sites_list[site_id]['pages-count'] ) || 0;
			var pages_count_string = ( pages_count !== 1 ) ? pages_count + ' Templates' : pages_count + ' Template';
			var pages_count_class = '';
			if( pages_count ) {
				pages_count_class = 'has-pages';
			} else {
				pages_count_class = 'no-pages';
			}
			var site_title = sites_list[site_id]['site-title'] || '';

		#>
			<div class="theme adaptive-theme site-single {{favorite_class}} {{pages_count_class}} adaptive-sites-previewing-{{type}}" data-site-id="{{current_site_id}}" data-page-id="{{page_id}}">
				<div class="inner">
					<span class="site-preview" data-title="{{{title}}}">
						<div class="theme-screenshot one loading" data-src="{{thumbnail_image_url}}" data-featured-src="{{featured_image_url}}"></div>
					</span>
					<div class="theme-id-container">
						<div class="theme-name">
							<span class="title">
								<# if ( 'site' === type ) { #>
									<div class='site-title'>{{{title}}}</div>
									<# if ( pages_count ) { #>
										<div class='pages-count'>{{{pages_count_string}}}</div>
									<# } #>
								<# } else { #>
									<div class='site-title'>{{{site_title}}}</div>
									<div class='page-title'>{{{title}}}</div>
								<# } #>
							</span>
						</div>
						<# if ( '' === type || 'site' === type ) { #>
							<div class="favorite-action-wrap" data-favorite="{{favorite_class}}" title="{{favorite_title}}">
								<i class="ast-icon-heart"></i>
							</div>
						<# } #>
					</div>
					<# if ( site_type && 'free' !== site_type ) { #>
						<?php /* translators: %s are white label strings. */ ?>
						<div class="agency-ribbons" title="<?php printf( esc_attr__( '此高级模板可使用%1$s"高级"封装。', 'adaptive-sites' ), Adaptive_Sites_White_Label::get_instance()->get_white_label_name() ); // phpcs:ignore GeChiUI.Security.EscapeOutput.OutputNotEscaped ?>"><?php esc_html_e( '高级', 'adaptive-sites' ); ?></div>
					<# } #>
				</div>
			</div>
		<# } #>
		</div>
	<# } #>
	<# if ( pages_list_arr.length > 0 ) { #>

		<h3 class="ast-sites__search-title"><?php esc_html_e( '页面模板', 'adaptive-sites' ); ?></h3>
		<div class="ast-sites__search-wrap">
		<# for ( site_id in pages_list ) { #>
		<#
			var current_site_id     = site_id;
			var type                = pages_list[site_id]['type'] || 'site';
			var page_site_id        = pages_list[site_id]['site_id'] || '';
			var favorite_status     = false;
			var favorite_class      = '';
			var favorite_title      = '<?php esc_html_e( '装作喜欢的', 'adaptive-sites' ); ?>';
			var featured_image_url = pages_list[site_id]['featured-image-url'];
			var thumbnail_image_url = pages_list[site_id]['thumbnail-image-url'] || featured_image_url;

			var site_type = pages_list[site_id]['adaptive-sites-type'] || '';
			var page_id = '';
			thumbnail_image_url = featured_image_url;
			current_site_id = page_site_id;
			page_id = site_id;

			var title = pages_list[site_id]['title'] || '';
			var pages_count = pages_list[site_id]['pages-count'] || 0;
			var pages_count_class = '';
			if( 'site' === type ) {
				if( pages_count ) {
					pages_count_class = 'has-pages';
				} else {
					pages_count_class = 'no-pages';
				}
			}
			var site_title = pages_list[site_id]['site-title'] || '';

		#>
			<div class="theme adaptive-theme site-single {{favorite_class}} {{pages_count_class}} adaptive-sites-previewing-{{type}}" data-site-id="{{current_site_id}}" data-page-id="{{page_id}}">
				<div class="inner">
					<span class="site-preview" data-title="{{{title}}}">
						<div class="theme-screenshot one loading" data-src="{{thumbnail_image_url}}" data-featured-src="{{featured_image_url}}"></div>
					</span>
					<div class="theme-id-container">
						<div class="theme-name">
							<span class="title">
								<div class='site-title'>{{{site_title}}}</div>
								<div class='page-title'>{{{title}}}</div>
							</span>
						</div>
						<# if ( '' === type || 'site' === type ) { #>
							<div class="favorite-action-wrap" data-favorite="{{favorite_class}}" title="{{favorite_title}}">
								<i class="ast-icon-heart"></i>
							</div>
						<# } #>
					</div>
					<# if ( site_type && 'free' !== site_type ) { #>
						<?php /* translators: %s are white label strings. */ ?>
						<div class="agency-ribbons" title="<?php printf( esc_attr__( '此高级模板可使用%1$s"高级"封装。', 'adaptive-sites' ), Adaptive_Sites_White_Label::get_instance()->get_white_label_name() ); // phpcs:ignore GeChiUI.Security.EscapeOutput.OutputNotEscaped ?>"><?php esc_html_e( '高级', 'adaptive-sites' ); ?></div>
					<# } #>
				</div>
			</div>
		<# } #>
		</div>
	<# } #>

</script>

<?php
/**
 * TMPL - Pro Site Description
 */
?>
<script type="text/template" id="tmpl-adaptive-sites-pro-site-description">
	<p>
		<?php
			_e( '<span class="highlighted-note">这是一个高级模板，可与基本和增长捆绑包。</span><br /><a href="{{AdaptiveSitesAdmin.premium_popup_cta_link}}" target="_blank">访问</a>此高级模板。', 'adaptive-sites' ); // phpcs:ignore GeChiUI.Security.EscapeOutput.UnsafePrintingFunction
		?>
	</p>
	<p>
		<?php
			/* translators: %s is article link */
			printf( __( '已经拥有了基本捆绑包或增长捆绑包？ <a href="%s" target="_blank">知道如何</a> 导入高级模板。', 'adaptive-sites' ), esc_url( 'https://docs.gechiui.com/adaptive/starter-templates-complete-site/' ) ); // phpcs:ignore GeChiUI.Security.EscapeOutput.OutputNotEscaped
		?>
	</p>
</script>

<?php
/**
 * TMPL - Activate License
 */
?>
<script type="text/template" id="tmpl-adaptive-sites-skip-templates">
	<p><?php esc_html_e( '包含动态小部件/模块的页面模板不能用于单个模板导入。通过“导入站点”选项，您可以获得这些页面。', 'adaptive-sites' ); ?></p>
	<p><?php esc_html_e( '您可以从左下角按钮有完整的站点预览。', 'adaptive-sites' ); ?></p>
</script>

<?php
/**
 * TMPL - Activate License
 */
?>
<script type="text/template" id="tmpl-adaptive-sites-activate-license">
	<?php
		do_action( 'adaptive_sites_activate_license_popup' );
	?>
</script>

<?php
/**
 * TMPL - Invalid Essential Bundle License
 */
?>
<script type="text/template" id="tmpl-adaptive-sites-invalid-mini-agency-license">
	<p>
		<?php
			$page_builder = get_option( 'adaptive-sites-license-page-builder', '' );
		if ( 'elementor' === $page_builder ) {
			$current_page_builder = 'Elementor';
			$upgrade_page_builder = 'Beaver Builder';
		} else {
			$current_page_builder = 'Beaver Builder';
			$upgrade_page_builder = 'Elementor';
		}

		/* translators: %s is pricing page link */
		printf( __( '<p>Seems like you have purchased the %4$s \'Essential Bundle\' package with a choice of \'%1$s\' page builder addon.</p><p>While this template is built with \'%2$s\' page builder addon.</p><p>Read article <a href="%3$s" target="_blank">here</a> to see how you can proceed.</p>', 'adaptive-sites' ), esc_html( $current_page_builder ), esc_html( $upgrade_page_builder ), esc_url( Adaptive_Sites_White_Label::get_instance()->get_white_label_link( 'https://docs.gechiui.com/adaptive/not-valid-license/' ) ), Adaptive_Sites_White_Label::get_instance()->get_white_label_name() ); // phpcs:ignore GeChiUI.Security.EscapeOutput.OutputNotEscaped
		?>
	</p>
</script>

<?php
/**
 * TMPL - Pro Site Description for Inactive license
 */
?>
<script type="text/template" id="tmpl-adaptive-sites-pro-inactive-site-description">
	<p><?php esc_html_e( '你只需要2分钟就可以导入这个演示!', 'adaptive-sites' ); ?></p>
	<p><?php esc_html_e( '这是一个高级网站的演示模板。请激活您的许可证来访问它。', 'adaptive-sites' ); ?></p>
	<p>
		<?php
			/* translators: %s is article link */
			printf( __( '了解如何 <a href="%1$s" target="_blank">激活</a>%2$s插件的许可证</a>。', 'adaptive-sites' ), esc_url( 'https://docs.gechiui.com/adaptive/activate-license-for-adaptive-premium-sites-plugin/' ), Adaptive_Sites_White_Label::get_instance()->get_white_label_name() ); // phpcs:ignore GeChiUI.Security.EscapeOutput.OutputNotEscaped
		?>
	</p>
</script>

<?php
/**
 * TMPL - Third Party Required Plugins
 */
?>
<script type="text/template" id="tmpl-adaptive-sites-third-party-required-plugins">
	<div class="skip-and-import">
		<div class="heading">
			<h3><?php esc_html_e( '缺少需要的插件', 'adaptive-sites' ); ?></h3>
			<span class="dashicons close dashicons-no-alt"></span>
		</div>
		<div class="adaptive-sites-import-content">
			<p><?php esc_html_e( '这个启动器模板需要高级插件。因为这些是第三方的高级插件，首先你需要购买，安装和激活他们。', 'adaptive-sites' ); ?></p>
			<ul class="adaptive-sites-third-party-required-plugins">
				<# for ( key in data ) { #>
					<li class="plugin-card plugin-card-{{data[ key ].slug}}'" data-slug="{{data[ key ].slug }}" data-init="{{data[ key ].init}}" data-name="{{data[ key ].name}}"><a href="{{data[ key ].link}}" target="_blank">{{data[ key ].name}}</a></li>
				<# } #>
			</ul>
		</div>
		<div class="ast-actioms-wrap">
			<a href="#" class="button button-hero button-primary adaptive-sites-skip-and-import-step"><?php esc_html_e( '跳过&导入', 'adaptive-sites' ); ?></a>
			<div class="button button-hero site-import-cancel"><?php esc_html_e( '取消', 'adaptive-sites' ); ?></div>
		</div>
	</div>
</script>

<?php
/**
 * TMPL - Single Site Preview
 */
?>
<script type="text/template" id="tmpl-adaptive-sites-single-site-preview">
	<div class="single-site-wrap">
		<div class="single-site">
			<div class="single-site-preview-wrap">
				<div class="single-site-pages-header">
					<h3 class="adaptive-site-title">{{{data['title']}}}</h3>
					<span class="count" style="display: none"></span>
				</div>
				<div class="single-site-preview">
					<img class="theme-screenshot" data-src="" src="{{data['featured-image-url']}}" />
				</div>
			</div>
			<div class="single-site-pages-wrap">
				<div class="adaptive-pages-title-wrap">
					<span class="adaptive-pages-title"><?php esc_html_e( '页面模板', 'adaptive-sites' ); ?></span>
				</div>
				<div class="single-site-pages">
					<div id="single-pages">
						<# for ( page_id in data.pages ) {
							var dynamic_page = data.pages[page_id]['dynamic-page'] || 'no'; #>
							<div class="theme adaptive-theme site-single" data-page-id="{{page_id}}" data-dynamic-page="{{dynamic_page}}" >
								<div class="inner">
									<#
									var featured_image_class = '';
									var featured_image = data.pages[page_id]['featured-image-url'] || '';
									if( '' === featured_image ) {
										featured_image = '<?php echo esc_url( ADAPTIVE_SITES_URI . 'inc/assets/images/placeholder.png' ); ?>';
										featured_image_class = ' no-featured-image ';
									}

									var thumbnail_image = data.pages[page_id]['thumbnail-image-url'] || '';
									if( '' === thumbnail_image ) {
										thumbnail_image = featured_image;
									}
									#>
									<span class="site-preview" data-title="{{ data.pages[page_id]['title'] }}">
										<div class="theme-screenshot one loading {{ featured_image_class }}" data-src="{{ thumbnail_image }}" data-featured-src="{{ featured_image }}"></div>
									</span>
									<div class="theme-id-container">
										<h3 class="theme-name">
											{{{ data.pages[page_id]['title'] }}}
										</h3>
									</div>
								</div>
							</div>
						<# } #>
					</div>
				</div>
			</div>
			<div class="single-site-footer">
				<div class="site-action-buttons-wrap">
					<?php $white_label_class = ( Adaptive_Sites_White_Label::get_instance()->get_white_label_name() !== ADAPTIVE_SITES_NAME ) ? 'ast-white-label-flag' : ''; ?>
					<a href="{{data['adaptive-site-url']}}/" class="button button-hero site-preview-button <?php echo esc_html( $white_label_class ); ?>" target="_blank"><?php esc_html_e( '演示站点', 'adaptive-sites' ); ?> <i class="dashicons dashicons-external"></i></a>
					<div class="site-action-buttons-right">
						<# if( 'free' !== data['adaptive-sites-type'] && '' !== adaptiveSitesVars.license_page_builder && data['adaptive-site-page-builder'] !== adaptiveSitesVars.license_page_builder && ( 'brizy' !== data['adaptive-site-page-builder'] && 'gutenberg' !== data['adaptive-site-page-builder']  ) ) { #>
							<a class="button button-hero button-primary disabled adaptive-sites-invalid-mini-agency-license-button"><?php esc_html_e( '没有有效的许可证', 'adaptive-sites' ); ?></a>
							<span class="dashicons dashicons-editor-help adaptive-sites-invalid-mini-agency-license-button"></span>
						<# } else if( 'free' !== data['adaptive-sites-type'] && ! adaptiveSitesVars.license_status ) { #>
							<# if( ! adaptiveSitesVars.isPro ) { #>
								<a class="button button-hero button-primary adaptive-sites-get-agency-bundle-button" target="_blank">{{adaptiveSitesVars.getProText}}<i class="dashicons dashicons-external"></i></a>
							<# } else { #>
								<span class="button button-hero button-primary adaptive-sites-activate-license-button">{{adaptiveSitesVars.getProText}}</span>
							<# } #>
						<# } else { #>
							<div class="button button-hero button-primary site-import-site-button"><?php esc_html_e( '导入完整站点', 'adaptive-sites' ); ?></div>
							<div style="margin-left: 5px;" class="button button-hero button-primary site-import-layout-button disabled"><?php esc_html_e( '导入模板', 'adaptive-sites' ); ?></div>
						<# } #>
					</div>
				</div>
			</div>
		</div>
	</div>
</script>

<script type="text/template" id="tmpl-adaptive-sites-site-import-success">
	<div class="heading">
		<h3><?php esc_html_e( '导入成功！', 'adaptive-sites' ); ?></h3>
		<span class="dashicons close dashicons-no-alt"></span>
	</div>
	<div class="adaptive-sites-import-content">
		<p><b><?php esc_html_e( '网站导入成功！', 'adaptive-sites' ); ?></b></p>
		<p>
			<?php esc_html_e( '继续，自定义文本，图像和设计，使它成为你的!', 'adaptive-sites' ); ?>&nbsp;
			<# if ( '46177' == AdaptiveSitesAdmin.templateData.id ) { #>
			<?php
				$kit_doc_url = 'https://docs.gechiui.com/adaptive/mountain-template-elementor-theme-style/';
				/* translators: %1$s External Link */
				printf( __( '关于自定义这个元素或风格的工具包站点，请%1$s参阅更多的%2$s。', 'adaptive-sites' ), '<a href="' . esc_url( $kit_doc_url ) . '" target="_blank">', '</a>' ); // phpcs:ignore GeChiUI.Security.EscapeOutput.OutputNotEscaped
			?>
			<# } #>
		</p>
		<p><?php esc_html_e( '玩的开心！', 'adaptive-sites' ); ?></p>
		<p><?php esc_html_e( 'PS:我们尽量使用从法律角度来说是免费的图片。但是，我们不承担任何责任。我们强烈建议网站所有者在网上发布这些图片和任何受版权保护的媒体之前更换它们。', 'adaptive-sites' ); ?></p>
	</div>
	<div class="ast-actioms-wrap">
		<a class="button button-primary button-hero" href="<?php echo esc_url( site_url() ); ?>" target="_blank"><?php esc_html_e( '查看站点', 'adaptive-sites' ); ?> <i class="dashicons dashicons-external"></i></a>
	</div>
</script>

<script type="text/template" id="tmpl-adaptive-sites-page-import-success">
	<div class="heading">
		<h3><?php esc_html_e( '导入成功！', 'adaptive-sites' ); ?></h3>
		<span class="dashicons close dashicons-no-alt"></span>
	</div>
	<div class="adaptive-sites-import-content">
		<p><b><?php esc_html_e( '模板导入成功!', 'adaptive-sites' ); ?></b></p>
		<p><?php esc_html_e( '继续，自定义文本，图像和设计，使它成为你的!', 'adaptive-sites' ); ?></p>
		<p><?php esc_html_e( '玩的开心！', 'adaptive-sites' ); ?></p>
		<p><?php esc_html_e( 'PS:我们尽量使用从法律角度来说是免费的图片。但是，我们不承担任何责任。我们强烈建议网站所有者在网上发布这些图片和任何受版权保护的媒体之前更换它们。', 'adaptive-sites' ); ?></p>
	</div>
	<div class="ast-actioms-wrap">
		<a class="button button-primary button-hero" href="{{data['link']}}" target="_blank"><?php esc_html_e( '查看模板', 'adaptive-sites' ); ?> <i class="dashicons dashicons-external"></i></a>
	</div>
</script>

<?php
/**
 * TMPL - Import Process Interrupted
 */
?>
<script type="text/template" id="tmpl-adaptive-sites-request-failed">
	<p><?php esc_html_e( '我们不能启动导入过程。这是来自 HTTP 请求的消息：', 'adaptive-sites' ); ?></p>
	<p>
	<?php
		/* translators: %s doc link. */
		printf( __( '请提出 <a href="%s" target="_blank">支持请求</a> ，以便我们帮助您。', 'adaptive-sites' ), 'https://www.gechiui.com/shop/pro/adaptive-sites/' ); // phpcs:ignore GeChiUI.Security.EscapeOutput.OutputNotEscaped
	?>
	</p>
</script>
<?php
/**
 * TMPL - Import Process Interrupted - User end
 */
?>
<script type="text/template" id="tmpl-adaptive-sites-request-failed-user">
	<p>{{{ data.primary }}}</p>
	<# if ( 'Cloudflare' === data.error.code ) { #>
		<div class="current-importing-status">{{{ data.error.message + ' (' + data.error.code + ')' }}}</div>
	<# } else { #>
		<div class="current-importing-status">{{{ data.error.code }}} - {{{ data.error.message }}}</div>
	<# } #>
	<# if ( 'GC_Error' === data.error.code ) { #>
	<p>
		<?php
		/* translators: %s doc link. */
		printf( __( '我们在此列出了 <a href="%s" target="_blank">可能的解决方案</a> ，以帮助您解决这个问题。', 'adaptive-sites' ), 'https://docs.gechiui.com/adaptive/fix-starter-template-importing-issues/' ); // phpcs:ignore GeChiUI.Security.EscapeOutput.OutputNotEscaped
		?>
	</p>
	<# } else if ( 'Cloudflare' === data.error.code ) { #>
	<p>
		<# if ( '522' == data.error.response_code ) { #>
			<?php
			/* translators: %s doc link. */
			printf( __( '请 <a class="ast-try-again" href="">点击这里，再试一次</a>。如果这仍然不工作后，几次尝试，请 <a href="%s" target="_blank">报告在这里</a>。', 'adaptive-sites' ), 'https://www.gechiui.com/shop/pro/adaptive-sites/' ); // phpcs:ignore GeChiUI.Security.EscapeOutput.OutputNotEscaped
			?>
		<# } else { #>
			<?php
			/* translators: %s doc link. */
			printf( __( '请 <a href="%s" target="_blank">在此</a> 处报告此错误，以便我们可以修复它。', 'adaptive-sites' ), 'https://www.gechiui.com/shop/pro/adaptive-sites/' ); // phpcs:ignore GeChiUI.Security.EscapeOutput.OutputNotEscaped
			?>
		<# } #>
	</p>
	<# } else { #>
	<p>
		<?php
		$ip       = Adaptive_Sites_Helper::get_client_ip();
		$url_text = __( '请 <a href="#LINK" target="_blank">在此</a> 处报告此错误，以便我们可以修复它。', 'adaptive-sites' );
		$url      = 'https://www.gechiui.com/shop/pro/adaptive-sites/?ip=' . $ip;
		?>
		<#
		var url = '<?php echo $url; // phpcs:ignore GeChiUI.Security.EscapeOutput.OutputNotEscaped ?>';
		url += '&template-id=' + data.id;
		url += '&subject=' + data.error.code + ' - ' + data.error.message;

		var url_text = '<?php echo $url_text; // phpcs:ignore GeChiUI.Security.EscapeOutput.OutputNotEscaped ?>';
		url_text = url_text.replace( "#LINK", url );
		#>
		{{{url_text}}}
	</p>
	<# } #>
</script>

<?php
/**
 * TMPL - Dynamic Page
 */
?>
<script type="text/template" id="tmpl-adaptive-sites-dynamic-page">
	<div class="skip-and-import">
		<div class="heading">
			<h3><?php esc_html_e( '警告', 'adaptive-sites' ); ?></h3>
			<span class="dashicons close dashicons-no-alt"></span>
		</div>
		<div class="adaptive-sites-import-content">
			<p><?php esc_html_e( '您将要导入的页面模板包含一个动态小工具/模块。请注意，导入的页面无法使用此动态数据。', 'adaptive-sites' ); ?></p>
			<p><?php esc_html_e( '您需要在页面上手动添加它。', 'adaptive-sites' ); ?></p>
			<p><?php esc_html_e( '当您导入整个站点时，该动态内容将可用。', 'adaptive-sites' ); ?></p>
		</div>
		<div class="ast-actioms-wrap">
			<a href="#" class="button button-hero button-primary adaptive-sites-skip-and-import-step"><?php esc_html_e( '跳过&导入', 'adaptive-sites' ); ?></a>
			<div class="button button-hero site-import-cancel"><?php esc_html_e( '取消', 'adaptive-sites' ); ?></div>
		</div>
	</div>
</script>

<script type="text/template" id="tmpl-adaptive-sites-subscription-form-one">
<div class="subscription-fields">
	<div class="subscription-field-wrap">
		<select class="subscription-input subscription-input-gc-user-type" name="gc_user_type">
			<option value=""></option>
			<option value="1"><?php esc_html_e( '初学者', 'adaptive-sites' ); ?></option>
			<option value="2"><?php esc_html_e( '中间', 'adaptive-sites' ); ?></option>
			<option value="3"><?php esc_html_e( '专家级', 'adaptive-sites' ); ?></option>
		</select>
		<small class="subscription-desc"><?php esc_html_e( '字段是必需的', 'adaptive-sites' ); ?></small>
		<label class="subscription-label"><?php esc_html_e( '我是GeChiUI：', 'adaptive-sites' ); ?></label>
	</div>
	<div class="subscription-field-wrap">
		<select class="subscription-input subscription-input-build-website-for" name="build_website_for">
			<option value=""></option>
			<option value="1"><?php esc_html_e( '我/我的公司', 'adaptive-sites' ); ?></option>
			<option value="2"><?php esc_html_e( '我的客户', 'adaptive-sites' ); ?></option>
		</select>
		<small class="subscription-desc"><?php esc_html_e( '字段是必需的', 'adaptive-sites' ); ?></small>
		<label class="subscription-label"><?php esc_html_e( '我正在建立以下网站：', 'adaptive-sites' ); ?></label>
	</div>
</div>
</script>

<script type="text/template" id="tmpl-adaptive-sites-subscription-form-two">
	<div class="subscription-fields">
		<div class="subscription-field-wrap">
			<input class="subscription-input subscription-input-name" type="text" name="first_name" />
			<small class="subscription-desc"><?php esc_html_e( '名字是必填的。', 'adaptive-sites' ); ?></small>
			<label class="subscription-label"><?php esc_html_e( '请输入您的名字。', 'adaptive-sites' ); ?></label>
		</div>
		<div class="subscription-field-wrap">
			<input class="subscription-input subscription-input-email" type="email" name="email" />
			<small class="subscription-desc"><?php esc_html_e( '请填写电子邮件地址', 'adaptive-sites' ); ?></small>
			<label class="subscription-label"><?php esc_html_e( '您的工作电子邮件', 'adaptive-sites' ); ?></label>
		</div>
	</div>
</script>

<?php
/**
 * TMPL - First Screen
 */
?>
<script type="text/template" id="tmpl-adaptive-sites-result-preview">

	<div class="overlay"></div>
	<div class="inner <?php echo esc_attr( $subscription_class ); ?>">

		<div class="subscription-popup">
			<div class="heading">
				<h3><?php esc_html_e( '最后一步..', 'adaptive-sites' ); ?></h3>
				<span class="dashicons close dashicons-no-alt"></span>
			</div>
			<div class="adaptive-sites-import-content">
				<p><?php esc_html_e( '要访问独家入门模板，主题和更新，请在下面输入您的详细信息：', 'adaptive-sites' ); ?></p>
				<div id="adaptive-sites-subscription-form-two" class="subscription-form adaptive-sites-subscription-form-two"></div>
				<div class="subscription-actions">
					<button class="button button-hero button-primary button-subscription-submit">
						<span class="button-title"><?php esc_html_e( '提交并开始导入', 'adaptive-sites' ); ?></span>
						<span class="dashicons dashicons-update"></span>
					</button>
				</div>
				<div class="subscription-footer">
					<?php /* translators: %1$s and %3$s are opening anchor tags, and %2$s and %4$s is closing anchor tags. */ ?>
					<p><?php printf( __( '提交即表示您同意我们的%1$s条款%2$s和%3$s隐私权政策%4$s。', 'adaptive-sites' ), '<a href="https://www.gechiui.com/terms-and-conditions/" target="_blank">', '</a>', '<a href="https://www.gechiui.com/privacy-policy/" target="_blank">', '</a>' ); // phpcs:ignore GeChiUI.Security.EscapeOutput.OutputNotEscaped ?></p>
					<a href="#" class="button-subscription-skip"><?php esc_html_e( '跳過', 'adaptive-sites' ); ?></a>
				</div>
			</div>
		</div>

		<div class="default">
			<div class="heading">
				<# if( 'adaptive-sites' === data ) { #>
					<h3><?php esc_html_e( '您选择的网站正在被导入。', 'adaptive-sites' ); ?></h3>
				<# } else { #>
					<h3><?php esc_html_e( '您选择的模板将被导入。', 'adaptive-sites' ); ?></h3>
				<# } #>
				<span class="dashicons close dashicons-no-alt"></span>
			</div>

			<div class="adaptive-sites-import-content">
				<div class="install-theme-info">
					<div class="adaptive-sites-advanced-options-wrap">

						<?php if ( true === $subscription_status ) : ?>
							<p class="user-building-for-title"><?php esc_html_e( '为了提供更精美的入门模板，我们想进一步了解您：', 'adaptive-sites' ); ?></p>
						<?php endif; ?>

						<div id="adaptive-sites-subscription-form-one" class="subscription-form adaptive-sites-subscription-form-one"></div>

						<h2 class="adaptive-sites-advanced-options-heading"><?php esc_html_e( '高级选项', 'adaptive-sites' ); ?> <span class="dashicons dashicons-arrow-right-alt2"></span></h2>

						<div class="adaptive-sites-advanced-options">
							<ul class="adaptive-site-contents">

								<li class="adaptive-sites-import-themes">
									<input type="checkbox" name="themes" class="checkbox"  <?php checked( $site_import_options['themes'], true ); ?> />
									<strong><?php esc_html_e( '安装并激活所需主题', 'adaptive-sites' ); ?></strong>
									<span class="adaptive-sites-tooltip-icon" data-tip-id="adaptive-sites-tooltip-themes-settings"><span class="dashicons dashicons-editor-help"></span></span>
									<div class="adaptive-sites-tooltip-message" id="adaptive-sites-tooltip-themes-settings" style="display: none;">
										<p><?php esc_html_e( '要以原始格式导入站点，需要激活所需主题。您可以将它与任何其他主题一起导入，但是站点可能会丢失一些设计设置，看起来有点不同。', 'adaptive-sites' ); ?></p>
										<ul class="required-themes-list"><span class="spinner is-active"></span></ul>
									</div>
								</li>

								<# if( 'adaptive-sites' === data ) { #>

									<li class="adaptive-sites-import-customizer">
										<label>
											<input type="checkbox" name="customizer" class="checkbox" <?php checked( $site_import_options['customizer'], true ); ?> />
											<strong><?php esc_html_e( '导入编辑器设置', 'adaptive-sites' ); ?></strong>
											<span class="adaptive-sites-tooltip-icon" data-tip-id="adaptive-sites-tooltip-customizer-settings"><span class="dashicons dashicons-editor-help"></span></span>
											<div class="adaptive-sites-tooltip-message" id="adaptive-sites-tooltip-customizer-settings" style="display: none;">
											<?php /* translators: %s are white label strings. */ ?>
											<p><?php printf( esc_html__( '%1$s 定制服务的全局设置，给予网站统一的设计。选择此选项将覆盖当前的自定义器设置。', 'adaptive-sites' ), Adaptive_Sites_White_Label::get_instance()->get_white_label_name() ); // phpcs:ignore GeChiUI.Security.EscapeOutput.OutputNotEscaped ?></p>
											</div>
										</label>
									</li>

								<# } #>

								<# if( 'adaptive-sites' === data ) { #>
									<li class="adaptive-sites-import-widgets">
										<label>
											<input type="checkbox" name="widgets" class="checkbox" <?php checked( $site_import_options['widgets'], true ); ?> />
											<strong><?php esc_html_e( '导入小工具', 'adaptive-sites' ); ?></strong>
										</label>
									</li>
								<# } #>

								<li class="adaptive-sites-import-plugins">
									<input type="checkbox" name="plugins" class="disabled checkbox" readonly <?php checked( $site_import_options['plugins'], true ); ?> />
									<strong><?php esc_html_e( '需要安装插件', 'adaptive-sites' ); ?></strong>
									<span class="adaptive-sites-tooltip-icon" data-tip-id="adaptive-sites-tooltip-plugins-settings"><span class="dashicons dashicons-editor-help"></span></span>
									<div class="adaptive-sites-tooltip-message" id="adaptive-sites-tooltip-plugins-settings" style="display: none;">
										<p><?php esc_html_e( '缺少导入此模板所需的插件。所需的插件将被安装并自动激活。', 'adaptive-sites' ); ?></p>
										<ul class="required-plugins-list"><span class="spinner is-active"></span></ul>
									</div>
								</li>

								<# if( 'adaptive-sites' === data ) { #>
									<li class="adaptive-sites-import-xml">
										<label>
											<input type="checkbox" name="xml" class="checkbox" <?php checked( $site_import_options['xml'], true ); ?> />
											<strong><?php esc_html_e( '导入内容', 'adaptive-sites' ); ?></strong>
										</label>
										<span class="adaptive-sites-tooltip-icon" data-tip-id="adaptive-sites-tooltip-site-content"><span class="dashicons dashicons-editor-help"></span></span>
										<div class="adaptive-sites-tooltip-message" id="adaptive-sites-tooltip-site-content" style="display: none;"><p><?php esc_html_e( '选择此选项将导入虚拟页面、文章、图像和菜单。如果您不想导入虚拟内容，请取消勾选此选项。', 'adaptive-sites' ); ?></p></div>
									</li>
								<# } #>
							</ul>
						</div>
					</div>
				</div>
				<div class="ast-importing-wrap">
					<#
					if( 'adaptive-sites' === data ) {
						var string = 'sites';
					} else {
						var string = 'template';
					}
					#>
					<p>
					<?php
					/* translators: %s is the dynamic string. */
					printf( esc_html__( '导入过程可能会花费几分钟时间，具体取决于%s的大小和连接的速度。', 'adaptive-sites' ), '{{string}}' );
					?>
					</p>
					<p>
					<?php
					/* translators: %s is the dynamic string. */
					printf( esc_html__( '在完全导入%s之前，请不要关闭此浏览器窗口。', 'adaptive-sites' ), '{{string}}' );
					?>
					</p>

					<div class="current-importing-status-wrap">
						<div class="current-importing-status">
							<div class="current-importing-status-title"></div>
							<div class="current-importing-status-description"></div>
						</div>
					</div>
				</div>
			</div>

			<div class="ast-actioms-wrap">
				<a href="#" class="button button-hero button-primary adaptive-demo-import disabled site-install-site-button">
				<?php
				if ( true === $subscription_status ) {
					esc_html_e( '下一页', 'adaptive-sites' );
				} else {
					esc_html_e( '导入', 'adaptive-sites' );
				}
				?>
				</a>
				<a href="#" class="button button-hero button-primary adaptive-sites-skip-and-import" style="display: none;"><?php esc_html_e( '跳过&导入', 'adaptive-sites' ); ?></a>
				<div class="button button-hero site-import-cancel"><?php esc_html_e( '取消', 'adaptive-sites' ); ?></div>
			</div>
		</div>
	</div>
</script>

<script type="text/template" id="tmpl-adaptive-sites-delete-previous-site">
	<li class="adaptive-sites-reset-data">
		<label>
			<input type="checkbox" name="reset" class="checkbox" <?php checked( 'reset', true ); ?>>
			<strong><?php esc_html_e( '删除以前导入的站点', 'adaptive-sites' ); ?></strong>
			<div class="adaptive-sites-tooltip-message" id="adaptive-sites-tooltip-reset-data" style="display: none;"><p><?php esc_html_e( '警告:选择此选项将删除之前导入中的所有数据。只有在需要时才选择此选项。', 'adaptive-sites' ); ?></p>
			<?php if ( ! Adaptive_Sites_White_Label::get_instance()->is_white_labeled() ) { ?>
			<p><?php esc_html_e( '您可以在下面找到当前定制程序设置的备份', 'adaptive-sites' ); ?><code><?php esc_html_e( '/gc-content/uploads/adaptive-sites/', 'adaptive-sites' ); ?></code></p>
			<?php } ?>
			</div>
		</label>
	</li>

</script>
<?php
gc_print_admin_notice_templates();